@@ -27,6 +27,7 @@
 #include "SVNClient.h"
 #include "Revision.h"
 #include "Notify.h"
+#include "Notify2.h"
 #include "CommitMessage.h"
 #include "Prompter.h"
 #include "Targets.h"
@@ -105,12 +106,13 @@ JNIEXPORT jstring JNICALL Java_org_tigris_subversion_javahl_SVNClient_getLastPat
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    list
- * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;Z)
+ * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;Z)
  *            [Lorg/tigris/subversion/javahl/DirEntry;
  */
 JNIEXPORT jobjectArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_list
   (JNIEnv* env, jobject jthis, jstring jurl, jobject jrevision, 
-   jboolean jrecurse)
+   jobject jpegRevision, jboolean jrecurse)
 {
     JNIEntry(SVNClient, list);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -128,17 +130,22 @@ JNIEXPORT jobjectArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_list
     {
         return NULL;
     }
-    return cl->list(url, revision, jrecurse ? true:false);
-
+    Revision pegRevision(jpegRevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    return cl->list(url, revision, pegRevision, jrecurse ? true:false);
 }
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    status
- * Signature: (Ljava/lang/String;ZZZZ)[Lorg/tigris/subversion/javahl/Status;
+ * Signature: (Ljava/lang/String;ZZZZZ)[Lorg/tigris/subversion/javahl/Status;
  */
 JNIEXPORT jobjectArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_status
   (JNIEnv* env, jobject jthis, jstring jpath, jboolean jrecurse, 
-   jboolean jonServer, jboolean jgetAll, jboolean jnoIgnore)
+   jboolean jonServer, jboolean jgetAll, jboolean jnoIgnore, 
+   jboolean jignoreExternals)
 {
     JNIEntry(SVNClient, status);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -152,7 +159,8 @@ JNIEXPORT jobjectArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_statu
         return NULL;
     }
     return cl->status(path, jrecurse ? true: false, jonServer ? true:false, 
-                      jgetAll ? true:false, jnoIgnore ? true:false);
+                      jgetAll ? true:false, jnoIgnore ? true:false,
+                      jignoreExternals ? true:false);
 }
 
 /*
@@ -249,11 +257,14 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_setPrompt
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    logMessages
- * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;Lorg/tigris/subversion/javahl/Revision;ZZ)[Lorg/tigris/subversion/javahl/LogMessage;
+ * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;ZZJ)
+ *             [Lorg/tigris/subversion/javahl/LogMessage;
  */
 JNIEXPORT jobjectArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_logMessages
   (JNIEnv* env, jobject jthis, jstring jpath, jobject jrevisionStart,
-   jobject jrevisionEnd, jboolean jstopOnCopy, jboolean jdisoverPaths)
+   jobject jrevisionEnd, jboolean jstopOnCopy, jboolean jdisoverPaths,
+   jlong jlimit)
 {
     JNIEntry(SVNClient, logMessages);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -278,18 +289,20 @@ JNIEXPORT jobjectArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_logMe
         return NULL;
     }
     return cl->logMessages(path, revisionStart, revisionEnd,
-        jstopOnCopy ? true: false, jdisoverPaths ? true : false);
+        jstopOnCopy ? true: false, jdisoverPaths ? true : false, jlimit);
 }
 
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    checkout
  * Signature: (Ljava/lang/String;Ljava/lang/String;
- *             Lorg/tigris/subversion/javahl/Revision;Z)J
+ *             Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;ZZ)J
  */
 JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClient_checkout
   (JNIEnv* env, jobject jthis, jstring jmoduleName, jstring jdestPath, 
-   jobject jrevision, jboolean jrecurse)
+   jobject jrevision, jobject jpegRevision, jboolean jrecurse, 
+   jboolean jignoreExternals)
 {
     JNIEntry(SVNClient, checkout);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -303,6 +316,11 @@ JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClient_checkout
     {
         return -1;
     }
+    Revision pegRevision(jpegRevision, true);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return -1;
+    }
     JNIStringHolder moduleName(jmoduleName);
     if(JNIUtil::isExceptionThrown())
     {
@@ -313,7 +331,8 @@ JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClient_checkout
     {
         return -1;
     }
-    return cl->checkout(moduleName, destPath, revision, jrecurse ? true:false);
+    return cl->checkout(moduleName, destPath, revision, pegRevision, 
+        jrecurse ? true : false, jignoreExternals ? true : false);
 }
 
 /*
@@ -338,6 +357,28 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_notification
     }
     cl->notification(notify);
 }
+/*
+ * Class:     org_tigris_subversion_javahl_SVNClient
+ * Method:    notification2
+ * Signature: (Lorg/tigris/subversion/javahl/Notify2;)V
+ */
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_notification2
+  (JNIEnv* env, jobject jthis, jobject jnotify2)
+{
+    JNIEntry(SVNClient, notification2);
+    SVNClient *cl = SVNClient::getCppObject(jthis);
+    if(cl == NULL)
+    {
+        JNIUtil::throwError(_("bad c++ this"));
+        return;
+    }
+    Notify2 *notify2 = Notify2::makeCNotify(jnotify2);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    cl->notification2(notify2);
+}
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    commitMessageHandler
@@ -412,10 +453,11 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_revert
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    add
- * Signature: (Ljava/lang/String;Z)V
+ * Signature: (Ljava/lang/String;ZZ)V
  */
 JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_add
-  (JNIEnv* env, jobject jthis, jstring jpath, jboolean jrecurse)
+  (JNIEnv* env, jobject jthis, jstring jpath, jboolean jrecurse, 
+   jboolean jforce)
 {
     JNIEntry(SVNClient, add);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -429,46 +471,47 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_add
     {
         return;
     }
-    cl->add(path, jrecurse ? true : false);
+    cl->add(path, jrecurse ? true : false, jforce ? true : false);
 }
 
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    update
- * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;Z)J
+ * Signature: ([Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;ZZ)[J
  */
-JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClient_update
-  (JNIEnv* env, jobject jthis, jstring jpath, jobject jrevision, 
-   jboolean jrecurse)
+JNIEXPORT jlongArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_update
+  (JNIEnv* env, jobject jthis, jobjectArray jpath, jobject jrevision, 
+   jboolean jrecurse, jboolean jignoreExternals)
 {
     JNIEntry(SVNClient, update);
     SVNClient *cl = SVNClient::getCppObject(jthis);
     if(cl == NULL)
     {
         JNIUtil::throwError(_("bad c++ this"));
-        return -1;
+        return NULL;
     }
     Revision revision(jrevision);
     if(JNIUtil::isExceptionThrown())
     {
-        return -1;
+        return NULL;
     }
-    JNIStringHolder path(jpath);
+    Targets targets(jpath);
     if(JNIUtil::isExceptionThrown())
     {
-        return -1;
+        return NULL;
     }
-    return cl->update(path, revision, jrecurse ? true:false);
+    return cl->update(targets, revision, jrecurse ? true : false, 
+        jignoreExternals ? true : false);
 }
 
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    commit
- * Signature: ([Ljava/lang/String;Ljava/lang/String;Z)J
+ * Signature: ([Ljava/lang/String;Ljava/lang/String;ZZ)J
  */
 JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClient_commit
   (JNIEnv* env, jobject jthis, jobjectArray jtargets, jstring jmessage, 
-   jboolean jrecurse)
+   jboolean jrecurse, jboolean jnoUnlock)
 {
     JNIEntry(SVNClient, commit);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -483,7 +526,8 @@ JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClient_commit
     {
         return -1;
     }
-    return cl->commit(targets, message, jrecurse ? true : false);
+    return cl->commit(targets, message, jrecurse ? true : false, 
+        jnoUnlock ? true : false);
 }
 
 /*
@@ -529,12 +573,11 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_copy
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    move
- * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;
- *             Lorg/tigris/subversion/javahl/Revision;Z)V
+ * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Z)V
  */
 JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_move
   (JNIEnv *env, jobject jthis, jstring jsrcPath, jstring jdestPath, 
-   jstring jmessage, jobject jrevision, jboolean jforce)
+   jstring jmessage, jboolean jforce)
 {
     JNIEntry(SVNClient, move);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -543,11 +586,6 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_move
         JNIUtil::throwError(_("bad c++ this"));
         return;
     }
-    Revision revision(jrevision);
-    if(JNIUtil::isExceptionThrown())
-    {
-        return;
-    }
     JNIStringHolder srcPath(jsrcPath);
     if(JNIUtil::isExceptionThrown())
     {
@@ -563,7 +601,7 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_move
     {
         return;
     }
-    cl->move(srcPath, destPath, message, revision, jforce ? true:false);
+    cl->move(srcPath, destPath, message, jforce ? true:false);
 }
 
 /*
@@ -640,11 +678,13 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_resolved
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    doExport
  * Signature: (Ljava/lang/String;Ljava/lang/String;
- *             Lorg/tigris/subversion/javahl/Revision;)J
+ *             Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;ZZZLjava/lang/String;)J
  */
 JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClient_doExport
   (JNIEnv* env, jobject jthis, jstring jsrcPath, jstring jdestPath, 
-   jobject jrevision,jboolean jforce)
+   jobject jrevision, jobject jpegRevision, jboolean jforce, 
+   jboolean jignoreExternals, jboolean jrecurse, jstring jnativeEOL)
 {
     JNIEntry(SVNClient, doExport);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -658,6 +698,11 @@ JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClient_doExport
     {
         return -1;
     }
+    Revision pegRevision(jpegRevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return -1;
+    }
     JNIStringHolder srcPath(jsrcPath);
     if(JNIUtil::isExceptionThrown())
     {
@@ -668,7 +713,14 @@ JNIEXPORT jlong JNICALL Java_org_tigris_subversion_javahl_SVNClient_doExport
     {
         return -1;
     }
-    return cl->doExport(srcPath, destPath, revision, jforce ? true : false);
+    JNIStringHolder nativeEOL(jnativeEOL);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return -1;
+    }
+    return cl->doExport(srcPath, destPath, revision, pegRevision, 
+        jforce ? true : false, jignoreExternals ? true : false, 
+        jrecurse ? true: false, nativeEOL);
 }
 
 /*
@@ -746,12 +798,12 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_doImport
  * Method:    merge
  * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
  *             Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
- *             Ljava/lang/String;ZZ)V
+ *             Ljava/lang/String;ZZZZ)V
  */
-JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_merge
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_merge__Ljava_lang_String_2Lorg_tigris_subversion_javahl_Revision_2Ljava_lang_String_2Lorg_tigris_subversion_javahl_Revision_2Ljava_lang_String_2ZZZZ
   (JNIEnv* env, jobject jthis, jstring jpath1, jobject jrevision1, 
    jstring jpath2, jobject jrevision2, jstring jlocalPath, jboolean jforce, 
-   jboolean jrecurse)
+   jboolean jrecurse, jboolean jignoreAncestry, jboolean jdryRun)
 {
     JNIEntry(SVNClient, merge);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -786,16 +838,68 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_merge
         return;
     }
     cl->merge(path1, revision1, path2, revision2, localPath, 
-              jforce ? true:false, jrecurse ? true:false);
+        jforce ? true:false, jrecurse ? true:false, 
+        jignoreAncestry ? true:false, jdryRun ? true:false);
+}
+/*
+ * Class:     org_tigris_subversion_javahl_SVNClient
+ * Method:    merge
+ * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;Ljava/lang/String;ZZZZ)V
+ */
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_merge__Ljava_lang_String_2Lorg_tigris_subversion_javahl_Revision_2Lorg_tigris_subversion_javahl_Revision_2Lorg_tigris_subversion_javahl_Revision_2Ljava_lang_String_2ZZZZ
+  (JNIEnv* env, jobject jthis, jstring jpath, jobject jpegRevision, 
+   jobject jrevision1, jobject jrevision2, jstring jlocalPath, jboolean jforce,
+   jboolean jrecurse, jboolean jignoreAncestry, jboolean jdryRun)
+{
+    JNIEntry(SVNClient, merge);
+    SVNClient *cl = SVNClient::getCppObject(jthis);
+    if(cl == NULL)
+    {
+        JNIUtil::throwError(_("bad c++ this"));
+        return;
+    }
+    Revision revision1(jrevision1);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    JNIStringHolder path(jpath);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    Revision revision2(jrevision2);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    Revision pegRevision(jpegRevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    JNIStringHolder localPath(jlocalPath);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    cl->merge(path, pegRevision, revision1, revision2, localPath, 
+        jforce ? true:false, jrecurse ? true:false, 
+        jignoreAncestry ? true:false, jdryRun ? true:false);
 }
 
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    properties
- * Signature: (Ljava/lang/String;)[Lorg/tigris/subversion/javahl/PropertyData;
+ * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;)
+ *            [Lorg/tigris/subversion/javahl/PropertyData;
  */
 JNIEXPORT jobjectArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_properties
-  (JNIEnv* env, jobject jthis, jstring jpath)
+  (JNIEnv* env, jobject jthis, jstring jpath, jobject jrevision, 
+   jobject jpegRevision)
 {
     JNIEntry(SVNClient, properties);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -809,17 +913,27 @@ JNIEXPORT jobjectArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_prope
     {
         return NULL;
     }
-    return cl->properties(jthis, path);
+    Revision revision(jrevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    Revision pegRevision(jpegRevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    return cl->properties(jthis, path, revision, pegRevision);
 }
 
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    propertySet
- * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Z)V
+ * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;ZZ)V
  */
-JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertySet__Ljava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2Z
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertySet__Ljava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2ZZ
   (JNIEnv* env, jobject jthis, jstring jpath, jstring jname, jstring jvalue, 
-   jboolean jrecurse)
+   jboolean jrecurse, jboolean jforce)
 {
     JNIEntry(SVNClient, propertySet);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -843,17 +957,18 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertySet__
     {
         return;
     }
-    cl->propertySet(path, name, value, jrecurse ? true:false);
+    cl->propertySet(path, name, value, jrecurse ? true:false, 
+        jforce ? true:false);
 }
 
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    propertySet
- * Signature: (Ljava/lang/String;Ljava/lang/String;[BZ)V
+ * Signature: (Ljava/lang/String;Ljava/lang/String;[BZZ)V
  */
-JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertySet__Ljava_lang_String_2Ljava_lang_String_2_3BZ
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertySet__Ljava_lang_String_2Ljava_lang_String_2_3BZZ
   (JNIEnv* env, jobject jthis, jstring jpath, jstring jname, 
-   jbyteArray jvalue, jboolean jrecurse)
+   jbyteArray jvalue, jboolean jrecurse, jboolean jforce )
 {
     JNIEntry(SVNClient, propertySet);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -877,7 +992,8 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertySet__
     {
         return;
     }
-    cl->propertySet(path, name, value, jrecurse ? true:false);
+    cl->propertySet(path, name, value, jrecurse ? true:false,
+        jforce ? true:false);
 }
 
 /*
@@ -911,11 +1027,11 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertyRemov
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    propertyCreate
- * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Z)V
+ * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;ZZ)V
  */
-JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertyCreate__Ljava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2Z
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertyCreate__Ljava_lang_String_2Ljava_lang_String_2Ljava_lang_String_2ZZ
   (JNIEnv* env, jobject jthis, jstring jpath, jstring jname, jstring jvalue, 
-   jboolean jrecurse)
+   jboolean jrecurse, jboolean jforce)
 {
     JNIEntry(SVNClient, propertyCreate);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -939,18 +1055,19 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertyCreat
     {
         return;
     }
-    cl->propertyCreate(path, name, value, jrecurse ? true:false);
+    cl->propertyCreate(path, name, value, jrecurse ? true:false, 
+        jforce ? true:false);
 }
 
 
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    propertyCreate
- * Signature: (Ljava/lang/String;Ljava/lang/String;[BZ)V
+ * Signature: (Ljava/lang/String;Ljava/lang/String;[BZZ)V
  */
-JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertyCreate__Ljava_lang_String_2Ljava_lang_String_2_3BZ
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertyCreate__Ljava_lang_String_2Ljava_lang_String_2_3BZZ
   (JNIEnv* env, jobject jthis, jstring jpath, jstring jname, jbyteArray jvalue, 
-   jboolean jrecurse)
+   jboolean jrecurse, jboolean jforce)
 {
     JNIEntry(SVNClient, propertyCreate);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -974,12 +1091,15 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertyCreat
     {
         return;
     }
-    cl->propertyCreate(path, name, value, jrecurse ? true:false);
+    cl->propertyCreate(path, name, value, jrecurse ? true:false, 
+        jforce ? true:false);
 }
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    revProperty
- * Signature: (Ljava/lang/String;Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;)Lorg/tigris/subversion/javahl/PropertyData;
+ * Signature: (Ljava/lang/String;Ljava/lang/String;
+ *             Lorg/tigris/subversion/javahl/Revision;)
+ *            Lorg/tigris/subversion/javahl/PropertyData;
  */
 JNIEXPORT jobject JNICALL Java_org_tigris_subversion_javahl_SVNClient_revProperty
   (JNIEnv *env, jobject jthis, jstring jpath, jstring jname, jobject jrevision)
@@ -1008,13 +1128,85 @@ JNIEXPORT jobject JNICALL Java_org_tigris_subversion_javahl_SVNClient_revPropert
     }
     return cl->revProperty(jthis, path, name, revision);
 }
-  /*
+/*
+ * Class:     org_tigris_subversion_javahl_SVNClient
+ * Method:    revProperties
+ * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;)
+ *            [Lorg/tigris/subversion/javahl/PropertyData;
+ */
+JNIEXPORT jobjectArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_revProperties
+  (JNIEnv *env, jobject jthis, jstring jpath, jobject jrevision)
+{
+    JNIEntry(SVNClient, revProperty);
+    SVNClient *cl = SVNClient::getCppObject(jthis);
+    if(cl == NULL)
+    {
+        JNIUtil::throwError(_("bad c++ this"));
+        return NULL;
+    }
+    JNIStringHolder path(jpath);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    Revision revision(jrevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    return cl->revProperties(jthis, path, revision);
+}
+/*
+ * Class:     org_tigris_subversion_javahl_SVNClient
+ * Method:    setRevProperty
+ * Signature: (Ljava/lang/String;Ljava/lang/String;
+ *             Lorg/tigris/subversion/javahl/Revision;Ljava/lang/String;Z)V
+ */
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_setRevProperty
+  (JNIEnv *env, jobject jthis, jstring jpath, jstring jname, jobject jrevision, 
+   jstring jvalue, jboolean jforce)
+{
+    JNIEntry(SVNClient, setRevProperty);
+    SVNClient *cl = SVNClient::getCppObject(jthis);
+    if(cl == NULL)
+    {
+        JNIUtil::throwError(_("bad c++ this"));
+        return;
+    }
+    JNIStringHolder path(jpath);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    JNIStringHolder name(jname);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    Revision revision(jrevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    JNIStringHolder value(jvalue);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    cl->setRevProperty(jthis, path, name, revision, value, 
+        jforce ? true: false);
+}
+/*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    propertyGet
- * Signature: (Ljava/lang/String;Ljava/lang/String;)Lorg/tigris/subversion/javahl/PropertyData;
+ * Signature: (Ljava/lang/String;Ljava/lang/String;
+ *             Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;)
+ *            Lorg/tigris/subversion/javahl/PropertyData;
  */
 JNIEXPORT jobject JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertyGet
-  (JNIEnv *env, jobject jthis, jstring jpath, jstring jname)
+  (JNIEnv *env, jobject jthis, jstring jpath, jstring jname, jobject jrevision,
+   jobject jpegRevision)
 {
     JNIEntry(SVNClient, propertyGet);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -1033,21 +1225,32 @@ JNIEXPORT jobject JNICALL Java_org_tigris_subversion_javahl_SVNClient_propertyGe
     {
         return NULL;
     }
-    return cl->propertyGet(jthis, path, name);
+    Revision revision(jrevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    Revision pegRevision(jpegRevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    return cl->propertyGet(jthis, path, name, revision, pegRevision);
 }
 
 
-  /*
+/*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    diff
  * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
  *             Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
- *             Ljava/lang/String;Z)V
+ *             Ljava/lang/String;ZZZZ)V
  */
-JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_diff
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_diff__Ljava_lang_String_2Lorg_tigris_subversion_javahl_Revision_2Ljava_lang_String_2Lorg_tigris_subversion_javahl_Revision_2Ljava_lang_String_2ZZZZ
   (JNIEnv *env, jobject jthis, jstring jtarget1, jobject jrevision1, 
    jstring jtarget2, jobject jrevision2, jstring joutfileName,
-   jboolean jrecurse)
+   jboolean jrecurse, jboolean jignoreAncestry, jboolean jnoDiffDeleted, 
+   jboolean jforce)
 {
     JNIEntry(SVNClient, diff);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -1082,17 +1285,69 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_diff
         return;
     }
     cl->diff(target1, revision1, target2, revision2, outfileName,
-             jrecurse ? true:false);
+        jrecurse ? true:false, jignoreAncestry ? true:false,
+        jnoDiffDeleted ? true:false, jforce ? true:false);
+}
+/*
+ * Class:     org_tigris_subversion_javahl_SVNClient
+ * Method:    diff
+ * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;Ljava/lang/String;ZZZZ)V
+ */
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_diff__Ljava_lang_String_2Lorg_tigris_subversion_javahl_Revision_2Lorg_tigris_subversion_javahl_Revision_2Lorg_tigris_subversion_javahl_Revision_2Ljava_lang_String_2ZZZZ
+  (JNIEnv *env, jobject jthis, jstring jtarget, jobject jpegRevision, 
+   jobject jstartRevision, jobject jendRevision, jstring joutfileName, 
+   jboolean jrecurse, jboolean jignoreAncestry, jboolean jnoDiffDeleted, 
+   jboolean jforce)
+{
+    JNIEntry(SVNClient, diff);
+    SVNClient *cl = SVNClient::getCppObject(jthis);
+    if(cl == NULL)
+    {
+        JNIUtil::throwError(_("bad c++ this"));
+        return;
+    }
+    JNIStringHolder target(jtarget);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    Revision pegRevision(jpegRevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    Revision startRevision(jstartRevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    Revision endRevision(jendRevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    JNIStringHolder outfileName(joutfileName);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    cl->diff(target, pegRevision, startRevision, endRevision, outfileName,
+        jrecurse ? true:false, jignoreAncestry ? true:false,
+        jnoDiffDeleted ? true:false, jforce ? true:false);
 }
 
 
-  /*
+/*
  * Class:     org_tigris_subversion_javahl_SVNClient
  * Method:    fileContent
- * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;)[B
+ * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;)[B
  */
 JNIEXPORT jbyteArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_fileContent
-  (JNIEnv *env, jobject jthis, jstring jpath, jobject jrevision)
+  (JNIEnv *env, jobject jthis, jstring jpath, jobject jrevision, 
+   jobject jpegRevision)
 {
     JNIEntry(SVNClient, propertyCreate);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -1111,7 +1366,36 @@ JNIEXPORT jbyteArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_fileCon
     {
         return NULL;
     }
-    return cl->fileContent(path, revision);
+    Revision pegRevision(jpegRevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    return cl->fileContent(path, revision, pegRevision);
+}
+/*
+ * Class:     org_tigris_subversion_javahl_SVNClient
+ * Method:    getVersionInfo
+ * Signature: (Ljava/lang/String;Ljava/lang/String;Z)Ljava/lang/String;
+ */
+JNIEXPORT jstring JNICALL Java_org_tigris_subversion_javahl_SVNClient_getVersionInfo
+  (JNIEnv *env, jobject jthis, jstring jpath, jstring jtrailUrl, 
+   jboolean jlastChanged)
+{
+    JNIEntry(SVNClient, getVersionInfo);
+    SVNClient *cl = SVNClient::getCppObject(jthis);
+    if(cl == NULL)
+    {
+        JNIUtil::throwError(_("bad c++ this"));
+        return NULL;
+    }
+    JNIStringHolder path(jpath);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    JNIStringHolder trailUrl(jtrailUrl);
+    return cl->getVersionInfo(path, trailUrl, jlastChanged ? true:false);
 }
 
 /*
@@ -1264,11 +1548,12 @@ JNIEXPORT jbyteArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_blame__
  * Method:    blame
  * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
  *             Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;
  *             Lorg/tigris/subversion/javahl/BlameCallback;)V
  */
-JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_blame__Ljava_lang_String_2Lorg_tigris_subversion_javahl_Revision_2Lorg_tigris_subversion_javahl_Revision_2Lorg_tigris_subversion_javahl_BlameCallback_2
-  (JNIEnv *env, jobject jthis, jstring jpath, jobject jrevisionStart, 
-   jobject jrevisionEnd, jobject jblameCallback)
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_blame__Ljava_lang_String_2Lorg_tigris_subversion_javahl_Revision_2Lorg_tigris_subversion_javahl_Revision_2Lorg_tigris_subversion_javahl_Revision_2Lorg_tigris_subversion_javahl_BlameCallback_2
+  (JNIEnv *env, jobject jthis, jstring jpath, jobject jpegRevision, 
+   jobject jrevisionStart, jobject jrevisionEnd, jobject jblameCallback)
 {
     JNIEntry(SVNClient, blame);
     SVNClient *cl = SVNClient::getCppObject(jthis);
@@ -1282,6 +1567,11 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_blame__Ljava_
     {
         return;
     }
+    Revision pegRevision(jpegRevision, false, true);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
     Revision revisionStart(jrevisionStart, false, true);
     if(JNIUtil::isExceptionThrown())
     {
@@ -1293,7 +1583,7 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_blame__Ljava_
         return;
     }
     BlameCallback callback(jblameCallback);
-    cl->blame(path, revisionStart, revisionEnd, &callback);
+    cl->blame(path, pegRevision, revisionStart, revisionEnd, &callback);
 }
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
@@ -1353,6 +1643,7 @@ JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_cancelOperati
         JNIUtil::throwError("bad c++ this");
         return;
     }
+    cl->cancelOperation();
 }
 /*
  * Class:     org_tigris_subversion_javahl_SVNClient
@@ -1377,3 +1668,91 @@ JNIEXPORT jobject JNICALL Java_org_tigris_subversion_javahl_SVNClient_info
     }
     return cl->info(path);
 }
+/*
+ * Class:     org_tigris_subversion_javahl_SVNClient
+ * Method:    lock
+ * Signature: ([Ljava/lang/String;Ljava/lang/String;Z)V
+ */
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_lock
+  (JNIEnv *env, jobject jthis, jobjectArray jtargets, jstring jcomment, 
+   jboolean jforce)
+{
+    JNIEntry(SVNClient, lock);
+    SVNClient *cl = SVNClient::getCppObject(jthis);
+    if(cl == NULL)
+    {
+        JNIUtil::throwError("bad c++ this");
+        return;
+    }
+    Targets targets(jtargets);    
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    JNIStringHolder comment(jcomment);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+    cl->lock(targets, comment, jforce ? true : false);
+}
+
+/*
+ * Class:     org_tigris_subversion_javahl_SVNClient
+ * Method:    unlock
+ * Signature: ([Ljava/lang/String;Z)V
+ */
+JNIEXPORT void JNICALL Java_org_tigris_subversion_javahl_SVNClient_unlock
+  (JNIEnv *env, jobject jthis, jobjectArray jtargets, jboolean jforce)
+{
+    JNIEntry(SVNClient, unlock);
+    SVNClient *cl = SVNClient::getCppObject(jthis);
+    if(cl == NULL)
+    {
+        JNIUtil::throwError("bad c++ this");
+        return;
+    }
+    
+    Targets targets(jtargets);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return;
+    }
+
+    cl->unlock(targets, jforce ? true : false);
+}
+/*
+ * Class:     org_tigris_subversion_javahl_SVNClient
+ * Method:    info2
+ * Signature: (Ljava/lang/String;Lorg/tigris/subversion/javahl/Revision;
+ *             Lorg/tigris/subversion/javahl/Revision;Z)
+ *            [Lorg/tigris/subversion/javahl/Info2;
+ */
+JNIEXPORT jobjectArray JNICALL Java_org_tigris_subversion_javahl_SVNClient_info2
+  (JNIEnv *env, jobject jthis, jstring jpath, jobject jrevision, 
+   jobject jpegRevision, jboolean jrecurse)
+{
+    JNIEntry(SVNClient, unlock);
+    SVNClient *cl = SVNClient::getCppObject(jthis);
+    if(cl == NULL)
+    {
+        JNIUtil::throwError("bad c++ this");
+        return NULL;
+    }
+    JNIStringHolder path(jpath);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    Revision revision(jrevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    Revision pegRevision(jpegRevision);
+    if(JNIUtil::isExceptionThrown())
+    {
+        return NULL;
+    }
+    return cl->info2(path, revision, pegRevision, jrecurse ? true : false);
+}
