@@ -18,6 +18,7 @@
 package org.tigris.subversion.javahl.tests;
 
 import junit.framework.TestSuite;
+import junit.framework.TestResult;
 import org.tigris.subversion.javahl.*;
 
 import java.io.File;
@@ -63,7 +64,11 @@ public class BasicTests extends SVNTests
      */
     public static void main(String[] args) {
         processArgs(args);
-        junit.textui.TestRunner.run(suite());
+        TestResult tr = junit.textui.TestRunner.run(suite());
+        if (tr.errorCount() != 0 || tr.failureCount() != 0) {
+            System.exit(1);
+        }
+        System.exit(0);
     }
 
     /**
@@ -1261,7 +1266,7 @@ public class BasicTests extends SVNTests
     }
 
     /**
-     * test the basic SVNClientInfo.logMessage functionality
+     * test the basic SVNClient.logMessage functionality
      * @throws Throwable
      */
     public void testBasicLogMessage() throws Throwable
@@ -1284,4 +1289,74 @@ public class BasicTests extends SVNTests
         assertNull("wrong copy source path", cp[0].getCopySrcPath());
         assertEquals("wrong action", 'A', cp[0].getAction());
     }
+
+    /**
+     * test the basic SVNClient.getVersionInfo functionality
+     * @throws Throwable
+     * @since 1.2
+     */
+    public void testBasicVersionInfo() throws Throwable
+    {
+        // create the working copy
+        OneTest thisTest = new OneTest();
+        assertEquals("wrong version info","1",
+                client.getVersionInfo(thisTest.getWCPath(), null, false));        
+    }
+
+    /**
+     * test the baisc SVNClient locking functionality
+     * @throws Throwable
+     * @since 1.2
+     */
+    public void testBasicLocking() throws Throwable
+    {
+        // build the first working copy
+        OneTest thisTest = new OneTest();
+
+        client.propertySet(thisTest.getWCPath()+"/A/mu",
+                           PropertyData.NEEDS_LOCK, "*", false);
+
+        addExpectedCommitItem(thisTest.getWCPath(),
+                thisTest.getUrl(), "A/mu",NodeKind.file,
+                CommitItemStateFlags.PropMods);
+        assertEquals("bad revision number on commit", 2,
+                     client.commit(new String[] {thisTest.getWCPath()},
+                                   "message", true));
+        File f = new File(thisTest.getWCPath()+"/A/mu");
+        assertEquals("file should be read only now", false, f.canWrite());
+        client.lock(new String[] {thisTest.getWCPath()+"/A/mu"},
+                                "comment", false);
+        assertEquals("file should be read write now", true, f.canWrite());
+        client.unlock(new String[]{thisTest.getWCPath()+"/A/mu"},
+                false);
+        assertEquals("file should be read only now", false, f.canWrite());
+        client.lock(new String[]{thisTest.getWCPath()+"/A/mu"},
+                           "comment", false);
+        assertEquals("file should be read write now", true, f.canWrite());
+        addExpectedCommitItem(thisTest.getWCPath(),
+                thisTest.getUrl(), "A/mu",NodeKind.file,
+                    0);
+        assertEquals("rev number from commit",-1, client.commit(
+                new String[]{thisTest.getWCPath()},"message", true));
+        assertEquals("file should be read write now", true, f.canWrite());
+    }
+
+    /**
+     * test the baisc SVNClient.info2 functionality 
+     * @throws Throwable
+     * @since 1.2
+     */
+    public void testBasicInfo2() throws Throwable
+    {
+        // build the first working copy
+        OneTest thisTest = new OneTest();
+
+        Info2[] infos = client.info2(thisTest.getWCPath(), null, null, false);
+        assertEquals("this should return 1 info object", 1, infos.length);
+        infos = client.info2(thisTest.getWCPath(), null, null, true);
+        assertEquals("this should return 21 info objects", 21, infos.length);
+        infos = client.info2(thisTest.getWCPath(), new Revision.Number(1),
+                             new Revision.Number(1), true);
+        assertEquals("this should return 21 info objects", 21, infos.length);
+    }
 }
