@@ -23,58 +23,24 @@
 /*** Includes. ***/
 
 #define APR_WANT_STRFUNC
+#define APR_WANT_STDIO
 #include <apr_want.h>
 
-#include "svn_wc.h"
 #include "svn_client.h"
 #include "svn_string.h"
 #include "svn_path.h"
-#include "svn_delta.h"
 #include "svn_error.h"
-#include "svn_pools.h"
 #include "svn_sorts.h"
 #include "svn_xml.h"
 #include "svn_time.h"
 #include "svn_cmdline.h"
-#include "svn_subst.h"
 #include "cl.h"
 
+#include "svn_private_config.h"
+
 
 /*** Code. ***/
 
-/* Helper for log_message_receiver(). 
- *
- * Return the number of lines in MSG, allowing any kind of newline
- * termination (CR, CRLF, or LFCR), even inconsistent.  The minimum
- * number of lines in MSG is 1 -- even the empty string is considered
- * to have one line, due to the way we print log messages.
- */
-static int
-num_lines (const char *msg)
-{
-  int count = 1;
-  const char *p;
-
-  for (p = msg; *p; p++)
-    {
-      if (*p == '\n')
-        {
-          count++;
-          if (*(p + 1) == '\r')
-            p++;
-        }
-      else if (*p == '\r')
-        {
-          count++;
-          if (*(p + 1) == '\n')
-            p++;
-        }
-    }
-
-  return count;
-}
-
-
 /* Baton for log_message_receiver() and log_message_receiver_xml(). */
 struct log_receiver_baton
 {
@@ -84,16 +50,12 @@ struct log_receiver_baton
 
   /* Don't print log message body nor its line count. */
   svn_boolean_t omit_log_message;
-
-  /* Output stream */
-  svn_stream_t *out;
 };
 
 
 /* The separator between log messages. */
 #define SEP_STRING \
-  "------------------------------------------------------------------------" \
-  APR_EOL_STR
+  "------------------------------------------------------------------------\n"
 
 
 /* Implement `svn_log_message_receiver_t', printing the logs in
@@ -183,9 +145,6 @@ log_message_receiver (void *baton,
                       apr_pool_t *pool)
 {
   struct log_receiver_baton *lb = baton;
-  const char *author_stdout, *date_stdout;
-  const char *msg_stdout = NULL;     /* Silence a gcc uninitialized warning */
-  svn_error_t *err;
 
   /* Number of lines in the msg. */
   int lines;
@@ -193,68 +152,43 @@ log_message_receiver (void *baton,
   if (lb->cancel_func)
     SVN_ERR (lb->cancel_func (lb->cancel_baton));
 
-  if (rev == 0)
-    {
-      return svn_stream_printf (lb->out, pool,
-                                "No commit for revision 0." APR_EOL_STR);
-    }
+  if (rev == 0 && msg == NULL)
+    return SVN_NO_ERROR;
 
   /* ### See http://subversion.tigris.org/issues/show_bug.cgi?id=807
      for more on the fallback fuzzy conversions below. */
 
   if (author == NULL)
-    author = "(no author)";
-
-  err = svn_cmdline_cstring_from_utf8 (&author_stdout, author, pool);
-  if (err && (APR_STATUS_IS_EINVAL (err->apr_err)))
-    {
-      author_stdout = svn_cmdline_cstring_from_utf8_fuzzy (author, pool);
-      svn_error_clear (err);
-    }
-  else if (err)
-    return err;
+    author = _("(no author)");
 
   if (date && date[0])
     {
       /* Convert date to a format for humans. */
       apr_time_t time_temp;
-      const char *date_utf8;
       
       SVN_ERR (svn_time_from_cstring (&time_temp, date, pool));
-      date_utf8 = svn_time_to_human_cstring(time_temp, pool);
-      SVN_ERR (svn_cmdline_cstring_from_utf8 (&date_stdout, date_utf8, pool));
+      date = svn_time_to_human_cstring(time_temp, pool);
     }
   else
-    date_stdout = "(no date)";
-  
-  if (! lb->omit_log_message)
-    {
-      if (msg == NULL)
-        msg = "";
+    date = _("(no date)");
 
-      {
-        /* Convert log message from UTF8/LF to native locale and eol-style. */
-        svn_string_t *logmsg = svn_string_create (msg, pool);
-        svn_subst_detranslate_string (&logmsg, logmsg, TRUE, pool);
-        msg_stdout = logmsg->data;
-      }
-    }
+  if (! lb->omit_log_message && msg == NULL)
+    msg = "";
 
-  SVN_ERR (svn_stream_printf (lb->out, pool, SEP_STRING));
-
-  SVN_ERR (svn_stream_printf (lb->out, pool,
-                              "r%" SVN_REVNUM_T_FMT " | %s | %s",
-                              rev, author_stdout, date_stdout));
+  SVN_ERR (svn_cmdline_printf (pool,
+                               SEP_STRING "r%ld | %s | %s",
+                               rev, author, date));
 
   if (! lb->omit_log_message)
     {
-      lines = num_lines (msg_stdout);
-      SVN_ERR (svn_stream_printf (lb->out, pool,
-                                  " | %d line%s", lines,
-                                  (lines > 1) ? "s" : ""));
+      lines = svn_cstring_count_newlines (msg) + 1;
+      SVN_ERR (svn_cmdline_printf (pool,
+                                   (lines != 1)
+                                   ? " | %d lines"
+                                   : " | %d line", lines));
     }
 
-  SVN_ERR (svn_stream_printf (lb->out, pool, APR_EOL_STR));
+  SVN_ERR (svn_cmdline_printf (pool, "\n"));
 
   if (changed_paths)
     {
@@ -265,13 +199,13 @@ log_message_receiver (void *baton,
       sorted_paths = svn_sort__hash (changed_paths,
                                      svn_sort_compare_items_as_paths, pool);
 
-      SVN_ERR (svn_stream_printf (lb->out, pool,
-                                  "Changed paths:" APR_EOL_STR));
+      SVN_ERR (svn_cmdline_printf (pool,
+                                   _("Changed paths:\n")));
       for (i = 0; i < sorted_paths->nelts; i++)
         {
           svn_sort__item_t *item = &(APR_ARRAY_IDX (sorted_paths, i,
                                                     svn_sort__item_t));
-          const char *path_stdout, *path = item->key;
+          const char *path = item->key;
           svn_log_changed_path_t *log_item 
             = apr_hash_get (changed_paths, item->key, item->klen);
           const char *copy_data = "";
@@ -279,26 +213,22 @@ log_message_receiver (void *baton,
           if (log_item->copyfrom_path 
               && SVN_IS_VALID_REVNUM (log_item->copyfrom_rev))
             {
-              SVN_ERR (svn_cmdline_cstring_from_utf8
-                       (&path_stdout, log_item->copyfrom_path, pool));
               copy_data 
                 = apr_psprintf (pool, 
-                                " (from %s:%" SVN_REVNUM_T_FMT ")",
-                                path_stdout,
+                                _(" (from %s:%ld)"),
+                                log_item->copyfrom_path,
                                 log_item->copyfrom_rev);
             }
-          SVN_ERR (svn_cmdline_cstring_from_utf8 (&path_stdout, path, pool));
-          SVN_ERR (svn_stream_printf (lb->out, pool, "   %c %s%s" APR_EOL_STR,
-                                      log_item->action, path_stdout,
-                                      copy_data));
+          SVN_ERR (svn_cmdline_printf (pool, "   %c %s%s\n",
+                                       log_item->action, path,
+                                       copy_data));
         }
     }
 
   if (! lb->omit_log_message)
     {
       /* A blank line always precedes the log message. */
-      SVN_ERR (svn_stream_printf (lb->out, pool, APR_EOL_STR "%s" APR_EOL_STR,
-                                  msg_stdout));
+      SVN_ERR (svn_cmdline_printf (pool, "\n%s\n", msg));
     }
 
   return SVN_NO_ERROR;
@@ -317,14 +247,14 @@ log_message_receiver (void *baton,
  * <logentry
  *    revision="1648">
  * <author>david</author>
- * <date>Sat 6 Apr 2002 16:34:51.428043 (day 096, dst 0, gmt_off -21600)</date>
+ * <date>2002-04-06T16:34:51.428043Z</date>
  * <msg> * packages/rpm/subversion.spec : Now requires apache 2.0.36.
  * </msg>
  * </logentry>
  * <logentry
  *    revision="1649">
  * <author>cmpilato</author>
- * <date>Sat 6 Apr 2002 17:01:28.185136 (day 096, dst 0, gmt_off -21600)</date>
+ * <date>2002-04-06T17:01:28.185136Z</date>
  * <msg>Fix error handling when the $EDITOR is needed but unavailable.  Ah
  * ... now that&apos;s *much* nicer.
  * 
@@ -354,15 +284,14 @@ log_message_receiver_xml (void *baton,
   /* Collate whole log message into sb before printing. */
   svn_stringbuf_t *sb = svn_stringbuf_create ("", pool);
   char *revstr;
-  const char *msg_native_eol;  
 
   if (lb->cancel_func)
     SVN_ERR (lb->cancel_func (lb->cancel_baton));
 
-  if (rev == 0)
+  if (rev == 0 && msg == NULL)
     return SVN_NO_ERROR;
 
-  revstr = apr_psprintf (pool, "%" SVN_REVNUM_T_FMT, rev);
+  revstr = apr_psprintf (pool, "%ld", rev);
   /* <logentry revision="xxx"> */
   svn_xml_make_open_tag (&sb, pool, svn_xml_normal, "logentry",
                          "revision", revstr, NULL);
@@ -415,7 +344,7 @@ log_message_receiver_xml (void *baton,
               svn_stringbuf_t *escpath = svn_stringbuf_create ("", pool);
               svn_xml_escape_attr_cstring (&escpath,
                                            log_item->copyfrom_path, pool);
-              revstr = apr_psprintf (pool, "%" SVN_REVNUM_T_FMT, 
+              revstr = apr_psprintf (pool, "%ld", 
                                      log_item->copyfrom_rev);
               svn_xml_make_open_tag (&sb, pool, svn_xml_protect_pcdata, "path",
                                      "action", action,
@@ -444,20 +373,14 @@ log_message_receiver_xml (void *baton,
 
       /* <msg>xxx</msg> */
       svn_xml_make_open_tag (&sb, pool, svn_xml_protect_pcdata, "msg", NULL);
-      SVN_ERR (svn_subst_translate_cstring (msg, &msg_native_eol,
-                                            APR_EOL_STR, /* the 'native' eol */
-                                            FALSE,       /* no need to repair */
-                                            NULL,        /* no keywords */
-                                            FALSE,       /* no expansion */
-                                            pool));
-      svn_xml_escape_cdata_cstring (&sb, msg_native_eol, pool);
+      svn_xml_escape_cdata_cstring (&sb, msg, pool);
       svn_xml_make_close_tag (&sb, pool, "msg");
     }
 
   /* </logentry> */
   svn_xml_make_close_tag (&sb, pool, "logentry");
 
-  SVN_ERR (svn_stream_printf (lb->out, pool, "%s", sb->data));
+  SVN_ERR (svn_cl__error_checked_fputs (sb->data, stdout));
 
   return SVN_NO_ERROR;
 }
@@ -473,16 +396,18 @@ svn_cl__log (apr_getopt_t *os,
   svn_client_ctx_t *ctx = ((svn_cl__cmd_baton_t *) baton)->ctx;
   apr_array_header_t *targets;
   struct log_receiver_baton lb;
+  const char *target;
+  int i;
 
-  SVN_ERR (svn_opt_args_to_target_array (&targets, os, 
-                                         opt_state->targets,
-                                         &(opt_state->start_revision),
-                                         &(opt_state->end_revision),
-                                         FALSE, pool));
+  SVN_ERR (svn_opt_args_to_target_array2 (&targets, os, 
+                                          opt_state->targets, pool));
 
   /* Add "." if user passed 0 arguments */
   svn_opt_push_implicit_dot_target(targets, pool);
 
+  /* Retrieve the first target in the list. */
+  target = APR_ARRAY_IDX (targets, 0, const char *);
+
   if ((opt_state->start_revision.kind != svn_opt_revision_unspecified)
       && (opt_state->end_revision.kind == svn_opt_revision_unspecified))
     {
@@ -499,10 +424,8 @@ svn_cl__log (apr_getopt_t *os,
     }
   else if (opt_state->start_revision.kind == svn_opt_revision_unspecified)
     {
-      const char *target = APR_ARRAY_IDX (targets, 0, const char *);
-
-      /* If the first target is a URL, then we default to HEAD:1.
-         Otherwise, the default is BASE:1 since WC@HEAD may not exist. */
+      /* If the first target is a URL, then we default to HEAD:0.
+         Otherwise, the default is BASE:0 since WC@HEAD may not exist. */
       if (svn_path_is_url (target))
         opt_state->start_revision.kind = svn_opt_revision_head;
       else
@@ -511,19 +434,40 @@ svn_cl__log (apr_getopt_t *os,
       if (opt_state->end_revision.kind == svn_opt_revision_unspecified)
         {
           opt_state->end_revision.kind = svn_opt_revision_number;
-          opt_state->end_revision.value.number = 1;  /* oldest commit */
+          opt_state->end_revision.value.number = 0;
         }
     }
 
+  /* Verify that we pass at most one working copy path. */
+  if (! svn_path_is_url (target) )
+    {
+      if (targets->nelts > 1)
+        return svn_error_create (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+                                 _("When specifying working copy paths, only "
+                                   "one target may be given"));
+    }
+  else
+    {
+      /* Check to make sure there are no other URLs. */
+      for (i = 1; i < targets->nelts; i++)
+        {
+          target = APR_ARRAY_IDX (targets, i, const char *);
+          
+          if (svn_path_is_url (target))
+            return svn_error_create (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+                                     _("Only relative paths can be specified "
+                                       "after a URL"));
+        }
+    }
+  
   lb.cancel_func = ctx->cancel_func;
   lb.cancel_baton = ctx->cancel_baton;
   lb.omit_log_message = opt_state->quiet;
-  SVN_ERR (svn_stream_for_stdout (&lb.out, pool));
-
+  
   if (! opt_state->quiet)
-    svn_cl__get_notifier (&ctx->notify_func, &ctx->notify_baton, FALSE, FALSE,
-                          FALSE, pool);
-
+    svn_cl__get_notifier (&ctx->notify_func2, &ctx->notify_baton2, FALSE,
+                          FALSE, FALSE, pool);
+  
   if (opt_state->xml)
     {
       /* If output is not incremental, output the XML header and wrap
@@ -539,18 +483,19 @@ svn_cl__log (apr_getopt_t *os,
           /* "<log>" */
           svn_xml_make_open_tag (&sb, pool, svn_xml_normal, "log", NULL);
 
-          SVN_ERR (svn_stream_printf (lb.out, pool, "%s", sb->data));
+          SVN_ERR (svn_cl__error_checked_fputs (sb->data, stdout));
         }
       
-      SVN_ERR (svn_client_log (targets,
-                               &(opt_state->start_revision),
-                               &(opt_state->end_revision),
-                               opt_state->verbose,
-                               opt_state->stop_on_copy,
-                               log_message_receiver_xml,
-                               &lb,
-                               ctx,
-                               pool));
+      SVN_ERR (svn_client_log2 (targets,
+                                &(opt_state->start_revision),
+                                &(opt_state->end_revision),
+                                opt_state->limit,
+                                opt_state->verbose,
+                                opt_state->stop_on_copy,
+                                log_message_receiver_xml,
+                                &lb,
+                                ctx,
+                                pool));
       
       if (! opt_state->incremental)
         {
@@ -559,7 +504,7 @@ svn_cl__log (apr_getopt_t *os,
           /* "</log>" */
           svn_xml_make_close_tag (&sb, pool, "log");
 
-          SVN_ERR (svn_stream_printf (lb.out, pool, "%s", sb->data));
+          SVN_ERR (svn_cl__error_checked_fputs (sb->data, stdout));
         }
     }
   else  /* default output format */
@@ -571,18 +516,19 @@ svn_cl__log (apr_getopt_t *os,
        * is concerned, the result of 'svn log --quiet' is the same
        * either way.
        */
-      SVN_ERR (svn_client_log (targets,
-                               &(opt_state->start_revision),
-                               &(opt_state->end_revision),
-                               opt_state->verbose,
-                               opt_state->stop_on_copy,
-                               log_message_receiver,
-                               &lb,
-                               ctx,
-                               pool));
+      SVN_ERR (svn_client_log2 (targets,
+                                &(opt_state->start_revision),
+                                &(opt_state->end_revision),
+                                opt_state->limit,
+                                opt_state->verbose,
+                                opt_state->stop_on_copy,
+                                log_message_receiver,
+                                &lb,
+                                ctx,
+                                pool));
 
       if (! opt_state->incremental)
-        SVN_ERR (svn_stream_printf (lb.out, pool, SEP_STRING));
+        SVN_ERR (svn_cmdline_printf (pool, SEP_STRING));
     }
 
   return SVN_NO_ERROR;
