@@ -28,7 +28,6 @@
 #include <apr_strings.h>
 #include <apr_tables.h>
 #include <apr_general.h>
-#include <apr_lib.h>
 #include <apr_signal.h>
 
 #include "svn_cmdline.h"
@@ -39,10 +38,10 @@
 #include "svn_string.h"
 #include "svn_path.h"
 #include "svn_delta.h"
+#include "svn_diff.h"
 #include "svn_error.h"
 #include "svn_io.h"
 #include "svn_opt.h"
-#include "svn_time.h"
 #include "svn_utf.h"
 #include "svn_auth.h"
 #include "cl.h"
@@ -97,7 +96,7 @@ const apr_getopt_option_t svn_cl__options[] =
                       N_("specify a username ARG")},
     {"password",      svn_cl__auth_password_opt, 1, 
                       N_("specify a password ARG")},
-    {"extensions",    'x', 1, N_("pass ARG as bundled options to GNU diff")},
+    {"extensions",    'x', 1, N_("pass ARG to --diff-cmd as options (default: '-u')")},
     {"targets",       svn_cl__targets_opt, 1,
                       N_("pass contents of file ARG as additional args")},
     {"xml",           svn_cl__xml_opt, 0, N_("output in XML")},
@@ -113,11 +112,13 @@ const apr_getopt_option_t svn_cl__options[] =
     {"dry-run",       svn_cl__dry_run_opt, 0,
                       N_("try operation but make no changes")},
     {"no-diff-deleted", svn_cl__no_diff_deleted, 0,
-                       N_("do not print differences for deleted files")},
+                      N_("do not print differences for deleted files")},
     {"notice-ancestry", svn_cl__notice_ancestry_opt, 0,
-                       N_("notice ancestry when calculating differences")},
+                      N_("notice ancestry when calculating differences")},
     {"ignore-ancestry", svn_cl__ignore_ancestry_opt, 0,
-                       N_("ignore ancestry when calculating merges")},
+                      N_("ignore ancestry when calculating merges")},
+    {"ignore-externals", svn_cl__ignore_externals_opt, 0,
+                      N_("ignore externals definitions")},
     {"diff-cmd",      svn_cl__diff_cmd_opt, 1,
                       N_("use ARG as diff command")},
     {"diff3-cmd",     svn_cl__merge_cmd_opt, 1,
@@ -144,6 +145,10 @@ const apr_getopt_option_t svn_cl__options[] =
                       "with a native svn:eol-style\n"
                       "                             property.  ARG may be one "
                       "of 'LF', 'CR', 'CRLF'")},
+    {"limit",         svn_cl__limit_opt, 1,
+                      N_("maximum number of log entries")},
+    {"no-unlock",     svn_cl__no_unlock_opt, 0,
+                      N_("don't unlock the targets")},
     {0,               0, 0, 0}
   };
 
@@ -160,10 +165,16 @@ const apr_getopt_option_t svn_cl__options[] =
  * either a path or an url can be used.  Hmm, should this be part of the
  * help text?
  */
+/* Options for authentication. */
 #define SVN_CL__AUTH_OPTIONS svn_cl__auth_username_opt, \
                              svn_cl__auth_password_opt, \
                              svn_cl__no_auth_cache_opt, \
                              svn_cl__non_interactive_opt
+/* Options for giving a log message.  (Some of these also have other uses.) */
+#define SVN_CL__LOG_MSG_OPTIONS 'm', 'F', \
+                                svn_cl__force_log_opt, \
+                                svn_cl__editor_cmd_opt, \
+                                svn_cl__encoding_opt
  
 const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
 {
@@ -177,23 +188,33 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
   { "blame", svn_cl__blame, {"praise", "annotate", "ann"},
     N_("Output the content of specified files or\n"
        "URLs with revision and author information in-line.\n"
-       "usage: blame TARGET...\n"),
+       "usage: blame TARGET[@REV]...\n"
+       "\n"
+       "  If specified, REV determines in which revision the target is first\n"
+       "  looked up.\n"),
     {'r', 'v', SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
 
   { "cat", svn_cl__cat, {0},
     N_("Output the content of specified files or URLs.\n"
-       "usage: cat TARGET...\n"),
+       "usage: cat TARGET[@REV]...\n"
+       "\n"
+       "  If specified, REV determines in which revision the target is first\n"
+       "  looked up.\n"),
     {'r', SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
 
   { "checkout", svn_cl__checkout, {"co"}, N_
     ("Check out a working copy from a repository.\n"
-     "usage: checkout URL... [PATH]\n"
+     "usage: checkout URL[@REV]... [PATH]\n"
      "\n"
-     "  Note: If PATH is omitted, the basename of the URL will be used as\n"
+     "  If specified, REV determines in which revision the URL is first\n"
+     "  looked up.\n"
+     "\n"
+     "  If PATH is omitted, the basename of the URL will be used as\n"
      "  the destination. If multiple URLs are given each will be checked\n"
      "  out into a sub-directory of PATH, with the name of the sub-directory\n"
      "  being the basename of the URL.\n"),
-    {'r', 'q', 'N', SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
+    {'r', 'q', 'N', SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt,
+     svn_cl__ignore_externals_opt} },
 
   { "cleanup", svn_cl__cleanup, {0},
     N_("Recursively clean up the working copy, removing locks, resuming\n"
@@ -203,13 +224,14 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
   
   { "commit", svn_cl__commit, {"ci"},
     N_("Send changes from your working copy to the repository.\n"
-     "usage: commit [PATH...]\n"
-     "\n"
-     "  A log message must be provided, but it can be empty.  If it is not\n"
-     "  given by a --message or --file option, an editor will be started.\n"),
-    {'m', 'F', 'q', 'N', svn_cl__targets_opt,
-     svn_cl__force_log_opt, SVN_CL__AUTH_OPTIONS,
-     svn_cl__editor_cmd_opt, svn_cl__encoding_opt, svn_cl__config_dir_opt} },
+       "usage: commit [PATH...]\n"
+       "\n"
+       "  A log message must be provided, but it can be empty.  If it is not\n"
+       "  given by a --message or --file option, an editor will be started.\n"
+       "  If any targets are (or contain) locked items, those will be\n"
+       "  unlocked after a successful commit.\n"),
+    {'q', 'N', svn_cl__targets_opt, svn_cl__no_unlock_opt,
+     SVN_CL__LOG_MSG_OPTIONS, SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
   
   { "copy", svn_cl__copy, {"cp"},
     N_("Duplicate something in working copy or repository, remembering "
@@ -221,8 +243,8 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "    WC  -> URL:  immediately commit a copy of WC to URL\n"
        "    URL -> WC:   check out URL into WC, schedule for addition\n"
        "    URL -> URL:  complete server-side copy;  used to branch & tag\n"),
-    {'m', 'F', 'r', 'q', SVN_CL__AUTH_OPTIONS, svn_cl__force_log_opt,
-     svn_cl__editor_cmd_opt, svn_cl__encoding_opt, svn_cl__config_dir_opt} },
+    {'r', 'q',
+     SVN_CL__LOG_MSG_OPTIONS, SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
   
   { "delete", svn_cl__delete, {"del", "remove", "rm"},
     N_("Remove files and directories from version control.\n"
@@ -237,9 +259,8 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "\n"
        "  2. Each item specified by a URL is deleted from the repository\n"
        "    via an immediate commit.\n"),
-    {svn_cl__force_opt, svn_cl__force_log_opt, 'm', 'F', 'q', 
-     svn_cl__targets_opt, SVN_CL__AUTH_OPTIONS,
-     svn_cl__editor_cmd_opt, svn_cl__encoding_opt, svn_cl__config_dir_opt} },
+    {svn_cl__force_opt, 'q', svn_cl__targets_opt,
+     SVN_CL__LOG_MSG_OPTIONS, SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
   
   { "diff", svn_cl__diff, {"di"},
     N_("Display the differences between two paths.\n"
@@ -276,15 +297,15 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "\n"
        "  Use just 'svn diff' to display local modifications in "
        "a working copy.\n"),
-    {'r', svn_cl__old_cmd_opt, svn_cl__new_cmd_opt, 'x', 'N',
-     svn_cl__diff_cmd_opt, svn_cl__no_diff_deleted,
-     svn_cl__notice_ancestry_opt, SVN_CL__AUTH_OPTIONS,
+    {'r', svn_cl__old_cmd_opt, svn_cl__new_cmd_opt, 'N',
+     svn_cl__diff_cmd_opt, 'x', svn_cl__no_diff_deleted,
+     svn_cl__notice_ancestry_opt, svn_cl__force_opt, SVN_CL__AUTH_OPTIONS,
      svn_cl__config_dir_opt} },
 
   { "export", svn_cl__export, {0},
     N_("Create an unversioned copy of a tree.\n"
-       "usage: 1. export [-r REV] URL [PATH]\n"
-       "       2. export [-r REV] PATH1 [PATH2]\n"
+       "usage: 1. export [-r REV] URL[@PEGREV] [PATH]\n"
+       "       2. export [-r REV] PATH1[@PEGREV] [PATH2]\n"
        "\n"
        "  1. Exports a clean directory tree from the repository specified by\n"
        "     URL, at revision REV if it is given, otherwise at HEAD, into\n"
@@ -301,9 +322,14 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        " all local\n"
        "     changes will be preserved, but files not under version "
        "control will\n"
-       "     not be copied.\n"),
-    {'r', 'q', svn_cl__force_opt, SVN_CL__AUTH_OPTIONS,
-     svn_cl__config_dir_opt, svn_cl__native_eol_opt} },
+       "     not be copied.\n"
+       "\n"
+       "  If specified, PEGREV determines in which revision the target is "
+       "first\n"
+       "  looked up.\n"),
+    {'r', 'q', 'N', svn_cl__force_opt, SVN_CL__AUTH_OPTIONS,
+     svn_cl__config_dir_opt, svn_cl__native_eol_opt, 
+     svn_cl__ignore_externals_opt} },
 
   { "help", svn_cl__help, {"?", "h"},
     N_("Describe the usage of this program or its subcommands.\n"
@@ -321,25 +347,28 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "  Recursively commit a copy of PATH to URL.\n"
        "  If PATH is omitted '.' is assumed.  Parent directories are created\n"
        "  as necessary in the repository.\n"),
-    {'m', 'F', 'q', 'N', SVN_CL__AUTH_OPTIONS, svn_cl__force_log_opt,
-     svn_cl__editor_cmd_opt, svn_cl__encoding_opt, svn_cl__config_dir_opt,
-     svn_cl__autoprops_opt, svn_cl__no_autoprops_opt} },
+    {'q', 'N', svn_cl__autoprops_opt, svn_cl__no_autoprops_opt,
+     SVN_CL__LOG_MSG_OPTIONS, SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
  
   { "info", svn_cl__info, {0},
-    N_("Display information about a file or directory.\n"
-       "usage: info [PATH...]\n"
+    N_("Display information about a local or remote item.\n"
+       "usage: info [TARGET...]\n"
        "\n"
-       "  Print information about each PATH (default: '.').\n"),
-    {svn_cl__targets_opt, 'R', svn_cl__config_dir_opt} },
+       "  Print information about each TARGET (default: '.')\n"
+       "  TARGET may be either a working-copy path or URL.\n"),
+    {'r', 'R', svn_cl__targets_opt, SVN_CL__AUTH_OPTIONS, 
+     svn_cl__config_dir_opt} },
  
   { "list", svn_cl__ls, {"ls"},
     N_("List directory entries in the repository.\n"
-       "usage: list [TARGET...]\n"
+       "usage: list [TARGET[@REV]...]\n"
        "\n"
        "  List each TARGET file and the contents of each TARGET directory as\n"
        "  they exist in the repository.  If TARGET is a working copy path, "
        "the\n"
-       "  corresponding repository URL will be used.\n"
+       "  corresponding repository URL will be used. If specified, REV "
+       "determines\n"
+       "  in which revision the target is first looked up.\n"  
        "\n"
        "  The default TARGET is '.', meaning the repository URL of the "
        "current\n"
@@ -351,8 +380,18 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "    Author of the last commit\n"
        "    Size (in bytes)\n"
        "    Date and time of the last commit\n"),
-    {'r', 'v', 'R', SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
+    {'r', 'v', 'R', svn_cl__incremental_opt, svn_cl__xml_opt,
+     SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
   
+  { "lock", svn_cl__lock, {0},
+    N_("Lock working copy paths or URLs in the repository, so that\n"
+       "no other user can commit changes to them.\n"
+       "usage: lock TARGET...\n"
+       "\n"
+       "  Use --force to steal the lock from another user or working copy.\n"),
+    { svn_cl__targets_opt, 'm', 'F', svn_cl__force_log_opt,
+      svn_cl__encoding_opt, SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt,
+      svn_cl__force_opt } },
   { "log", svn_cl__log, {0},
     N_("Show the log messages for a set of revision(s) and/or file(s).\n"
        "usage: 1. log [PATH]\n"
@@ -383,7 +422,7 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "    svn log http://www.example.com/repo/project foo.c bar.c\n"),
     {'r', 'q', 'v', svn_cl__targets_opt, svn_cl__stop_on_copy_opt,
      svn_cl__incremental_opt, svn_cl__xml_opt, SVN_CL__AUTH_OPTIONS, 
-     svn_cl__config_dir_opt} },
+     svn_cl__config_dir_opt, svn_cl__limit_opt} },
 
   { "merge", svn_cl__merge, {0},
     N_("Apply the differences between two sources to a working copy path.\n"
@@ -432,8 +471,8 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "\n"
        "  In both cases, all the intermediate directories must already "
        "exist.\n"),
-    {'m', 'F', 'q', SVN_CL__AUTH_OPTIONS, svn_cl__editor_cmd_opt,
-     svn_cl__encoding_opt, svn_cl__force_log_opt, svn_cl__config_dir_opt} },
+    {'q',
+     SVN_CL__LOG_MSG_OPTIONS, SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
 
   { "move", svn_cl__move, {"mv", "rename", "ren"},
     N_("Move and/or rename something in working copy or repository.\n"
@@ -444,9 +483,8 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "  SRC and DST can both be working copy (WC) paths or URLs:\n"
        "    WC  -> WC:   move and schedule for addition (with history)\n"
        "    URL -> URL:  complete server-side rename.\n"),    
-    {'m', 'F', 'r', 'q', svn_cl__force_opt, SVN_CL__AUTH_OPTIONS,
-     svn_cl__editor_cmd_opt, svn_cl__encoding_opt, svn_cl__force_log_opt,
-     svn_cl__config_dir_opt} },
+    {'r', 'q', svn_cl__force_opt,
+     SVN_CL__LOG_MSG_OPTIONS, SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
   
   { "propdel", svn_cl__propdel, {"pdel", "pd"},
     N_("Remove PROPNAME from files, dirs, or revisions.\n"
@@ -471,10 +509,11 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
   
   { "propget", svn_cl__propget, {"pget", "pg"},
     N_("Print value of PROPNAME on files, dirs, or revisions.\n"
-       "usage: 1. propget PROPNAME [PATH...]\n"
+       "usage: 1. propget PROPNAME [TARGET[@REV]...]\n"
        "       2. propget PROPNAME --revprop -r REV [URL]\n"
        "\n"
-       "  1. Prints versioned prop in working copy.\n"
+       "  1. Prints versioned prop. If specified, REV determines in which\n"
+       "     revision the target is first looked up.\n"
        "  2. Prints unversioned remote prop on repos revision.\n"
        "\n"
        "  By default, this subcommand will add an extra newline to the end\n"
@@ -488,10 +527,11 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
 
   { "proplist", svn_cl__proplist, {"plist", "pl"},
     N_("List all properties on files, dirs, or revisions.\n"
-       "usage: 1. proplist [PATH...]\n"
+       "usage: 1. proplist [TARGET[@REV]...]\n"
        "       2. proplist --revprop -r REV [URL]\n"
        "\n"
-       "  1. Lists versioned props in working copy.\n"
+       "  1. Lists versioned props. If specified, REV determines in which\n"
+       "     revision the target is first looked up.\n"
        "  2. Lists unversioned remote props on repos revision.\n"),
     {'v', 'R', 'r', 'q', svn_cl__revprop_opt, SVN_CL__AUTH_OPTIONS,
      svn_cl__config_dir_opt} },
@@ -521,11 +561,7 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "      Id                       - A compressed summary of the "
        "previous\n"
        "                                   4 keywords.\n"
-       "    svn:executable - If present, make the file executable. This\n"
-       "      property cannot be set on a directory.  A non-recursive "
-       "attempt\n"
-       "      will fail, and a recursive attempt will set the property only\n"
-       "      on the file children of the directory.\n"
+       "    svn:executable - If present, make the file executable.\n"
        "    svn:eol-style  - One of 'native', 'LF', 'CR', 'CRLF'.\n"
        "    svn:mime-type  - The mimetype of the file.  Used to determine\n"
        "      whether to merge the file, and how to serve it from Apache.\n"
@@ -535,9 +571,19 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "      each of which consists of a relative directory path, optional\n"
        "      revision flags, and an URL.  For example\n"
        "        foo             http://example.com/repos/zig\n"
-       "        foo/bar -r 1234 http://example.com/repos/zag\n"),
-    {'F', 'q', 'r', svn_cl__targets_opt, 'R', svn_cl__revprop_opt,
-     SVN_CL__AUTH_OPTIONS, svn_cl__encoding_opt, svn_cl__force_opt,
+       "        foo/bar -r 1234 http://example.com/repos/zag\n"
+       "    svn:needs-lock - If present, indicates that the file should be "
+       "locked\n"
+       "      before it is modified.  Makes the working copy file read-only\n"
+       "      when it is not locked.\n"
+       "  The svn:keywords, svn:executable, svn:eol-style, svn:mime-type "
+       "and\n"
+       "  svn:needs-lock properties cannot be set on a directory.  A "
+       "non-recursive\n"
+       "  attempt will fail, and a recursive attempt will set the property\n"
+       "  only on the file children of the directory.\n"),
+    {'F', svn_cl__encoding_opt, 'q', 'r', svn_cl__targets_opt, 'R',
+     svn_cl__revprop_opt, SVN_CL__AUTH_OPTIONS, svn_cl__force_opt,
      svn_cl__config_dir_opt} },
   
   { "resolved", svn_cl__resolved, {0},
@@ -567,7 +613,7 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
      "  With -u, add working revision and server out-of-date information.\n"
      "  With -v, print full revision information on every item.\n"
      "\n"
-     "  The first five columns in the output are each one character wide:\n"
+     "  The first six columns in the output are each one character wide:\n"
      "    First column: Says if item was added, deleted, or otherwise "
      "changed\n"
      "      ' ' no modifications\n"
@@ -595,6 +641,16 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
      "    Fifth column: Whether the item is switched relative to its parent\n"
      "      ' ' normal\n"
      "      'S' switched\n"
+     "    Sixth column: Repository lock token\n"
+     "      (without -u)\n"
+     "      ' ' no lock token\n"
+     "      'K' lock token present\n"
+     "      (with -u)\n"
+     "      ' ' not locked in repository, no lock token\n"
+     "      'K' locked in repository, lock toKen present\n"
+     "      'O' locked in repository, lock token in some Other working copy\n"
+     "      'T' locked in repository, lock token present but sTolen\n"
+     "      'B' not locked in repository, lock token present but Broken\n"
      "\n"
      "  The out-of-date information appears in the eighth column (with -u):\n"
      "      '*' a newer revision exists on the server\n"
@@ -615,16 +671,16 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
      "     M           965    wc/bar.c\n"
      "           *     965    wc/foo.c\n"
      "    A  +         965    wc/qax.c\n"
-     "    Head revision:   981\n"
+     "    Status against revision:   981\n"
      "\n"
      "    svn status --show-updates --verbose wc\n"
      "     M           965       938 kfogel       wc/bar.c\n"
      "           *     965       922 sussman      wc/foo.c\n"
      "    A  +         965       687 joe          wc/qax.c\n"
      "                 965       687 joe          wc/zig.c\n"
-     "    Head revision:   981\n"),
+     "    Status against revision:   981\n"),
     { 'u', 'v', 'N', 'q', svn_cl__no_ignore_opt, SVN_CL__AUTH_OPTIONS, 
-      svn_cl__config_dir_opt} },
+      svn_cl__config_dir_opt, svn_cl__ignore_externals_opt} },
   
   { "switch", svn_cl__switch, {"sw"},
     N_("Update the working copy to a different URL.\n"
@@ -640,13 +696,21 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "  2. Rewrite working copy URL metadata to reflect a syntactic change "
        "only.\n"
        "     This is used when repository's root URL changes (such as a "
-       "schema\n"
+       "scheme\n"
        "     or hostname change) but your working copy still reflects the "
        "same\n"
        "     directory within the same repository.\n"),
     { 'r', 'N', 'q', svn_cl__merge_cmd_opt, svn_cl__relocate_opt,
       SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
  
+  { "unlock", svn_cl__unlock, {0},
+    N_("Unlock working copy paths or URLs.\n"
+       "usage: unlock TARGET...\n"
+       "\n"
+       "  Use --force to break the lock.\n"),
+    { svn_cl__targets_opt, SVN_CL__AUTH_OPTIONS,
+      svn_cl__config_dir_opt, svn_cl__force_opt } },
+
   { "update", svn_cl__update, {"up"}, 
     N_("Bring changes from the repository into the working copy.\n"
        "usage: update [PATH...]\n"
@@ -667,9 +731,12 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "  A character in the first column signifies an update to the actual "
        "file,\n"
        "  while updates to the file's properties are shown in the second "
-       "column.\n"),
+       "column.\n"
+       "  A 'B' in the third column signifies that the lock for the file has\n"
+       "  been broken or stolen.\n"
+       ),
     {'r', 'N', 'q', svn_cl__merge_cmd_opt, SVN_CL__AUTH_OPTIONS, 
-     svn_cl__config_dir_opt} },
+     svn_cl__config_dir_opt, svn_cl__ignore_externals_opt} },
 
   { NULL, NULL, {0}, NULL, {0} }
 };
@@ -698,11 +765,12 @@ check_lib_versions (void)
       { "svn_wc",     svn_wc_version },
       { "svn_ra",     svn_ra_version },
       { "svn_delta",  svn_delta_version },
+      { "svn_diff",   svn_diff_version },
       { NULL, NULL }
     };
 
-   SVN_VERSION_DEFINE (my_version);
-   return svn_ver_check_list (&my_version, checklist);
+  SVN_VERSION_DEFINE (my_version);
+  return svn_ver_check_list (&my_version, checklist);
 }
 
 
@@ -771,6 +839,11 @@ main (int argc, const char * const *argv)
   if (err)
     return error_exit (err, stderr, FALSE, pool);
 
+  /* Initialize the RA library. */
+  err = svn_ra_initialize (pool);
+  if (err)
+    return error_exit (err, stderr, FALSE, pool);
+
   /* Begin processing arguments. */
   opt_state.start_revision.kind = svn_opt_revision_unspecified;
   opt_state.end_revision.kind = svn_opt_revision_unspecified;
@@ -807,6 +880,22 @@ main (int argc, const char * const *argv)
       num_opts++;
 
       switch (opt_id) {
+      case svn_cl__limit_opt:
+        {
+          char *end;
+          opt_state.limit = strtol (opt_arg, &end, 10);
+          if (end == opt_arg || *end != '\0')
+            {
+              /* ### See r14699 on trunk for the error message that
+                     should be used here.  We removed it for 1.2.x in
+                     order to avoid slipping an untranslated string
+                     into the release at the last minute. */
+              err = svn_error_create (SVN_ERR_CL_ARG_PARSING_ERROR,
+                                      NULL, NULL);
+              return error_exit (err, stderr, FALSE, pool);
+            }
+        }
+        break;
       case 'm':
         /* Note that there's no way here to detect if the log message
            contains a zero byte -- if it does, then opt_arg will just
@@ -945,6 +1034,9 @@ main (int argc, const char * const *argv)
       case svn_cl__ignore_ancestry_opt:
         opt_state.ignore_ancestry = TRUE;
         break;
+      case svn_cl__ignore_externals_opt:
+        opt_state.ignore_externals = TRUE;
+        break;
       case svn_cl__relocate_opt:
         opt_state.relocate = TRUE;
         break;
@@ -1009,6 +1101,9 @@ main (int argc, const char * const *argv)
             svn_pool_destroy (pool);
             return EXIT_FAILURE;
           }
+      case svn_cl__no_unlock_opt:
+        opt_state.no_unlock = TRUE;
+        break;
       default:
         /* Hmmm. Perhaps this would be a good place to squirrel away
            opts that commands like svn diff might need. Hmmm indeed. */
@@ -1113,7 +1208,8 @@ main (int argc, const char * const *argv)
           || subcommand->cmd_func == svn_cl__delete
           || subcommand->cmd_func == svn_cl__import
           || subcommand->cmd_func == svn_cl__mkdir
-          || subcommand->cmd_func == svn_cl__move))
+          || subcommand->cmd_func == svn_cl__move
+          || subcommand->cmd_func == svn_cl__lock))
     {
       /* If the -F argument is a file that's under revision control,
          that's probably not what the user intended. */
@@ -1122,16 +1218,22 @@ main (int argc, const char * const *argv)
           svn_wc_adm_access_t *adm_access;
           const svn_wc_entry_t *e;
           const char *fname_utf8 = svn_path_internal_style (dash_F_arg, pool);
-          err = svn_wc_adm_probe_open2 (&adm_access, NULL, fname_utf8,
-                                        FALSE, 0, pool);
+          err = svn_wc_adm_probe_open3 (&adm_access, NULL, fname_utf8,
+                                        FALSE, 0, NULL, NULL, pool);
           if (! err)
             err = svn_wc_entry (&e, fname_utf8, adm_access, FALSE, pool);
           if ((err == SVN_NO_ERROR) && e)
             {
-              err = svn_error_create 
-                (SVN_ERR_CL_LOG_MESSAGE_IS_VERSIONED_FILE, NULL,
-                 _("Log message file is a versioned file; "
-                   "use '--force-log' to override"));
+              if (subcommand->cmd_func != svn_cl__lock)
+                err = svn_error_create 
+                  (SVN_ERR_CL_LOG_MESSAGE_IS_VERSIONED_FILE, NULL,
+                   _("Log message file is a versioned file; "
+                     "use '--force-log' to override"));
+              else
+                err = svn_error_create 
+                  (SVN_ERR_CL_LOG_MESSAGE_IS_VERSIONED_FILE, NULL,
+                   _("Lock comment file is a versioned file; "
+                     "use '--force-log' to override"));
               return error_exit (err, stderr, FALSE, pool);
             }
           if (err)
@@ -1146,10 +1248,16 @@ main (int argc, const char * const *argv)
           if (apr_stat (&finfo, dash_m_arg, 
                         APR_FINFO_MIN, pool) == APR_SUCCESS)
             {
-              err = svn_error_create 
-                (SVN_ERR_CL_LOG_MESSAGE_IS_PATHNAME, NULL,
-                 _("The log message is a pathname "
-                   "(was -F intended?); use '--force-log' to override"));
+              if (subcommand->cmd_func != svn_cl__lock)
+                err = svn_error_create 
+                  (SVN_ERR_CL_LOG_MESSAGE_IS_PATHNAME, NULL,
+                   _("The log message is a pathname "
+                     "(was -F intended?); use '--force-log' to override"));
+              else
+                err = svn_error_create 
+                  (SVN_ERR_CL_LOG_MESSAGE_IS_PATHNAME, NULL,
+                   _("The lock comment is a pathname "
+                     "(was -F intended?); use '--force-log' to override"));
               return error_exit (err, stderr, FALSE, pool);
             }
         }
@@ -1208,6 +1316,11 @@ main (int argc, const char * const *argv)
         }
     }
 
+  /* Update the 'keep-locks' runtime option */
+  if (opt_state.no_unlock)
+    svn_config_set_bool (cfg, SVN_CONFIG_SECTION_MISCELLANY,
+                         SVN_CONFIG_OPTION_NO_UNLOCK, TRUE);
+
   /* Set the log message callback function.  Note that individual
      subcommands will populate the ctx->log_msg_baton */
   ctx->log_msg_func = svn_cl__get_log_message;
@@ -1223,6 +1336,10 @@ main (int argc, const char * const *argv)
 
     /* The main disk-caching auth providers, for both
        'username/password' creds and 'username' creds.  */
+#ifdef WIN32
+    svn_client_get_windows_simple_provider (&provider, pool);
+    APR_ARRAY_PUSH (providers, svn_auth_provider_object_t *) = provider;
+#endif
     svn_client_get_simple_provider (&provider, pool);
     APR_ARRAY_PUSH (providers, svn_auth_provider_object_t *) = provider;
     svn_client_get_username_provider (&provider, pool);
