@@ -1,7 +1,7 @@
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -51,6 +51,8 @@
 extern "C" {
 #endif /* __cplusplus */
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+
 #if defined(SVN_ERROR_BUILD_ARRAY)
 
 #define SVN_ERROR_START \
@@ -92,8 +94,6 @@ extern "C" {
         perturbed.
 */
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
-
 #define SVN_ERR_CATEGORY_SIZE 5000
 
 /* Leave one category of room at the beginning, for SVN_WARNING and
@@ -134,11 +134,15 @@ extern "C" {
                                            + (17 * SVN_ERR_CATEGORY_SIZE))
 #define SVN_ERR_RA_SVN_CATEGORY_START   (APR_OS_START_USERERR \
                                            + (18 * SVN_ERR_CATEGORY_SIZE))
-#define SVN_ERR_AUTH_CATEGORY_START     (APR_OS_START_USERERR \
+#define SVN_ERR_AUTHN_CATEGORY_START    (APR_OS_START_USERERR \
                                            + (19 * SVN_ERR_CATEGORY_SIZE))
+#define SVN_ERR_AUTHZ_CATEGORY_START    (APR_OS_START_USERERR \
+                                           + (20 * SVN_ERR_CATEGORY_SIZE))
 
 #endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
+/** Collection of Subversion error code values, located within the
+ * APR user error space. */
 SVN_ERROR_START
 
   /* validation ("BAD_FOO") errors */
@@ -187,6 +191,10 @@ SVN_ERROR_START
               SVN_ERR_XML_CATEGORY_START + 3,
               "XML data was not well-formed")
 
+  SVN_ERRDEF (SVN_ERR_XML_UNESCAPABLE_DATA,
+              SVN_ERR_XML_CATEGORY_START + 4,
+              "Data cannot be safely XML-escaped")
+
   /* io errors */
 
   SVN_ERRDEF (SVN_ERR_IO_INCONSISTENT_EOL,
@@ -197,6 +205,7 @@ SVN_ERROR_START
               SVN_ERR_IO_CATEGORY_START + 1,
               "Unrecognized line ending style")
 
+  /* @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_IO_CORRUPT_EOL,
               SVN_ERR_IO_CATEGORY_START + 2,
               "Line endings other than expected")
@@ -205,14 +214,20 @@ SVN_ERROR_START
               SVN_ERR_IO_CATEGORY_START + 3,
               "Ran out of unique names")
 
+  /* @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_IO_PIPE_FRAME_ERROR,
               SVN_ERR_IO_CATEGORY_START + 4,
               "Framing error in pipe protocol")
 
+  /* @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_IO_PIPE_READ_ERROR,
               SVN_ERR_IO_CATEGORY_START + 5,
               "Read error in pipe")
 
+  SVN_ERRDEF (SVN_ERR_IO_WRITE_ERROR,
+              SVN_ERR_IO_CATEGORY_START + 6,
+              "Write error")
+
   /* stream errors */
 
   SVN_ERRDEF (SVN_ERR_STREAM_UNEXPECTED_EOF,
@@ -255,7 +270,7 @@ SVN_ERROR_START
 
   SVN_ERRDEF (SVN_ERR_ENTRY_MISSING_URL,
               SVN_ERR_ENTRY_CATEGORY_START + 4,
-              "Entry has no url")
+              "Entry has no URL")
 
   SVN_ERRDEF (SVN_ERR_ENTRY_ATTRIBUTE_INVALID,
               SVN_ERR_ENTRY_CATEGORY_START + 5,
@@ -267,14 +282,17 @@ SVN_ERROR_START
               SVN_ERR_WC_CATEGORY_START + 0,
               "Obstructed update")
 
+  /* @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_WC_UNWIND_MISMATCH,
               SVN_ERR_WC_CATEGORY_START + 1,
               "Mismatch popping the WC unwind stack")
 
+  /* @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_WC_UNWIND_EMPTY,
               SVN_ERR_WC_CATEGORY_START + 2,
               "Attempt to pop empty WC unwind stack")
 
+  /* @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_WC_UNWIND_NOT_EMPTY,
               SVN_ERR_WC_CATEGORY_START + 3,
               "Attempt to unlock with non-empty unwind stack")
@@ -285,8 +303,9 @@ SVN_ERROR_START
 
   SVN_ERRDEF (SVN_ERR_WC_NOT_LOCKED,
               SVN_ERR_WC_CATEGORY_START + 5,
-              "Working copy not locked")
+              "Working copy not locked; this is probably a bug, please report")
 
+  /* @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_WC_INVALID_LOCK,
               SVN_ERR_WC_CATEGORY_START + 6,
               "Invalid lock")
@@ -321,7 +340,7 @@ SVN_ERROR_START
 
   SVN_ERRDEF (SVN_ERR_WC_PATH_FOUND,
               SVN_ERR_WC_CATEGORY_START + 14,
-              "Found a working copy path.")
+              "Found a working copy path")
 
   SVN_ERRDEF (SVN_ERR_WC_FOUND_CONFLICT,
               SVN_ERR_WC_CATEGORY_START + 15,
@@ -339,6 +358,27 @@ SVN_ERROR_START
               SVN_ERR_WC_CATEGORY_START + 18,
               "Cannot change node kind")
 
+  SVN_ERRDEF (SVN_ERR_WC_INVALID_OP_ON_CWD,
+              SVN_ERR_WC_CATEGORY_START + 19,
+              "Invalid operation on the current working directory")  
+
+  SVN_ERRDEF (SVN_ERR_WC_BAD_ADM_LOG_START,
+              SVN_ERR_WC_CATEGORY_START + 20,
+              "Problem on first log entry in a working copy")
+
+  SVN_ERRDEF (SVN_ERR_WC_UNSUPPORTED_FORMAT,
+              SVN_ERR_WC_CATEGORY_START + 21,
+              "Unsupported working copy format")  
+
+  SVN_ERRDEF (SVN_ERR_WC_BAD_PATH,
+              SVN_ERR_WC_CATEGORY_START + 22,
+              "Path syntax not supported in this context")  
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_WC_INVALID_SCHEDULE,
+              SVN_ERR_WC_CATEGORY_START + 23,
+              "Invalid schedule")  
+
   /* fs errors */
 
   SVN_ERRDEF (SVN_ERR_FS_GENERAL,
@@ -443,11 +483,11 @@ SVN_ERROR_START
 
   SVN_ERRDEF (SVN_ERR_FS_REP_CHANGED,
               SVN_ERR_FS_CATEGORY_START + 25,
-              "A representation vanished or changed between reads.")
+              "A representation vanished or changed between reads")
 
   SVN_ERRDEF (SVN_ERR_FS_REP_NOT_MUTABLE,
               SVN_ERR_FS_CATEGORY_START + 26,
-              "Tried to change an immutable representation.")
+              "Tried to change an immutable representation")
 
   SVN_ERRDEF (SVN_ERR_FS_MALFORMED_SKEL,
               SVN_ERR_FS_CATEGORY_START + 27,
@@ -465,28 +505,103 @@ SVN_ERROR_START
               SVN_ERR_FS_CATEGORY_START + 30,
               "Berkeley DB deadlock error")
 
+  SVN_ERRDEF (SVN_ERR_FS_TRANSACTION_DEAD,
+              SVN_ERR_FS_CATEGORY_START + 31,
+              "Transaction is dead")
+
+  SVN_ERRDEF (SVN_ERR_FS_TRANSACTION_NOT_DEAD,
+              SVN_ERR_FS_CATEGORY_START + 32,
+              "Transaction is not dead")
+
+  /* @since New in 1.1. */
+  SVN_ERRDEF (SVN_ERR_FS_UNKNOWN_FS_TYPE,
+              SVN_ERR_FS_CATEGORY_START + 33,
+              "Unknown FS type")
+
+  /* @since New in 1.2 */
+  SVN_ERRDEF (SVN_ERR_FS_NO_USER,
+              SVN_ERR_FS_CATEGORY_START + 34,
+              "No user associated with filesystem")
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_PATH_ALREADY_LOCKED,
+              SVN_ERR_FS_CATEGORY_START + 35,
+              "Path is already locked")
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_PATH_NOT_LOCKED,
+              SVN_ERR_FS_CATEGORY_START + 36,
+              "Path is not locked")
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_BAD_LOCK_TOKEN,
+              SVN_ERR_FS_CATEGORY_START + 37,
+              "Lock token is incorrect")
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_NO_LOCK_TOKEN,
+              SVN_ERR_FS_CATEGORY_START + 38,
+              "No lock token provided")
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_LOCK_OWNER_MISMATCH,
+              SVN_ERR_FS_CATEGORY_START + 39,
+              "Username does not match lock owner")
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_NO_SUCH_LOCK,
+              SVN_ERR_FS_CATEGORY_START + 40,
+              "Filesystem has no such lock")
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_LOCK_EXPIRED,
+              SVN_ERR_FS_CATEGORY_START + 41,
+              "Lock has expired")
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_OUT_OF_DATE,
+              SVN_ERR_FS_CATEGORY_START + 42,
+              "Item is out of date")
+
+  /* @since New in 1.2.
+   *
+   * This is analogous to SVN_ERR_REPOS_UNSUPPORTED_VERSION.  To avoid
+   * confusion with "versions" (i.e., releases) of Subversion, we've
+   * started calling this the "format" number instead.  The old
+   * SVN_ERR_REPOS_UNSUPPORTED_VERSION error predates this and so
+   * retains its name.
+   */
+  SVN_ERRDEF (SVN_ERR_FS_UNSUPPORTED_FORMAT,
+              SVN_ERR_FS_CATEGORY_START + 43,
+              "Unsupported FS format")
+
   /* repos errors */
 
   SVN_ERRDEF (SVN_ERR_REPOS_LOCKED,
               SVN_ERR_REPOS_CATEGORY_START + 0,
-              "The repository is locked, perhaps for db recovery.")
+              "The repository is locked, perhaps for db recovery")
 
   SVN_ERRDEF (SVN_ERR_REPOS_HOOK_FAILURE,
               SVN_ERR_REPOS_CATEGORY_START + 1,
-              "A repository hook failed.")
+              "A repository hook failed")
 
   SVN_ERRDEF (SVN_ERR_REPOS_BAD_ARGS,
               SVN_ERR_REPOS_CATEGORY_START + 2,
-              "Incorrect arguments supplied.")
+              "Incorrect arguments supplied")
 
   SVN_ERRDEF (SVN_ERR_REPOS_NO_DATA_FOR_REPORT,
               SVN_ERR_REPOS_CATEGORY_START + 3,
-              "A report cannot be generated because no data was supplied.")
+              "A report cannot be generated because no data was supplied")
 
   SVN_ERRDEF (SVN_ERR_REPOS_BAD_REVISION_REPORT,
               SVN_ERR_REPOS_CATEGORY_START + 4,
               "Bogus revision report")
  
+  /* This is analogous to SVN_ERR_FS_UNSUPPORTED_FORMAT.  To avoid
+   * confusion with "versions" (i.e., releases) of Subversion, we
+   * started using the word "format" instead of "version".  However,
+   * this error code's name predates that decision.
+   */
   SVN_ERRDEF (SVN_ERR_REPOS_UNSUPPORTED_VERSION,
               SVN_ERR_REPOS_CATEGORY_START + 5,
               "Unsupported repository version")
@@ -495,7 +610,22 @@ SVN_ERROR_START
               SVN_ERR_REPOS_CATEGORY_START + 6,
               "Disabled repository feature")
 
-  /* generic ra errors */
+  SVN_ERRDEF (SVN_ERR_REPOS_POST_COMMIT_HOOK_FAILED,
+              SVN_ERR_REPOS_CATEGORY_START + 7,
+              "Error running post-commit hook")
+
+  /* @since New in 1.2 */
+  SVN_ERRDEF (SVN_ERR_REPOS_POST_LOCK_HOOK_FAILED,
+              SVN_ERR_REPOS_CATEGORY_START + 8,
+              "Error running post-lock hook")
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_REPOS_POST_UNLOCK_HOOK_FAILED,
+              SVN_ERR_REPOS_CATEGORY_START + 9,
+              "Error running post-unlock hook")
+
+
+  /* generic RA errors */
 
   SVN_ERRDEF (SVN_ERR_RA_ILLEGAL_URL,
               SVN_ERR_RA_CATEGORY_START + 0,
@@ -515,12 +645,22 @@ SVN_ERROR_START
 
   SVN_ERRDEF (SVN_ERR_RA_OUT_OF_DATE,
               SVN_ERR_RA_CATEGORY_START + 4,
-              "Item is out-of-date.")
+              "Item is out-of-date")
 
   SVN_ERRDEF (SVN_ERR_RA_NO_REPOS_UUID,
               SVN_ERR_RA_CATEGORY_START + 5,
-              "Repository has no UUID.")
-      
+              "Repository has no UUID")
+
+  SVN_ERRDEF (SVN_ERR_RA_UNSUPPORTED_ABI_VERSION,
+              SVN_ERR_RA_CATEGORY_START + 6,
+              "Unsupported RA plugin ABI version")
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_RA_NOT_LOCKED,
+              SVN_ERR_RA_CATEGORY_START + 7,
+              "Path is not locked")
+
+
   /* ra_dav errors */
 
   SVN_ERRDEF (SVN_ERR_RA_DAV_SOCK_INIT,
@@ -551,15 +691,29 @@ SVN_ERROR_START
               SVN_ERR_RA_DAV_CATEGORY_START + 6,
               "Invalid configuration value")
 
+  SVN_ERRDEF (SVN_ERR_RA_DAV_PATH_NOT_FOUND,
+              SVN_ERR_RA_DAV_CATEGORY_START + 7,
+              "HTTP Path Not Found")
+
+  SVN_ERRDEF (SVN_ERR_RA_DAV_PROPPATCH_FAILED,
+              SVN_ERR_RA_DAV_CATEGORY_START + 8,
+              "Failed to execute WebDAV PROPPATCH")
+
+  /* @since New in 1.2 */
+  SVN_ERRDEF (SVN_ERR_RA_DAV_MALFORMED_DATA,
+              SVN_ERR_RA_DAV_CATEGORY_START + 9,
+              "Malformed network data")
+
+
   /* ra_local errors */
   
   SVN_ERRDEF (SVN_ERR_RA_LOCAL_REPOS_NOT_FOUND,
               SVN_ERR_RA_LOCAL_CATEGORY_START + 0,
-              "Couldn't find a repository.")
+              "Couldn't find a repository")
        
   SVN_ERRDEF (SVN_ERR_RA_LOCAL_REPOS_OPEN_FAILED,
               SVN_ERR_RA_LOCAL_CATEGORY_START + 1,
-              "Couldn't open a repository.")
+              "Couldn't open a repository")
   /* ra_svn errors */
 
   SVN_ERRDEF (SVN_ERR_RA_SVN_CMD_ERR,
@@ -594,22 +748,39 @@ SVN_ERROR_START
 
        /* this error can be used when an auth provider doesn't have
           the creds, but no other "real" error occurred. */
-  SVN_ERRDEF (SVN_ERR_AUTH_CREDS_UNAVAILABLE,
-              SVN_ERR_AUTH_CATEGORY_START + 0,
+  SVN_ERRDEF (SVN_ERR_AUTHN_CREDS_UNAVAILABLE,
+              SVN_ERR_AUTHN_CATEGORY_START + 0,
               "Credential data unavailable")
 
-  SVN_ERRDEF (SVN_ERR_AUTH_NO_PROVIDER,
-              SVN_ERR_AUTH_CATEGORY_START + 1,
+  SVN_ERRDEF (SVN_ERR_AUTHN_NO_PROVIDER,
+              SVN_ERR_AUTHN_CATEGORY_START + 1,
               "No authentication provider available")
 
-  SVN_ERRDEF (SVN_ERR_AUTH_PROVIDERS_EXHAUSTED,
-              SVN_ERR_AUTH_CATEGORY_START + 2,
+  SVN_ERRDEF (SVN_ERR_AUTHN_PROVIDERS_EXHAUSTED,
+              SVN_ERR_AUTHN_CATEGORY_START + 2,
               "All authentication providers exhausted")
 
-  SVN_ERRDEF (SVN_ERR_AUTH_CREDS_NOT_SAVED,
-              SVN_ERR_AUTH_CATEGORY_START + 3,
+  SVN_ERRDEF (SVN_ERR_AUTHN_CREDS_NOT_SAVED,
+              SVN_ERR_AUTHN_CATEGORY_START + 3,
               "All authentication providers exhausted")
 
+  /* authorization errors */
+
+  SVN_ERRDEF (SVN_ERR_AUTHZ_ROOT_UNREADABLE,
+              SVN_ERR_AUTHZ_CATEGORY_START + 0,
+              "Read access denied for root of edit")
+
+  /* @since New in 1.1. */
+  SVN_ERRDEF (SVN_ERR_AUTHZ_UNREADABLE,
+              SVN_ERR_AUTHZ_CATEGORY_START + 1,
+              "Item is not readable")
+
+  /* @since New in 1.1. */
+  SVN_ERRDEF (SVN_ERR_AUTHZ_PARTIALLY_READABLE,
+              SVN_ERR_AUTHZ_CATEGORY_START + 2,
+              "Item is partially readable")
+
+
   /* svndiff errors */
 
   SVN_ERRDEF (SVN_ERR_SVNDIFF_INVALID_HEADER,
@@ -650,11 +821,15 @@ SVN_ERROR_START
               SVN_ERR_APMOD_CATEGORY_START + 3,
               "Baseline incorrect")
 
+  SVN_ERRDEF (SVN_ERR_APMOD_CONNECTION_ABORTED,
+              SVN_ERR_APMOD_CATEGORY_START + 4,
+              "Input/output error")
+
   /* libsvn_client errors */
 
   SVN_ERRDEF (SVN_ERR_CLIENT_VERSIONED_PATH_REQUIRED,
               SVN_ERR_CLIENT_CATEGORY_START + 0,
-              "A path under revision control is needed for this operation")
+              "A path under version control is needed for this operation")
 
   SVN_ERRDEF (SVN_ERR_CLIENT_RA_ACCESS_REQUIRED,
               SVN_ERR_CLIENT_CATEGORY_START + 1,
@@ -668,13 +843,15 @@ SVN_ERROR_START
               SVN_ERR_CLIENT_CATEGORY_START + 3,
               "Attempting to commit to a URL more than once")
 
-  SVN_ERRDEF (SVN_ERR_CLIENT_UNVERSIONED,
+  SVN_ERRDEF (SVN_ERR_CLIENT_IS_BINARY_FILE,
               SVN_ERR_CLIENT_CATEGORY_START + 4,
-              "Attempting restricted operation for unversioned resource")
+              "Operation does not apply to binary file")
 
+       /*### SVN_PROP_EXTERNALS needed to be replaced with "svn:externals"
+         in order to get gettext translatable strings */
   SVN_ERRDEF (SVN_ERR_CLIENT_INVALID_EXTERNALS_DESCRIPTION,
               SVN_ERR_CLIENT_CATEGORY_START + 5,
-              "Format of an " SVN_PROP_EXTERNALS " property was invalid.")
+              "Format of an svn:externals property was invalid")
 
   SVN_ERRDEF (SVN_ERR_CLIENT_MODIFIED,
               SVN_ERR_CLIENT_CATEGORY_START + 6,
@@ -692,6 +869,24 @@ SVN_ERROR_START
               SVN_ERR_CLIENT_CATEGORY_START + 9,
               "Inter-repository relocation not allowed")
 
+  SVN_ERRDEF (SVN_ERR_CLIENT_REVISION_AUTHOR_CONTAINS_NEWLINE,
+              SVN_ERR_CLIENT_CATEGORY_START + 10,
+              "Author name cannot contain a newline")
+
+  SVN_ERRDEF (SVN_ERR_CLIENT_PROPERTY_NAME,
+              SVN_ERR_CLIENT_CATEGORY_START + 11,
+              "Bad property name")
+
+  /* @since New in 1.1. */
+  SVN_ERRDEF (SVN_ERR_CLIENT_UNRELATED_RESOURCES,
+              SVN_ERR_CLIENT_CATEGORY_START + 12,
+              "Two versioned resources are unrelated")
+
+  /* @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_CLIENT_MISSING_LOCK_TOKEN,
+              SVN_ERR_CLIENT_CATEGORY_START + 13,
+              "Path has no lock token")
+
   /* misc errors */
 
   SVN_ERRDEF (SVN_ERR_BASE,
@@ -766,6 +961,15 @@ SVN_ERROR_START
               SVN_ERR_MISC_CATEGORY_START + 17,
               "Property not found")
 
+  SVN_ERRDEF (SVN_ERR_NO_AUTH_FILE_PATH,
+              SVN_ERR_MISC_CATEGORY_START + 18,
+              "No auth file path available")
+
+  /* @since New in 1.1. */
+  SVN_ERRDEF (SVN_ERR_VERSION_MISMATCH,
+              SVN_ERR_MISC_CATEGORY_START + 19,
+              "Incompatible library version")
+
   /* command-line client errors */
 
   SVN_ERRDEF (SVN_ERR_CL_ARG_PARSING_ERROR,
@@ -778,7 +982,7 @@ SVN_ERROR_START
 
   SVN_ERRDEF (SVN_ERR_CL_MUTUALLY_EXCLUSIVE_ARGS,
               SVN_ERR_CL_CATEGORY_START + 2,
-              "Mutually exclusive arguments specified.")                   
+              "Mutually exclusive arguments specified")
 
   SVN_ERRDEF (SVN_ERR_CL_ADM_DIR_RESERVED,
               SVN_ERR_CL_CATEGORY_START + 3,
@@ -802,7 +1006,7 @@ SVN_ERROR_START
 
   SVN_ERRDEF (SVN_ERR_CL_BAD_LOG_MESSAGE,
               SVN_ERR_CL_CATEGORY_START + 8,
-              "Something is wrong with the log message's contents.")
+              "Something is wrong with the log message's contents")
 
 SVN_ERROR_END
 
