@@ -2,7 +2,7 @@
  * client.h :  shared stuff internal to the client library.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -37,21 +37,21 @@ extern "C" {
 /* Set *REVNUM to the revision number identified by REVISION.
  *
  * If REVISION->kind is svn_opt_revision_number, just use
- * REVISION->value.number, ignoring PATH, RA_LIB, and SESSION.
+ * REVISION->value.number, ignoring PATH and RA_SESSION.
  *
  * Else if REVISION->kind is svn_opt_revision_committed,
  * svn_opt_revision_previous, or svn_opt_revision_base, or
  * svn_opt_revision_working, then the revision can be identified
  * purely based on the working copy's administrative information for
- * PATH, so RA_LIB and SESSION are ignored.  If PATH is not under
- * revision control, return SVN_ERR_UNVERSIONED_RESOURCE, or if PATH
- * is null, return SVN_ERR_CLIENT_VERSIONED_PATH_REQUIRED.
+ * PATH, so RA_SESSION is ignored.  If PATH is not under revision
+ * control, return SVN_ERR_UNVERSIONED_RESOURCE, or if PATH is null,
+ * return SVN_ERR_CLIENT_VERSIONED_PATH_REQUIRED.
  * 
  * Else if REVISION->kind is svn_opt_revision_date or
- * svn_opt_revision_head, then RA_LIB and SESSION are used to
- * retrieve the revision from the repository (using
- * REVISION->value.date in the former case), and PATH is ignored.  If
- * RA_LIB or SESSION is null, return SVN_ERR_CLIENT_RA_ACCESS_REQUIRED. 
+ * svn_opt_revision_head, then RA_SESSION is used to retrieve the
+ * revision from the repository (using REVISION->value.date in the
+ * former case), and PATH is ignored.  If RA_SESSION is null,
+ * return SVN_ERR_CLIENT_RA_ACCESS_REQUIRED. 
  *
  * Else if REVISION->kind is svn_opt_revision_unspecified, set
  * *REVNUM to SVN_INVALID_REVNUM.  
@@ -62,8 +62,7 @@ extern "C" {
  */
 svn_error_t *
 svn_client__get_revision_number (svn_revnum_t *revnum,
-                                 svn_ra_plugin_t *ra_lib,
-                                 void *session,
+                                 svn_ra_session_t *ra_session,
                                  const svn_opt_revision_t *revision,
                                  const char *path,
                                  apr_pool_t *pool);
@@ -78,16 +77,103 @@ svn_client__compare_revisions (svn_opt_revision_t *revision1,
 
 
 /* Return true if the revision number for REVISION can be determined
- * from just the working copy, or false if it can be determined from
- * just the repository.
- *
- * Note: No other kinds of revisions should be possible; but if one
- * day there are, this will return true for those kinds.
+   from just the working copy, or false if it can be determined from
+   just the repository.
+ 
+   NOTE: No other kinds of revisions should be possible; but if one
+   day there are, this will return true for those kinds.
  */ 
-svn_boolean_t
+svn_boolean_t 
 svn_client__revision_is_local (const svn_opt_revision_t *revision);
 
 
+/* Given the CHANGED_PATHS and REVISION from an instance of a
+   svn_log_message_receiver_t function, determine at which location
+   PATH may be expected in the next log message, and set *PREV_PATH_P
+   to that value.  KIND is the node kind of PATH.  Set *ACTION_P to a
+   character describing the change that caused this revision (as
+   listed in svn_log_changed_path_t) and set *COPYFROM_REV_P to the
+   revision PATH was copied from, or SVN_INVALID_REVNUM if it was not
+   copied.  ACTION_P and COPYFROM_REV_P may be NULL, in which case
+   they are not used.  Perform all allocations in POOL.
+
+   This is useful for tracking the various changes in location a
+   particular resource has undergone when performing an RA->get_logs()
+   operation on that resource.  */
+svn_error_t *svn_client__prev_log_path (const char **prev_path_p,
+                                        char *action_p,
+                                        svn_revnum_t *copyfrom_rev_p,
+                                        apr_hash_t *changed_paths,
+                                        const char *path,
+                                        svn_node_kind_t kind,
+                                        svn_revnum_t revision,
+                                        apr_pool_t *pool);
+
+
+/** Set @a *start_url and @a *start_revision (and maybe @a *end_url
+ * and @a *end_revision) to the revisions and repository URLs of one
+ * (or two) points of interest along a particular versioned resource's
+ * line of history.  @a path as it exists in "peg revision" @a
+ * revision identifies that line of history, and @a start and @a end
+ * specify the point(s) of interest (typically the revisions referred
+ * to as the "operative range" for a given operation) along that history.
+ *
+ * @a end may be of kind svn_opt_revision_unspecified (in which case
+ * @a end_url and @a end_revision are not touched by the function);
+ * @a start and @a revision may not.
+ *
+ * A NOTE ABOUT FUTURE REPORTING:
+ *
+ * If either @a start or @end are greater than @a revision, then do a
+ * sanity check (since we cannot search future history yet): verify
+ * that @a path in the future revision(s) is the "same object" as the
+ * one pegged by @a revision.  In other words, all three objects must
+ * be connected by a single line of history which exactly passes
+ * through @a path at @a revision.  If this sanity check fails, return
+ * SVN_ERR_CLIENT_UNRELATED_RESOURCES.
+ *
+ * @a ctx is the client context baton.
+ *
+ * Use @a pool for all allocations.
+ */
+svn_error_t *
+svn_client__repos_locations (const char **start_url,
+                             svn_opt_revision_t **start_revision,
+                             const char **end_url,
+                             svn_opt_revision_t **end_revision,
+                             const char *path,
+                             const svn_opt_revision_t *revision,
+                             const svn_opt_revision_t *start,
+                             const svn_opt_revision_t *end,
+                             svn_client_ctx_t *ctx,
+                             apr_pool_t *pool);
+
+
+/* Given PATH_OR_URL, which contains either a working copy path or an
+   absolute url, a peg revision PEG_REVISON, and a desired revision
+   REVISION, create an RA connection to that object as it exists in
+   that revision, following copy history if necessary.  If REVISION is
+   younger than PEG_REVISION, then PATH_OR_URL will be checked to see
+   that it is the same node in both PEG_REVISION and REVISON.  If it
+   is not, then @c SVN_ERR_CLIENT_UNRELATED_RESOURCES is returned.
+
+   Store the resulting ra_session in *RA_SESSION_P.  Store the actual
+   revision number of the object in *REV_P, and the final resulting
+   url in *URL_P.
+
+   Use authentication baton cached in CTX to authenticate against the
+   repository.
+
+   Use POOL for all allocations. */
+svn_error_t *
+svn_client__ra_session_from_path (svn_ra_session_t **ra_session_p,
+                                  svn_revnum_t *rev_p,
+                                  const char **url_p,
+                                  const char *path_or_url,
+                                  const svn_opt_revision_t *peg_revision,
+                                  const svn_opt_revision_t *revision,
+                                  svn_client_ctx_t *ctx,
+                                  apr_pool_t *pool);
 
 
 /* ---------------------------------------------------------------- */
@@ -109,9 +195,8 @@ typedef struct
      during working copy commits. */
   apr_array_header_t *commit_items;
 
-  /* A hash of svn_config_t's, keyed off file name (i.e. the contents of 
-     ~/.subversion/config end up keyed off of 'config'). */
-  apr_hash_t *config;
+  /* A client context. */
+  svn_client_ctx_t *ctx;
 
   /* The pool to use for session-related items. */
   apr_pool_t *pool;
@@ -119,8 +204,7 @@ typedef struct
 } svn_client__callback_baton_t;
 
 
-/* Open an RA session, returning the session baton in SESSION_BATON. The
-   RA library to use is specified by RA_LIB.
+/* Open an RA session, returning it in *RA_SESSION.
 
    The root of the session is specified by BASE_URL and BASE_DIR.
    BASE_ACCESS is an access baton for BASE_DIR administrative data.
@@ -131,9 +215,10 @@ typedef struct
         structures, present only for working copy commits, NULL otherwise.
 
       - USE_ADMIN indicates that the RA layer should create tempfiles
-        in the administrative area instead of in the working copy itself.
+        in the administrative area instead of in the working copy itself,
+        and read properties from the administrative area.
 
-      - READ_ONLY_WC indicates that the RA layer should not attempt
+      - READ_ONLY_WC indicates that the RA layer should not attempt to
         modify the WC props directly.
 
    BASE_DIR may be NULL if the RA operation does not correspond to a
@@ -142,8 +227,7 @@ typedef struct
 
    The calling application's authentication baton is provided in CTX,
    and allocations related to this session are performed in POOL.  */
-svn_error_t * svn_client__open_ra_session (void **session_baton,
-                                           const svn_ra_plugin_t *ra_lib,
+svn_error_t * svn_client__open_ra_session (svn_ra_session_t **ra_session,
                                            const char *base_url,
                                            const char *base_dir,
                                            svn_wc_adm_access_t *base_access,
@@ -154,102 +238,68 @@ svn_error_t * svn_client__open_ra_session (void **session_baton,
                                            apr_pool_t *pool);
 
 
-/* Set *DIR_P to DIR if DIR is a working copy directory, else set to NULL.
- * DIR may not be a file.  Use POOL only for temporary allocation.
- *
- * Purpose: Helper for callers of svn_client__open_ra_session(),
- * who if not passed a working copy path as an argument, will often
- * wish to try the current directory for auth information, but only if
- * it is a working copy.
- */
-svn_error_t *svn_client__dir_if_wc (const char **dir_p,
-                                    const char *dir,
-                                    apr_pool_t *pool);
-
-
-/* Set *AUTH_DIR_P to PATH if PATH is a working copy directory, else
- * to PATH's parent if the parent is a working copy directory, else to
- * null.
- *
- * If set *AUTH_DIR_P to PATH's parent, allocate *AUTH_DIR_P in POOL;
- * otherwise, use POOL only for temporary allocation.
- *
- * Purpose: similar to svn_client__dir_if_wc().
- */
-svn_error_t *svn_client__default_auth_dir (const char **auth_dir_p,
-                                           const char *path,
-                                           apr_pool_t *pool);
-
-
 
 /* ---------------------------------------------------------------- */
 
 /*** Commit ***/
 
-/* If REVISION or AUTHOR or DATE has a valid value, then allocate (in
-   POOL) an svn_client_commit_info_t structure and populate it with
-   those values (that is, copies of them allocated in POOL).  */
-svn_client_commit_info_t *svn_client__make_commit_info (svn_revnum_t revision,
-                                                        const char *author,
-                                                        const char *date,
-                                                        apr_pool_t *pool);
+/* Get the commit_baton to be used in couple with commit_callback. */
+svn_error_t *svn_client__commit_get_baton (void **baton,
+                                           svn_client_commit_info_t **info,
+                                           apr_pool_t *pool);
+
+/* The commit_callback function for storing svn_client_commit_info_t
+   pointed by commit_baton. If the commit_info supplied by get_baton
+   points to NULL after close_edit, it means the commit is a no-op.
+*/
+svn_error_t *svn_client__commit_callback (svn_revnum_t revision,
+                                          const char *date,
+                                          const char *author,
+                                          void *baton);
 
 /* ---------------------------------------------------------------- */
 
 /*** Status ***/
 
-/* Verify that the path can be deleted without losing stuff, i.e. ensure
-   that there are no modified or unversioned resources under PATH.  This is
-   similar to checking the output of the status command.  ADM_ACCESS should 
-   be for the directory PATH is in, or for PATH if it is a directory.  CTX 
-   is used for the client's config options.  POOL is used for all temporary 
-   allocations. */
+/* Verify that the path can be deleted without losing stuff,
+   i.e. ensure that there are no modified or unversioned resources
+   under PATH.  This is similar to checking the output of the status
+   command.  CTX is used for the client's config options.  POOL is
+   used for all temporary allocations. */
 svn_error_t * svn_client__can_delete (const char *path,
-                                      svn_wc_adm_access_t *adm_access,
                                       svn_client_ctx_t *ctx,
                                       apr_pool_t *pool);
 
 
-/* ---------------------------------------------------------------- */
-
-/*** Export ***/
-
-/* Set *EDITOR and *EDIT_BATON to an editor (allocated in POOL) that
-   simply dumps data to disk, with no working copy administrative dirs
-   or bookkeeping.  Create the directory ROOT_PATH, and dump the
-   entire tree within.  ROOT_URL must be the the url being exported,
-   so that the 'url' keyword can be expanded.  Use CTX (if non-NULL)
-   for sending feedback. */
-svn_error_t * svn_client__get_export_editor (const svn_delta_editor_t **editor,
-                                             void **edit_baton,
-                                             const char *root_path,
-                                             const char *root_url,
-                                             svn_client_ctx_t *ctx,
-                                             apr_pool_t *pool);
-
-
 /* ---------------------------------------------------------------- */
 
 /*** Add/delete ***/
 
-/* The main logic of the public svn_client_add;  the only difference
-   is that this function uses an existing access baton.
-   (svn_client_add just generates an access baton and calls this func.) */
-svn_error_t * svn_client__add (const char *path, 
-                               svn_boolean_t recursive,
-                               svn_wc_adm_access_t *adm_access,
-                               svn_client_ctx_t *ctx,
-                               apr_pool_t *pool);
+/* Read automatic properties matching PATH from CTX->config.
+   Set *PROPERTIES to a hash containing propname/value pairs
+   (const char * keys mapping to svn_string_t * values), or if
+   auto-props are disabled, set *PROPERTIES to NULL.
+   Set *MIMETYPE to the mimetype, if any, or to NULL.
+   Allocate the hash table, keys, values, and mimetype in POOL. */
+svn_error_t *svn_client__get_auto_props (apr_hash_t **properties,
+                                         const char **mimetype,
+                                         const char *path,
+                                         svn_client_ctx_t *ctx,
+                                         apr_pool_t *pool);
+                            
 
 /* The main logic for client deletion from a working copy. Deletes PATH
    from ADM_ACCESS.  If PATH (or any item below a directory PATH) is
    modified the delete will fail and return an error unless FORCE is TRUE.
    If DRY_RUN is TRUE all the checks are made to ensure that the delete can
-   occur, but the working copy is not modifed. */
+   occur, but the working copy is not modifed.  If NOTIFY_FUNC is not
+   null, it is called with NOTIFY_BATON for each file or directory deleted. */
 svn_error_t * svn_client__wc_delete (const char *path,
                                      svn_wc_adm_access_t *adm_access,
                                      svn_boolean_t force,
                                      svn_boolean_t dry_run,
+                                     svn_wc_notify_func2_t notify_func,
+                                     void *notify_baton,
                                      svn_client_ctx_t *ctx,
                                      apr_pool_t *pool);
 
@@ -257,30 +307,39 @@ svn_error_t * svn_client__wc_delete (const char *path,
 
 /*** Checkout and update ***/
 
-/* Update a working copy PATH to REVISION.  If TIMESTAMP_SLEEP is NULL this
-   function will sleep before returning to ensure timestamp integrity.  If
-   TIMESTAMP_SLEEP is not NULL then the function will not sleep but will
-   set *TIMESTAMP_SLEEP to TRUE if a sleep is required, and will not change
-   *TIMESTAMP_SLEEP if no sleep is required. */
+/* Update a working copy PATH to REVISION, and (if not NULL) set
+   RESULT_REV to the update revision.  RECURSE if so commanded;
+   likewise, possibly IGNORE_EXTERNALS.  If TIMESTAMP_SLEEP is NULL this
+   function will sleep before returning to ensure timestamp integrity.
+   If TIMESTAMP_SLEEP is not NULL then the function will not sleep but
+   will set *TIMESTAMP_SLEEP to TRUE if a sleep is required, and will
+   not change *TIMESTAMP_SLEEP if no sleep is required. */
 svn_error_t *
-svn_client__update_internal (const char *path,
+svn_client__update_internal (svn_revnum_t *result_rev,
+                             const char *path,
                              const svn_opt_revision_t *revision,
                              svn_boolean_t recurse,
+                             svn_boolean_t ignore_externals,
                              svn_boolean_t *timestamp_sleep,
                              svn_client_ctx_t *ctx,
                              apr_pool_t *pool);
 
-/* Checkout into PATH a working copy of URL at REVISION.  If
-   TIMESTAMP_SLEEP is NULL this function will sleep before returning to
-   ensure timestamp integrity.  If TIMESTAMP_SLEEP is not NULL then the
-   function will not sleep but will set *TIMESTAMP_SLEEP to TRUE if a sleep
-   is required, and will not change *TIMESTAMP_SLEEP if no sleep is
-   required. */
+/* Checkout into PATH a working copy of URL at REVISION, and (if not
+   NULL) set RESULT_REV to the checked out revision.  RECURSE if so
+   commanded; likewise, possibly IGNORE_EXTERNALS.  If TIMESTAMP_SLEEP
+   is NULL this function will sleep before returning to ensure
+   timestamp integrity.  If TIMESTAMP_SLEEP is not NULL then the
+   function will not sleep but will set *TIMESTAMP_SLEEP to TRUE if a
+   sleep is required, and will not change *TIMESTAMP_SLEEP if no sleep
+   is required. */
 svn_error_t *
-svn_client__checkout_internal (const char *URL,
+svn_client__checkout_internal (svn_revnum_t *result_rev,
+                               const char *URL,
                                const char *path,
+                               const svn_opt_revision_t *peg_revision,
                                const svn_opt_revision_t *revision,
                                svn_boolean_t recurse,
+                               svn_boolean_t ignore_externals,
                                svn_boolean_t *timestamp_sleep,
                                svn_client_ctx_t *ctx,
                                apr_pool_t *pool);
@@ -307,7 +366,7 @@ svn_client__checkout_internal (const char *URL,
  *
  * DRY_RUN is set if this is a dry-run merge. It is not relevant for diff.
  *
- * RA_LIB/RA_SESSION define the additional ra session for requesting file
+ * RA_SESSION defines the additional RA session for requesting file
  * contents.
  *
  * REVISION is the start revision in the comparison.
@@ -320,14 +379,13 @@ svn_client__checkout_internal (const char *URL,
 svn_error_t *
 svn_client__get_diff_editor (const char *target,
                              svn_wc_adm_access_t *adm_access,
-                             const svn_wc_diff_callbacks_t *diff_cmd,
+                             const svn_wc_diff_callbacks2_t *diff_cmd,
                              void *diff_cmd_baton,
                              svn_boolean_t recurse,
                              svn_boolean_t dry_run,
-                             svn_ra_plugin_t *ra_lib,
-                             void *ra_session, 
+                             svn_ra_session_t *ra_session, 
                              svn_revnum_t revision,
-                             svn_wc_notify_func_t notify_func,
+                             svn_wc_notify_func2_t notify_func,
                              void *notify_baton,
                              svn_cancel_func_t cancel_func,
                              void *cancel_baton,
@@ -343,14 +401,14 @@ svn_client__get_diff_editor (const char *target,
 /* WARNING: This is all new, untested, un-peer-reviewed conceptual
    stuff.
 
-   The day that `svn switch' came into existence, our old commit
+   The day that 'svn switch' came into existence, our old commit
    crawler (svn_wc_crawl_local_mods) became obsolete.  It relied far
    too heavily on the on-disk heirarchy of files and directories, and
    simply had no way to support disjoint working copy trees or nest
    working copies.  The primary reason for this is that commit
    process, in order to guarantee atomicity, is a single drive of a
    commit editor which is based not on working copy paths, but on
-   URLs.  With the completion of `svn switch', it became all too
+   URLs.  With the completion of 'svn switch', it became all too
    likely that the on-disk working copy heirarchy would no longer be
    guaranteed to map to a similar in-repository heirarchy.
 
@@ -373,9 +431,9 @@ svn_client__get_diff_editor (const char *target,
    *** END-OF-KNOWLEDGE ***
 
    The prototypes below are still in development.  In general, the
-   idea is that commit-y processes (`svn mkdir URL`, `svn delete URL`,
-   `svn commit`, `svn copy WC_PATH URL`, `svn copy URL1 URL2`, `svn
-   move URL1 URL2`, others?) generate the cached commit candidate
+   idea is that commit-y processes ('svn mkdir URL', 'svn delete URL',
+   'svn commit', 'svn copy WC_PATH URL', 'svn copy URL1 URL2', 'svn
+   move URL1 URL2', others?) generate the cached commit candidate
    information, and hand this information off to a consumer which is
    responsible for driving the RA layer's commit editor in a
    URL-depth-first fashion and reporting back the post-commit
@@ -410,25 +468,38 @@ svn_client__get_diff_editor (const char *target,
        multi-repository support actually exists, the single key here
        will actually be some arbitrary thing to be ignored.  
 
+     - if the candidate has a lock token, add it to the LOCK_TOKENS hash.
+
+     - if the candidate is a directory scheduled for deletion, crawl
+       the directories children recursively for any lock tokens and
+       add them to the LOCK_TOKENS array.
+
    At the successful return of this function, COMMITTABLES will be an
    apr_hash_t * hash of apr_array_header_t * arrays (of
    svn_client_commit_item_t * structures), keyed on const char *
-   canonical repository URLs.  Also, LOCKED_DIRS will be an apr_hash_t *
-   hash of svn_wc_adm_access_t * keyed on const char * working copy path
-   directory names which were locked in the process of this crawl.
-   These will need to be unlocked again post-commit.
+   canonical repository URLs.  LOCK_TOKENS will point to a hash table
+   with const char * lock tokens, keyed on const char * URLs.  Also,
+   LOCKED_DIRS will be an apr_hash_t * hash of svn_wc_adm_access_t *
+   keyed
+   on const char * working copy path directory names which were locked
+   in the process of this crawl.  These will need to be unlocked again post-commit.
 
    If NONRECURSIVE is specified, subdirectories of directory targets
-   found in TARGETS will not be crawled for modifications. 
+   found in TARGETS will not be crawled for modifications.
+
+   If JUST_LOCKED is TRUE, treat unmodified items with lock tokens as
+   commit candidates.
 
    If CTX->CANCEL_FUNC is non-null, it will be called with 
    CTX->CANCEL_BATON while harvesting to determine if the client has 
    cancelled the operation.  */
 svn_error_t *
 svn_client__harvest_committables (apr_hash_t **committables,
+                                  apr_hash_t **lock_tokens,
                                   svn_wc_adm_access_t *parent_dir,
                                   apr_array_header_t *targets,
                                   svn_boolean_t nonrecursive,
+                                  svn_boolean_t just_locked,
                                   svn_client_ctx_t *ctx,
                                   apr_pool_t *pool);
 
@@ -474,11 +545,6 @@ svn_client__condense_commit_items (const char **base_url,
    COMMIT_ITEMS should have been run through (and BASE_URL generated
    by) svn_client__condense_commit_items.
 
-   REVNUM_FN/REV_BATON allows this routine to query the repository for
-   the latest revision.  It is used (temporarily) for checking that
-   directories are "up-to-date" when a dir-propchange is discovered.
-   We don't expect it to be here forever.  :-) 
-
    CTX->NOTIFY_FUNC/CTX->BATON will be called as the commit progresses, as 
    a way of describing actions to the application layer (if non NULL).
 
@@ -528,14 +594,44 @@ svn_client__do_commit (const char *base_url,
    is required.
 
    Use POOL for temporary allocation. */
-svn_error_t *svn_client__handle_externals
-   (svn_wc_traversal_info_t *traversal_info,
-    svn_boolean_t update_unchanged,
-    svn_boolean_t *timestamp_sleep,
-    svn_client_ctx_t *ctx,
-    apr_pool_t *pool);
+svn_error_t *
+svn_client__handle_externals (svn_wc_traversal_info_t *traversal_info,
+                              svn_boolean_t update_unchanged,
+                              svn_boolean_t *timestamp_sleep,
+                              svn_client_ctx_t *ctx,
+                              apr_pool_t *pool);
 
 
+/* Fetch externals definitions described by EXTERNALS, a hash of the
+   form returned by svn_wc_edited_externals() (which see).  If
+   IS_EXPORT is set, the external items will be exported instead of
+   checked out -- they will have no administrative subdirectories.
+
+   *TIMESTAMP_SLEEP will be set TRUE if a sleep is required to ensure
+   timestamp integrity, *TIMESTAMP_SLEEP will be unchanged if no sleep
+   is required.
+
+   Use POOL for temporary allocation. */
+svn_error_t *
+svn_client__fetch_externals (apr_hash_t *externals,
+                             svn_boolean_t is_export,
+                             svn_boolean_t *timestamp_sleep,
+                             svn_client_ctx_t *ctx,
+                             apr_pool_t *pool);
+
+
+/* Perform status operations on each external in TRAVERSAL_INFO.  All
+   other options are the same as those passed to svn_client_status(). */
+svn_error_t *
+svn_client__do_external_status (svn_wc_traversal_info_t *traversal_info,
+                                svn_wc_status_func2_t status_func,
+                                void *status_baton,
+                                svn_boolean_t get_all,
+                                svn_boolean_t update,
+                                svn_boolean_t no_ignore,
+                                svn_client_ctx_t *ctx,
+                                apr_pool_t *pool);
+
 
 #ifdef __cplusplus
 }
