@@ -36,16 +36,37 @@
 #include "svn_pools.h"
 #include "svn_error.h"
 #include "svn_path.h"
-#include "svn_test.h"
 #include "svn_io.h"
 #include "svn_md5.h"
 #include "svn_time.h"
 #include "svn_sorts.h"
+#include "svn_props.h"
 
 #include "client.h"
 
 #include "svn_private_config.h"
 
+/* Import context baton.
+
+   ### TODO:  Add the following items to this baton:
+      /` import editor/baton. `/
+      const svn_delta_editor_t *editor;
+      void *edit_baton;
+
+      /` Client context baton `/
+      svn_client_ctx_t `ctx;
+
+      /` Paths (keys) excluded from the import (values ignored) `/
+      apr_hash_t *excludes;
+*/
+typedef struct import_ctx_t
+{
+  /* Whether any changes were made to the repository */
+  svn_boolean_t repos_changed; 
+
+} import_ctx_t;
+
+
 /* Apply PATH's contents (as a delta against the empty string) to
    FILE_BATON in EDITOR.  Use POOL for any temporary allocation.
    PROPERTIES is the set of node properties set on this file.
@@ -154,8 +175,8 @@ send_file_contents (const char *path,
  * Accumulate file paths and their batons in FILES, which must be
  * non-null.  (These are used to send postfix textdeltas later).
  *
- * If CTX->NOTIFY_FUNC is non-null, invoke it with CTX->NOTIFY_BATON for each
- * file.
+ * If CTX->NOTIFY_FUNC is non-null, invoke it with CTX->NOTIFY_BATON
+ * for each file.
  *
  * Use POOL for any temporary allocation.
  */
@@ -164,6 +185,7 @@ import_file (const svn_delta_editor_t *editor,
              void *dir_baton,
              const char *path,
              const char *edit_path,
+             import_ctx_t *import_ctx,
              svn_client_ctx_t *ctx,
              apr_pool_t *pool)
 {
@@ -176,11 +198,24 @@ import_file (const svn_delta_editor_t *editor,
   svn_node_kind_t kind;
   svn_boolean_t is_special;
 
+  SVN_ERR (svn_path_check_valid (path, pool));
+
+  SVN_ERR (svn_io_check_special_path (path, &kind, &is_special, pool));
+
+  if (kind == svn_node_unknown)
+    {
+      return svn_error_createf
+        (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
+         _("Unknown or unversionable type for '%s'"),
+           svn_path_local_style (path, pool));
+    }
+
   /* Add the file, using the pool from the FILES hash. */
   SVN_ERR (editor->add_file (edit_path, dir_baton, NULL, SVN_INVALID_REVNUM, 
                              pool, &file_baton));
 
-  SVN_ERR (svn_io_check_special_path (path, &kind, &is_special, pool));
+  /* Remember that the repository was modified */
+  import_ctx->repos_changed = TRUE;
 
   if (! is_special)
     {
@@ -203,15 +238,17 @@ import_file (const svn_delta_editor_t *editor,
         }
     }
 
-  if (ctx->notify_func)
-    (*ctx->notify_func) (ctx->notify_baton,
-                         path,
-                         svn_wc_notify_commit_added,
-                         svn_node_file,
-                         mimetype,
-                         svn_wc_notify_state_inapplicable,
-                         svn_wc_notify_state_inapplicable,
-                         SVN_INVALID_REVNUM);
+  if (ctx->notify_func2)
+    {
+      svn_wc_notify_t *notify
+        = svn_wc_create_notify (path, svn_wc_notify_commit_added, pool);
+      notify->kind = svn_node_file;
+      notify->mime_type = mimetype;
+      notify->content_state = notify->prop_state
+        = svn_wc_notify_state_inapplicable;
+      notify->lock_state = svn_wc_notify_lock_state_inapplicable;
+      (*ctx->notify_func2) (ctx->notify_baton2, notify, pool);
+    }
 
   /* If this is a special file, we need to set the svn:special
      property and create a temporary detranslated version in order to
@@ -260,6 +297,7 @@ import_dir (const svn_delta_editor_t *editor,
             const char *edit_path,
             svn_boolean_t nonrecursive,
             apr_hash_t *excludes,
+            import_ctx_t *import_ctx,
             svn_client_ctx_t *ctx,
             apr_pool_t *pool)
 {
@@ -268,6 +306,8 @@ import_dir (const svn_delta_editor_t *editor,
   apr_hash_index_t *hi;
   apr_array_header_t *ignores;
 
+  SVN_ERR (svn_path_check_valid (path, pool));
+
   SVN_ERR (svn_wc_get_default_ignores (&ignores, ctx->config, pool));
 
   SVN_ERR (svn_io_get_dirents (&dirents, path, pool));
@@ -298,15 +338,18 @@ import_dir (const svn_delta_editor_t *editor,
              with that name, something is bound to blow up when they
              checkout what they've imported.  So, just skip items with
              that name.  */
-          if (ctx->notify_func)
-            (*ctx->notify_func) (ctx->notify_baton,
-                                 svn_path_join (path, filename, subpool),
-                                 svn_wc_notify_skip,
-                                 svn_node_dir,
-                                 NULL,
-                                 svn_wc_notify_state_inapplicable,
-                                 svn_wc_notify_state_inapplicable,
-                                 SVN_INVALID_REVNUM);
+          if (ctx->notify_func2)
+            {
+              svn_wc_notify_t *notify
+                = svn_wc_create_notify (svn_path_join (path, filename,
+                                                       subpool),
+                                        svn_wc_notify_skip, subpool);
+              notify->kind = svn_node_dir;
+              notify->content_state = notify->prop_state
+                = svn_wc_notify_state_inapplicable;
+              notify->lock_state = svn_wc_notify_lock_state_inapplicable;
+              (*ctx->notify_func2) (ctx->notify_baton2, notify, subpool);
+            }
           continue;
         }
 
@@ -336,24 +379,29 @@ import_dir (const svn_delta_editor_t *editor,
                                           NULL, SVN_INVALID_REVNUM, subpool,
                                           &this_dir_baton));
 
+          /* Remember that the repository was modified */
+          import_ctx->repos_changed = TRUE;
+
           /* By notifying before the recursive call below, we display
              a directory add before displaying adds underneath the
              directory.  To do it the other way around, just move this
              after the recursive call. */
-          if (ctx->notify_func)
-            (*ctx->notify_func) (ctx->notify_baton,
-                                 this_path,
-                                 svn_wc_notify_commit_added,
-                                 svn_node_dir,
-                                 NULL,
-                                 svn_wc_notify_state_inapplicable,
-                                 svn_wc_notify_state_inapplicable,
-                                 SVN_INVALID_REVNUM);
+          if (ctx->notify_func2)
+            {
+              svn_wc_notify_t *notify
+                = svn_wc_create_notify (this_path, svn_wc_notify_commit_added,
+                                        subpool);
+              notify->kind = svn_node_dir;
+              notify->content_state = notify->prop_state
+                = svn_wc_notify_state_inapplicable;
+              notify->lock_state = svn_wc_notify_lock_state_inapplicable;
+              (*ctx->notify_func2) (ctx->notify_baton2, notify, subpool);
+            }
 
           /* Recurse. */
           SVN_ERR (import_dir (editor, this_dir_baton, 
                                this_path, this_edit_path, 
-                               FALSE, excludes, ctx, subpool));
+                               FALSE, excludes, import_ctx, ctx, subpool));
 
           /* Finally, close the sub-directory. */
           SVN_ERR (editor->close_directory (this_dir_baton, subpool));
@@ -361,8 +409,8 @@ import_dir (const svn_delta_editor_t *editor,
       else if (*filetype == svn_node_file)
         {
           /* Import a file. */
-          SVN_ERR (import_file (editor, dir_baton, 
-                                this_path, this_edit_path, ctx, subpool));
+          SVN_ERR (import_file (editor, dir_baton, this_path, 
+                                this_edit_path, import_ctx, ctx, subpool));
         }
       /* We're silently ignoring things that aren't files or
          directories.  If we stop doing that, here is the place to
@@ -418,6 +466,7 @@ import (const char *path,
   apr_array_header_t *ignores;
   apr_array_header_t *batons = NULL;
   const char *edit_path = "";
+  import_ctx_t *import_ctx = apr_pcalloc (pool, sizeof (*import_ctx));
 
   /* Get a root dir baton.  We pass an invalid revnum to open_root
      to mean "base this on the youngest revision".  Should we have an
@@ -451,6 +500,9 @@ import (const char *path,
                                           root_baton,
                                           NULL, SVN_INVALID_REVNUM,
                                           pool, &root_baton));
+
+          /* Remember that the repository was modified */
+          import_ctx->repos_changed = TRUE;
         }
     }
   else if (kind == svn_node_file)
@@ -473,18 +525,19 @@ import (const char *path,
       SVN_ERR (svn_wc_get_default_ignores (&ignores, ctx->config, pool));
       if (! svn_cstring_match_glob_list (path, ignores))
         SVN_ERR (import_file (editor, root_baton, path, edit_path,
-                              ctx, pool));
+                              import_ctx, ctx, pool));
     }
   else if (kind == svn_node_dir)
     {
       SVN_ERR (import_dir (editor, root_baton, path, edit_path,
-                           nonrecursive, excludes, ctx, pool));
+                           nonrecursive, excludes, import_ctx, ctx, pool));
 
     }
   else if (kind == svn_node_none)
     {
       return svn_error_createf (SVN_ERR_NODE_UNKNOWN_KIND, NULL, 
-                                _("'%s' does not exist"), path);  
+                                _("'%s' does not exist"),
+                                svn_path_local_style (path, pool));  
     }
 
   /* Close up shop; it's time to go home. */
@@ -498,16 +551,17 @@ import (const char *path,
         }
     }
 
-  SVN_ERR (editor->close_edit (edit_baton, pool));
+  if (import_ctx->repos_changed)
+    SVN_ERR (editor->close_edit (edit_baton, pool));
+  else
+    SVN_ERR (editor->abort_edit (edit_baton, pool));
 
   return SVN_NO_ERROR;
 }
 
 
 static svn_error_t *
-get_ra_editor (void **ra_baton, 
-               void **session,
-               svn_ra_plugin_t **ra_lib,
+get_ra_editor (svn_ra_session_t **ra_session,
                svn_revnum_t *latest_rev,
                const svn_delta_editor_t **editor,
                void **edit_baton,
@@ -519,17 +573,14 @@ get_ra_editor (void **ra_baton,
                apr_array_header_t *commit_items,
                svn_client_commit_info_t **commit_info,
                svn_boolean_t is_commit,
+               apr_hash_t *lock_tokens,
+               svn_boolean_t keep_locks,
                apr_pool_t *pool)
 {
   void *commit_baton;
 
-  /* Get the RA vtable that matches URL. */
-  SVN_ERR (svn_ra_init_ra_libs (ra_baton, pool));
-  SVN_ERR (svn_ra_get_ra_library (ra_lib, *ra_baton, 
-                                  base_url, pool));
-  
   /* Open an RA session to URL. */
-  SVN_ERR (svn_client__open_ra_session (session, *ra_lib,
+  SVN_ERR (svn_client__open_ra_session (ra_session,
                                         base_url, base_dir, base_access,
                                         commit_items,
                                         is_commit, !is_commit,
@@ -541,8 +592,8 @@ get_ra_editor (void **ra_baton,
     {
       svn_node_kind_t kind;
 
-      SVN_ERR ((*ra_lib)->check_path (*session, "", SVN_INVALID_REVNUM,
-                                      &kind, pool));
+      SVN_ERR (svn_ra_check_path (*ra_session, "", SVN_INVALID_REVNUM,
+                                  &kind, pool));
       if (kind == svn_node_none)
         return svn_error_createf (SVN_ERR_FS_NO_SUCH_ENTRY, NULL,
                                   _("Path '%s' does not exist"),
@@ -551,13 +602,14 @@ get_ra_editor (void **ra_baton,
 
   /* Fetch the latest revision if requested. */
   if (latest_rev)
-    SVN_ERR ((*ra_lib)->get_latest_revnum (*session, latest_rev, pool));
+    SVN_ERR (svn_ra_get_latest_revnum (*ra_session, latest_rev, pool));
   
   /* Fetch RA commit editor. */
   SVN_ERR (svn_client__commit_get_baton (&commit_baton, commit_info, pool));
-  return (*ra_lib)->get_commit_editor (*session, editor, edit_baton, log_msg,
-                                       svn_client__commit_callback,
-                                       commit_baton, pool);
+  return svn_ra_get_commit_editor (*ra_session, editor, edit_baton, log_msg,
+                                   svn_client__commit_callback,
+                                   commit_baton, lock_tokens, keep_locks,
+                                   pool);
 }
 
 
@@ -575,8 +627,7 @@ svn_client_import (svn_client_commit_info_t **commit_info,
   const char *log_msg = "";
   const svn_delta_editor_t *editor;
   void *edit_baton;
-  void *ra_baton, *session;
-  svn_ra_plugin_t *ra_lib;
+  svn_ra_session_t *ra_session;
   apr_hash_t *excludes = apr_hash_make (pool);
   svn_node_kind_t kind;
   const char *base_dir = path;
@@ -586,7 +637,6 @@ svn_client_import (svn_client_commit_info_t **commit_info,
   const char *dir;
   apr_pool_t *subpool;
 
-
   /* Create a new commit item and add it to the array. */
   if (ctx->log_msg_func)
     {
@@ -647,10 +697,10 @@ svn_client_import (svn_client_commit_info_t **commit_info,
           url = temp;
         }
     }
-  while ((err = get_ra_editor (&ra_baton, &session, &ra_lib, NULL,
+  while ((err = get_ra_editor (&ra_session, NULL,
                                &editor, &edit_baton, ctx, url, base_dir,
                                NULL, log_msg, NULL, commit_info,
-                               FALSE, subpool)));
+                               FALSE, NULL, TRUE, subpool)));
 
   /* Reverse the order of the components we added to our NEW_ENTRIES array. */
   if (new_entries->nelts)
@@ -686,18 +736,35 @@ svn_client_import (svn_client_commit_info_t **commit_info,
     return svn_error_createf
       (SVN_ERR_CL_ADM_DIR_RESERVED, NULL,
        _("'%s' is a reserved name and cannot be imported"),
-       SVN_WC_ADM_DIR_NAME);
+       /* ### Is svn_path_local_style() really necessary for this? */
+       svn_path_local_style (SVN_WC_ADM_DIR_NAME, pool));
 
 
   /* If an error occurred during the commit, abort the edit and return
      the error.  We don't even care if the abort itself fails.  */
   if ((err = import (path, new_entries, editor, edit_baton, 
-                     nonrecursive, excludes, ctx, pool)))
+                     nonrecursive, excludes, ctx, subpool)))
     {
-      svn_error_clear (editor->abort_edit (edit_baton, pool));
+      svn_error_clear (editor->abort_edit (edit_baton, subpool));
       return err;
     }
 
+  /* Transfer *COMMIT_INFO from the subpool to the callers pool */
+  if (*commit_info)
+    {
+      svn_client_commit_info_t *tmp_commit_info;
+
+      tmp_commit_info = apr_palloc(pool, sizeof(*tmp_commit_info));
+      *tmp_commit_info = **commit_info;
+      if (tmp_commit_info->date)
+        tmp_commit_info->date = apr_pstrdup (pool, tmp_commit_info->date);
+      if (tmp_commit_info->author)
+        tmp_commit_info->author = apr_pstrdup (pool, tmp_commit_info->author);
+      *commit_info = tmp_commit_info;
+    }
+
+  svn_pool_destroy (subpool);
+
   return SVN_NO_ERROR;
 }
 
@@ -1026,18 +1093,59 @@ adjust_rel_targets (const char **pbase_dir,
   return SVN_NO_ERROR;
 }
 
+
+/* For all lock tokens in ALL_TOKENS for URLs under BASE_URL, add them
+   to a new hashtable allocated in POOL.  *RESULT is set to point to this
+   new hash table.  *RESULT will be keyed on const char * URI-decoded paths
+   relative to BASE_URL.  The lock tokens will not be duplicated. */
+static svn_error_t *
+collect_lock_tokens (apr_hash_t **result,
+                     apr_hash_t *all_tokens,
+                     const char *base_url,
+                     apr_pool_t *pool)
+{
+  apr_hash_index_t *hi;
+  size_t base_len = strlen (base_url);
+
+  *result = apr_hash_make (pool);
+
+  for (hi = apr_hash_first (pool, all_tokens); hi; hi = apr_hash_next (hi))
+    {
+      const void *key;
+      void *val;
+      const char *url;
+      const char *token;
+
+      apr_hash_this (hi, &key, NULL, &val);
+      url = key;
+      token = val;
+
+      if (strncmp (base_url, url, base_len) == 0
+          && (url[base_len] == '\0' || url[base_len] == '/'))
+        {
+          if (url[base_len] == '\0')
+            url = "";
+          else
+            url = svn_path_uri_decode (url + base_len + 1, pool);
+          apr_hash_set (*result, url, APR_HASH_KEY_STRING, token);
+        }
+    }
+
+  return SVN_NO_ERROR;
+}
+
 svn_error_t *
-svn_client_commit (svn_client_commit_info_t **commit_info,
-                   const apr_array_header_t *targets,
-                   svn_boolean_t nonrecursive,
-                   svn_client_ctx_t *ctx,
-                   apr_pool_t *pool)
+svn_client_commit2 (svn_client_commit_info_t **commit_info,
+                    const apr_array_header_t *targets,
+                    svn_boolean_t recurse,
+                    svn_boolean_t keep_locks,
+                    svn_client_ctx_t *ctx,
+                    apr_pool_t *pool)
 {
   const svn_delta_editor_t *editor;
   void *edit_baton;
-  void *ra_baton, *session;
+  svn_ra_session_t *ra_session;
   const char *log_msg;
-  svn_ra_plugin_t *ra_lib;
   const char *base_dir;
   const char *base_url;
   const char *target;
@@ -1045,7 +1153,7 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
   apr_array_header_t *dirs_to_lock;
   apr_array_header_t *dirs_to_lock_recursive;
   svn_boolean_t lock_base_dir_recursive = FALSE;
-  apr_hash_t *committables, *tempfiles = NULL;
+  apr_hash_t *committables, *lock_tokens, *tempfiles = NULL;
   svn_wc_adm_access_t *base_dir_access;
   apr_array_header_t *commit_items;
   svn_error_t *cmt_err = SVN_NO_ERROR, *unlock_err = SVN_NO_ERROR;
@@ -1066,8 +1174,7 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
 
   /* Condense the target list. */
   SVN_ERR (svn_path_condense_targets (&base_dir, &rel_targets, targets,
-                                      nonrecursive ? FALSE : TRUE,
-                                      pool));
+                                      recurse, pool));
 
   /* No targets means nothing to commit, so just return. */
   if (! base_dir)
@@ -1107,10 +1214,10 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
           /* If the final target is a dir, we want to recursively lock it */
           if (kind == svn_node_dir)
             {
-              if (nonrecursive)
-                APR_ARRAY_PUSH (dirs_to_lock, const char *) = target;
-              else
+              if (recurse)
                 APR_ARRAY_PUSH (dirs_to_lock_recursive, const char *) = target;
+              else
+                APR_ARRAY_PUSH (dirs_to_lock, const char *) = target;
             }
         }
       else
@@ -1150,11 +1257,11 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
                  lock it */
               if (kind == svn_node_dir)
                 {
-                  if (nonrecursive)
-                    APR_ARRAY_PUSH (dirs_to_lock, 
+                  if (recurse)
+                    APR_ARRAY_PUSH (dirs_to_lock_recursive, 
                                     const char *) = apr_pstrdup (pool, target);
                   else
-                    APR_ARRAY_PUSH (dirs_to_lock_recursive, 
+                    APR_ARRAY_PUSH (dirs_to_lock, 
                                     const char *) = apr_pstrdup (pool, target);
                 }
             }
@@ -1173,9 +1280,10 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
       svn_pool_destroy (subpool);
     }
 
-  SVN_ERR (svn_wc_adm_open2 (&base_dir_access, NULL, base_dir,
+  SVN_ERR (svn_wc_adm_open3 (&base_dir_access, NULL, base_dir,
                              TRUE,  /* Write lock */
                              lock_base_dir_recursive ? -1 : 0, /* Depth */
+                             ctx->cancel_func, ctx->cancel_baton,
                              pool));
 
   if (!lock_base_dir_recursive)
@@ -1210,10 +1318,12 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
             {
               target = APR_ARRAY_IDX (dirs_to_lock, i, const char *);
 
-              SVN_ERR (svn_wc_adm_open2 (&adm_access, base_dir_access,
+              SVN_ERR (svn_wc_adm_open3 (&adm_access, base_dir_access,
                                          target,
                                          TRUE,  /* Write lock */
                                          0,     /* Depth */
+                                         ctx->cancel_func,
+                                         ctx->cancel_baton,
                                          pool));
             }
         }
@@ -1225,10 +1335,12 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
             {
               target = APR_ARRAY_IDX (dirs_to_lock_recursive, i, const char *);
 
-              SVN_ERR (svn_wc_adm_open2 (&adm_access, base_dir_access,
+              SVN_ERR (svn_wc_adm_open3 (&adm_access, base_dir_access,
                                          target,
                                          TRUE, /* Write lock */
                                          -1,   /* Depth */
+                                         ctx->cancel_func,
+                                         ctx->cancel_baton,
                                          pool));
             }
         }
@@ -1251,16 +1363,16 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
                                             target, pool),
                  _("Are all the targets part of the same working copy?"));
 
-      if (nonrecursive)
+      if (!recurse)
         {
-          svn_wc_status_t *status;
+          svn_wc_status2_t *status;
           svn_node_kind_t kind;
           
           SVN_ERR (svn_io_check_path (target, &kind, pool));
 
           if (kind == svn_node_dir)
             {
-              SVN_ERR (svn_wc_status (&status, target, adm_access, pool));
+              SVN_ERR (svn_wc_status2 (&status, target, adm_access, pool));
               if (status->text_status == svn_wc_status_deleted ||
                   status->text_status == svn_wc_status_replaced)
                 return svn_error_create (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
@@ -1271,10 +1383,12 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
     }
 
   /* Crawl the working copy for commit items. */
-  if ((cmt_err = svn_client__harvest_committables (&committables, 
+  if ((cmt_err = svn_client__harvest_committables (&committables,
+                                                   &lock_tokens,
                                                    base_dir_access,
                                                    rel_targets, 
-                                                   nonrecursive,
+                                                   recurse ? FALSE : TRUE,
+                                                   ! keep_locks,
                                                    ctx,
                                                    pool)))
     goto cleanup;
@@ -1290,6 +1404,28 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
                                        APR_HASH_KEY_STRING))))
     goto cleanup;
 
+  /* If our array of targets contains only locks (and no actual file
+     or prop modifications), then we return here to avoid committing a
+     revision with no changes. */
+  {
+    svn_boolean_t found_changed_path = FALSE;
+
+    for (i = 0; i < commit_items->nelts; ++i)
+      {
+        svn_client_commit_item_t *item;
+        item = APR_ARRAY_IDX (commit_items, i, svn_client_commit_item_t *);
+        
+        if (item->state_flags != SVN_CLIENT_COMMIT_ITEM_LOCK_TOKEN) 
+          {
+            found_changed_path = TRUE;
+            break;
+          }
+      }
+
+    if (!found_changed_path)
+      goto cleanup;
+  }
+
   /* Go get a log message.  If an error occurs, or no log message is
      specified, abort the operation. */
   if (ctx->log_msg_func)
@@ -1309,11 +1445,16 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
                                                     pool)))
     goto cleanup;
 
-  if ((cmt_err = get_ra_editor (&ra_baton, &session, &ra_lib, NULL,
+  /* Collect our lock tokens with paths relative to base_url. */
+  if ((cmt_err = collect_lock_tokens (&lock_tokens, lock_tokens, base_url,
+                                      pool)))
+    goto cleanup;
+
+  if ((cmt_err = get_ra_editor (&ra_session, NULL,
                                 &editor, &edit_baton, ctx,
                                 base_url, base_dir, base_dir_access,
                                 log_msg, commit_items, commit_info,
-                                TRUE, pool)))
+                                TRUE, lock_tokens, keep_locks, pool)))
     goto cleanup;
 
   /* Make a note that we have a commit-in-progress. */
@@ -1344,10 +1485,11 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
         {
           svn_client_commit_item_t *item
             = ((svn_client_commit_item_t **) commit_items->elts)[i];
-          svn_boolean_t recurse = FALSE;
+          svn_boolean_t loop_recurse = FALSE;
           const char *adm_access_path;
           svn_wc_adm_access_t *adm_access;
           const svn_wc_entry_t *entry;
+          svn_boolean_t remove_lock;
 
           /* Clear the subpool here because there are some 'continue'
              statements in this loop. */
@@ -1405,17 +1547,21 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
           if ((item->state_flags & SVN_CLIENT_COMMIT_ITEM_ADD) 
               && (item->kind == svn_node_dir)
               && (item->copyfrom_url))
-            recurse = TRUE;
+            loop_recurse = TRUE;
 
+          remove_lock = (! keep_locks && (item->state_flags
+                                          & SVN_CLIENT_COMMIT_ITEM_LOCK_TOKEN));
           assert (*commit_info);
-          if ((bump_err = svn_wc_process_committed (item->path, adm_access,
-                                                    recurse,
-                                                    (*commit_info)->revision,
-                                                    (*commit_info)->date,
-                                                    (*commit_info)->author,
-                                                    item->wcprop_changes,
-                                                    subpool)))
+          if ((bump_err = svn_wc_process_committed2 (item->path, adm_access,
+                                                     loop_recurse,
+                                                     (*commit_info)->revision,
+                                                     (*commit_info)->date,
+                                                     (*commit_info)->author,
+                                                     item->wcprop_changes,
+                                                     remove_lock,
+                                                     subpool)))
             break;
+
         }
 
       /* Destroy the subpool. */
@@ -1433,7 +1579,7 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
   /* A bump error is likely to occur while running a working copy log file,
      explicitly unlocking and removing temporary files would be wrong in
      that case.  A commit error (cmt_err) should only occur before any
-     attempt to modify the working copy, so it doesn't prevent explict
+     attempt to modify the working copy, so it doesn't prevent explicit
      clean-up. */
   if (! bump_err)
     {
@@ -1445,3 +1591,16 @@ svn_client_commit (svn_client_commit_info_t **commit_info,
 
   return reconcile_errors (cmt_err, unlock_err, bump_err, cleanup_err, pool);
 }
+
+svn_error_t *
+svn_client_commit (svn_client_commit_info_t **commit_info,
+                   const apr_array_header_t *targets,
+                   svn_boolean_t nonrecursive,
+                   svn_client_ctx_t *ctx,
+                   apr_pool_t *pool)
+{
+  return svn_client_commit2 (commit_info, targets, 
+                             nonrecursive ? FALSE : TRUE, 
+                             TRUE,
+                             ctx, pool);
+}
