@@ -29,6 +29,7 @@
 #include "svn_wc.h"
 #include "svn_client.h"
 #include "svn_path.h"
+#include "svn_props.h"
 #include "client.h"
 
 #include "svn_private_config.h"
@@ -128,7 +129,7 @@ push_wc_prop (void *baton,
     return svn_error_createf
       (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
        _("Attempt to set wc property '%s' on '%s' in a non-commit operation"),
-       name, relpath);
+       name, svn_path_local_style (relpath, pool));
 
   for (i = 0; i < cb->commit_items->nelts; i++)
     {
@@ -185,7 +186,15 @@ set_wc_prop (void *baton,
                                  : svn_path_dirname (full_path, pool)),
                                 pool));
     
-  return svn_wc_prop_set (name, value, full_path, adm_access, pool);
+  /* We pass 1 for the 'force' parameter here.  Since the property is
+     coming from the repository, we definitely want to accept it.
+     Ideally, we'd raise a conflict if, say, the received property is
+     svn:eol-style yet the file has a locally added svn:mime-type
+     claiming that it's binary.  Probably the repository is still
+     right, but the conflict would remind the user to make sure.
+     Unfortunately, we don't have a clean mechanism for doing that
+     here, so we just set the property and hope for the best. */
+  return svn_wc_prop_set2 (name, value, full_path, adm_access, TRUE, pool);
 }
 
 
@@ -215,7 +224,10 @@ invalidate_wcprop_for_entry (const char *path,
                                  ? path
                                  : svn_path_dirname (path, pool)),
                                 pool));
-  return svn_wc_prop_set (wb->prop_name, NULL, path, entry_access, pool);
+  /* It doesn't matter if we pass 0 or 1 for force here, since
+     property deletion is always permitted. */
+  return svn_wc_prop_set2 (wb->prop_name, NULL, path, entry_access,
+                           FALSE, pool);
 }
 
 
@@ -238,16 +250,16 @@ invalidate_wc_props (void *baton,
   path = svn_path_join (cb->base_dir, path, pool);
   SVN_ERR (svn_wc_adm_probe_retrieve (&adm_access, cb->base_access, path,
                                       pool));
-  SVN_ERR (svn_wc_walk_entries (path, adm_access, &walk_callbacks, &wb,
-                                FALSE, pool));
+  SVN_ERR (svn_wc_walk_entries2 (path, adm_access, &walk_callbacks, &wb,
+                                 FALSE, cb->ctx->cancel_func,
+                                 cb->ctx->cancel_baton, pool));
 
   return SVN_NO_ERROR;
 }
 
 
 svn_error_t * 
-svn_client__open_ra_session (void **session_baton,
-                             const svn_ra_plugin_t *ra_lib,
+svn_client__open_ra_session (svn_ra_session_t **ra_session,
                              const char *base_url,
                              const char *base_dir,
                              svn_wc_adm_access_t *base_access,
@@ -271,10 +283,9 @@ svn_client__open_ra_session (void **session_baton,
   cb->base_access = base_access;
   cb->pool = pool;
   cb->commit_items = commit_items;
-  cb->config = ctx->config;
+  cb->ctx = ctx;
 
-  SVN_ERR (ra_lib->open (session_baton, base_url, cbtable, cb, ctx->config,
-                         pool));
+  SVN_ERR (svn_ra_open (ra_session, base_url, cbtable, cb, ctx->config, pool));
 
   return SVN_NO_ERROR;
 }
@@ -287,19 +298,16 @@ svn_client_uuid_from_url (const char **uuid,
                           svn_client_ctx_t *ctx,
                           apr_pool_t *pool)
 {
-  svn_ra_plugin_t *ra_lib;  
-  void *ra_baton, *session;
+  svn_ra_session_t *ra_session;
   apr_pool_t *subpool = svn_pool_create (pool);
 
   /* use subpool to create a temporary RA session */
-  SVN_ERR (svn_ra_init_ra_libs (&ra_baton, subpool));
-  SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, url, subpool));
-  SVN_ERR (svn_client__open_ra_session (&session, ra_lib, url,
+  SVN_ERR (svn_client__open_ra_session (&ra_session, url,
                                         NULL, /* no base dir */
                                         NULL, NULL, FALSE, TRUE, 
                                         ctx, subpool));
 
-  ra_lib->get_uuid (session, uuid, subpool);
+  SVN_ERR (svn_ra_get_uuid (ra_session, uuid, subpool));
 
   /* Copy the uuid in to the passed-in pool. */
   *uuid = apr_pstrdup (pool, *uuid);
@@ -325,7 +333,8 @@ svn_client_uuid_from_path (const char **uuid,
 
   if (! entry)
     return svn_error_createf (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-                              _("Can't find entry for '%s'"), path);
+                              _("Can't find entry for '%s'"),
+                              svn_path_local_style (path, pool));
 
   if (entry->uuid)
     {
@@ -453,7 +462,7 @@ svn_client__prev_log_path (const char **prev_path_p,
         return svn_error_createf (SVN_ERR_CLIENT_UNRELATED_RESOURCES, NULL,
                                   _("Missing changed-path information for "
                                     "'%s' in revision %ld"),
-                                  path, revision);
+                                  svn_path_local_style (path, pool), revision);
     }
   
   *prev_path_p = prev_path;
@@ -554,7 +563,7 @@ slow_locations (const char **start_path, const char** end_path,
                 const char *abs_path, svn_revnum_t peg_revnum,
                 svn_revnum_t start_revnum, svn_revnum_t end_revnum,
                 const char *orig_path,
-                svn_ra_plugin_t *ra_lib, void *ra_session,
+                svn_ra_session_t *ra_session,
                 svn_client_ctx_t *ctx,
                 apr_pool_t *pool)
 {
@@ -564,7 +573,7 @@ slow_locations (const char **start_path, const char** end_path,
   svn_boolean_t pegrev_is_youngest = FALSE;
 
   /* Sanity check:  verify that the peg-object exists in repos. */
-  SVN_ERR (ra_lib->check_path (ra_session, "", peg_revnum, &(lrb.kind), pool));
+  SVN_ERR (svn_ra_check_path (ra_session, "", peg_revnum, &(lrb.kind), pool));
   if (lrb.kind == svn_node_none)
     return svn_error_createf
       (SVN_ERR_FS_NOT_FOUND, NULL,
@@ -622,8 +631,8 @@ slow_locations (const char **start_path, const char** end_path,
   /* Let the RA layer drive our log information handler, which will do
      the work of finding the actual locations for our resource.
      Notice that we always run on the youngest rev of the 3 inputs. */
-  SVN_ERR (ra_lib->get_log (ra_session, targets, youngest, 1,
-                            TRUE, FALSE, log_receiver, &lrb, pool));
+  SVN_ERR (svn_ra_get_log (ra_session, targets, youngest, 1, 0,
+                           TRUE, FALSE, log_receiver, &lrb, pool));
 
   /* Check that we got the peg path. */
   if (! lrb.peg_path)
@@ -656,7 +665,6 @@ svn_client__repos_locations (const char **start_url,
                              const svn_opt_revision_t *revision,
                              const svn_opt_revision_t *start,
                              const svn_opt_revision_t *end,
-                             svn_ra_plugin_t *ra_lib,
                              svn_client_ctx_t *ctx,
                              apr_pool_t *pool)
 {
@@ -668,7 +676,7 @@ svn_client__repos_locations (const char **start_url,
   svn_revnum_t start_revnum, end_revnum;
   apr_array_header_t *revs;
   apr_hash_t *rev_locs;
-  void *ra_session;
+  svn_ra_session_t *ra_session;
   apr_pool_t *subpool = svn_pool_create (pool);
   svn_error_t *err;
 
@@ -686,8 +694,9 @@ svn_client__repos_locations (const char **start_url,
     {
       svn_wc_adm_access_t *adm_access;
       const svn_wc_entry_t *entry;
-      SVN_ERR (svn_wc_adm_probe_open2 (&adm_access, NULL, path,
-                                       FALSE, 0, pool));
+      SVN_ERR (svn_wc_adm_probe_open3 (&adm_access, NULL, path,
+                                       FALSE, 0, ctx->cancel_func,
+                                       ctx->cancel_baton, pool));
       SVN_ERR (svn_wc_entry (&entry, path, adm_access, FALSE, pool));
       SVN_ERR (svn_wc_adm_close (adm_access));
       if (entry->copyfrom_url && revision->kind == svn_opt_revision_working)
@@ -702,7 +711,8 @@ svn_client__repos_locations (const char **start_url,
       else
         {
           return svn_error_createf (SVN_ERR_ENTRY_MISSING_URL, NULL,
-                                    _("'%s' has no URL"), path);
+                                    _("'%s' has no URL"),
+                                    svn_path_local_style (path, pool));
         }
     }
   else
@@ -715,33 +725,33 @@ svn_client__repos_locations (const char **start_url,
      don't need to do anything more here in that case. */
 
   /* Open a RA session to this URL. */
-  SVN_ERR (svn_client__open_ra_session (&ra_session, ra_lib, url, NULL,
+  SVN_ERR (svn_client__open_ra_session (&ra_session, url, NULL,
                                         NULL, NULL, FALSE, TRUE,
                                         ctx, subpool));
 
   /* Resolve the opt_revision_ts. */
   if (peg_revnum == SVN_INVALID_REVNUM)
-    SVN_ERR (svn_client__get_revision_number (&peg_revnum, ra_lib, 
+    SVN_ERR (svn_client__get_revision_number (&peg_revnum,
                                               ra_session, revision, path,
                                               pool));
   
-  SVN_ERR (svn_client__get_revision_number (&start_revnum, ra_lib, 
+  SVN_ERR (svn_client__get_revision_number (&start_revnum,
                                             ra_session, start, path, pool));
   if (end->kind == svn_opt_revision_unspecified)
     end_revnum = start_revnum;
   else
-    SVN_ERR (svn_client__get_revision_number (&end_revnum, ra_lib, 
+    SVN_ERR (svn_client__get_revision_number (&end_revnum,
                                               ra_session, end, path, pool));
 
-  SVN_ERR (ra_lib->get_repos_root (ra_session, &repos_url, subpool));
+  SVN_ERR (svn_ra_get_repos_root (ra_session, &repos_url, subpool));
 
   revs = apr_array_make (subpool, 2, sizeof (svn_revnum_t));
   APR_ARRAY_PUSH (revs, svn_revnum_t) = start_revnum;
   if (end_revnum != start_revnum)
     APR_ARRAY_PUSH (revs, svn_revnum_t) = end_revnum;
 
-  if (! (err = ra_lib->get_locations (ra_session, &rev_locs, "", peg_revnum,
-                                      revs, subpool)))
+  if (! (err = svn_ra_get_locations (ra_session, &rev_locs, "", peg_revnum,
+                                     revs, subpool)))
     {
       start_path = apr_hash_get (rev_locs, &start_revnum,
                                  sizeof (svn_revnum_t));
@@ -755,7 +765,7 @@ svn_client__repos_locations (const char **start_url,
                                svn_path_uri_decode (url + strlen (repos_url),
                                                     subpool),
                                peg_revnum, start_revnum, end_revnum,
-                               path, ra_lib, ra_session, ctx, subpool));
+                               path, ra_session, ctx, subpool));
     }
   else
     return err;
@@ -803,75 +813,88 @@ svn_client__repos_locations (const char **start_url,
 
 
 svn_error_t *
-svn_client__ra_lib_from_path (svn_ra_plugin_t **ra_lib_p,
-                              void **session_p,
-                              svn_revnum_t *rev_p,
-                              const char **url_p,
-                              const char *path_or_url,
-                              const svn_opt_revision_t *revision,
-                              svn_client_ctx_t *ctx,
-                              apr_pool_t *pool)
+svn_client__ra_session_from_path (svn_ra_session_t **ra_session_p,
+                                  svn_revnum_t *rev_p,
+                                  const char **url_p,
+                                  const char *path_or_url,
+                                  const svn_opt_revision_t *peg_revision_p,
+                                  const svn_opt_revision_t *revision,
+                                  svn_client_ctx_t *ctx,
+                                  apr_pool_t *pool)
 {
+  svn_ra_session_t *ra_session;
   const char *initial_url, *url;
-  void *ra_baton, *session;
-  svn_ra_plugin_t *ra_lib;
   const svn_opt_revision_t *good_rev;
+  svn_opt_revision_t peg_revision, start_rev;
+  svn_opt_revision_t dead_end_rev;
+  svn_opt_revision_t *ignored_rev, *new_rev;
   svn_revnum_t rev;
+  const char *ignored_url;
   
-  /* Get an RA library for the incoming path. */
   SVN_ERR (svn_client_url_from_path (&initial_url, path_or_url, pool));
   if (! initial_url)
     return svn_error_createf (SVN_ERR_ENTRY_MISSING_URL, NULL,
                               _("'%s' has no URL"), path_or_url);
 
-  SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
-  SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, initial_url, pool));
-
+  /* If a peg revision was specified, but a desired revision was not,
+     assume it is the same as the peg revision. */
+  if (revision->kind == svn_opt_revision_unspecified &&
+      peg_revision_p->kind != svn_opt_revision_unspecified)
+    revision = peg_revision_p;
+  
   if (svn_path_is_url (path_or_url))
     {
-      /* If an explicit URL was passed in, just use it. */
-      good_rev = revision;
-      url = initial_url;
+      /* URLs get a default starting rev of HEAD. */
+      if (revision->kind == svn_opt_revision_unspecified)
+        start_rev.kind = svn_opt_revision_head;
+      else
+        start_rev = *revision;
+          
+      /* If an explicit URL was passed in, the default peg revision is
+         HEAD. */
+      if (peg_revision_p->kind == svn_opt_revision_unspecified)
+        peg_revision.kind = svn_opt_revision_head;
+      else
+        peg_revision = *peg_revision_p;
     }
   else
     {
-      /* For a working copy path, don't blindly use its initial_url
-         from the entries file.  Run the history function to get the
-         object's (possibly different) url in REVISION. */
-      svn_opt_revision_t base_rev, dead_end_rev, start_rev;
-      svn_opt_revision_t *ignored_rev, *new_rev;
-      const char *ignored_url;
-
-      dead_end_rev.kind = svn_opt_revision_unspecified;
-      base_rev.kind = svn_opt_revision_working;
-
+      /* And a default starting rev of BASE. */
       if (revision->kind == svn_opt_revision_unspecified)
         start_rev.kind = svn_opt_revision_base;
       else
         start_rev = *revision;
-
-      SVN_ERR (svn_client__repos_locations (&url, &new_rev,
-                                            &ignored_url, &ignored_rev,
-                                            /* peg coords are path@BASE: */
-                                            path_or_url, &base_rev,
-                                            /* search range: */
-                                            &start_rev, &dead_end_rev,
-                                            ra_lib, ctx, pool));
-      good_rev = new_rev;
+      
+      /* WC paths have a default peg revision of WORKING. */
+      if (peg_revision_p->kind == svn_opt_revision_unspecified)
+        peg_revision.kind = svn_opt_revision_working;
+      else
+        peg_revision = *peg_revision_p;
     }
+  
+  dead_end_rev.kind = svn_opt_revision_unspecified;
+  
+  /* Run the history function to get the object's (possibly
+     different) url in REVISION. */
+  SVN_ERR (svn_client__repos_locations (&url, &new_rev,
+                                        &ignored_url, &ignored_rev,
+                                        path_or_url, &peg_revision,
+                                        /* search range: */
+                                        &start_rev, &dead_end_rev,
+                                        ctx, pool));
+  good_rev = new_rev;
 
-  SVN_ERR (svn_client__open_ra_session (&session, ra_lib, url,
+  SVN_ERR (svn_client__open_ra_session (&ra_session, url,
                                         NULL, NULL, NULL, FALSE, FALSE,
                                         ctx, pool));
 
   /* Resolve good_rev into a real revnum. */
-  SVN_ERR (svn_client__get_revision_number (&rev, ra_lib, session,
+  SVN_ERR (svn_client__get_revision_number (&rev, ra_session,
                                             good_rev, url, pool));
   if (! SVN_IS_VALID_REVNUM (rev))
-    SVN_ERR (ra_lib->get_latest_revnum (session, &rev, pool));
+    SVN_ERR (svn_ra_get_latest_revnum (ra_session, &rev, pool));
 
-  *ra_lib_p = ra_lib;
-  *session_p = session;
+  *ra_session_p = ra_session;
   *rev_p = rev;
   *url_p = url;
 
