@@ -2,7 +2,7 @@
  * svndiff.c -- Encoding and decoding svndiff-format deltas.
  * 
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -23,6 +23,7 @@
 #include "svn_io.h"
 #include "delta.h"
 #include "svn_pools.h"
+#include "svn_private_config.h"
 
 #define NORMAL_BITS 7
 #define LENGTH_BITS 5
@@ -248,9 +249,28 @@ struct decode_baton
    file for more detail on the encoding format.  */
 
 static const unsigned char *
-decode_int (svn_filesize_t *val,
-            const unsigned char *p,
-            const unsigned char *end)
+decode_file_offset (svn_filesize_t *val,
+                    const unsigned char *p,
+                    const unsigned char *end)
+{
+  /* Decode bytes until we're done.  */
+  *val = 0;
+  while (p < end)
+    {
+      *val = (*val << 7) | (*p & 0x7f);
+      if (((*p++ >> 7) & 0x1) == 0)
+        return p;
+    }
+  return NULL;
+}
+
+
+/* Same as above, only decide into a size variable. */
+
+static const unsigned char *
+decode_size (apr_size_t *val,
+             const unsigned char *p,
+             const unsigned char *end)
 {
   /* Decode bytes until we're done.  */
   *val = 0;
@@ -273,8 +293,6 @@ decode_instruction (svn_txdelta_op_t *op,
                     const unsigned char *p,
                     const unsigned char *end)
 {
-  svn_filesize_t val;
-
   if (p == end)
     return NULL;
 
@@ -291,17 +309,15 @@ decode_instruction (svn_txdelta_op_t *op,
   op->length = *p++ & 0x3f;
   if (op->length == 0)
     {
-      p = decode_int (&val, p, end);
+      p = decode_size (&op->length, p, end);
       if (p == NULL)
         return NULL;
-      op->length = (apr_size_t) val; /* FIXME: Decode to apr_size_t! */
     }
   if (op->action_code != svn_txdelta_new)
     {
-      p = decode_int (&val, p, end);
+      p = decode_size (&op->offset, p, end);
       if (p == NULL)
         return NULL;
-      op->offset = (apr_size_t) val; /* FIXME: Decode to apr_size_t! */
     }
 
   return p;
@@ -320,31 +336,26 @@ count_and_verify_instructions (int *ninst,
                                apr_size_t new_len)
 {
   int n = 0;
-  svn_txdelta_op_t op = { 0 };
+  svn_txdelta_op_t op;
   apr_size_t tpos = 0, npos = 0;
 
   while (p < end)
     {
       p = decode_instruction (&op, p, end);
-      if (p == NULL || op.offset < 0 || op.length <= 0
-          || op.length > tview_len - tpos)
+      if (p == NULL || op.length <= 0 || op.length > tview_len - tpos)
         {
           if (p == NULL)
             return svn_error_createf
               (SVN_ERR_SVNDIFF_INVALID_OPS, NULL,
-               "insn %d cannot be decoded", n);
-          else if (op.offset < 0)
-            return svn_error_createf
-              (SVN_ERR_SVNDIFF_INVALID_OPS, NULL,
-               "insn %d has negative offset", n);
+               _("Invalid diff stream: insn %d cannot be decoded"), n);
           else if (op.length <= 0)
             return svn_error_createf
               (SVN_ERR_SVNDIFF_INVALID_OPS, NULL,
-               "insn %d has non-positive length", n);
+               _("Invalid diff stream: insn %d has non-positive length"), n);
           else
             return svn_error_createf
               (SVN_ERR_SVNDIFF_INVALID_OPS, NULL,
-               "insn %d overflows the target view", n);
+               _("Invalid diff stream: insn %d overflows the target view"), n);
         }
 
       switch (op.action_code)
@@ -353,19 +364,22 @@ count_and_verify_instructions (int *ninst,
           if (op.length > sview_len - op.offset)
             return svn_error_createf
               (SVN_ERR_SVNDIFF_INVALID_OPS, NULL,
-               "[src] insn %d overflows the source view", n);
+               _("Invalid diff stream: "
+                 "[src] insn %d overflows the source view"), n);
           break;
         case svn_txdelta_target:
           if (op.offset >= tpos)
             return svn_error_createf
               (SVN_ERR_SVNDIFF_INVALID_OPS, NULL,
-               "[tgt] insn %d starts beyond the target view position", n);
+               _("Invalid diff stream: "
+                 "[tgt] insn %d starts beyond the target view position"), n);
           break;
         case svn_txdelta_new:
           if (op.length > new_len - npos)
             return svn_error_createf
               (SVN_ERR_SVNDIFF_INVALID_OPS, NULL,
-               "[new] insn %d overflows the new data section", n);
+               _("Invalid diff stream: "
+                 "[new] insn %d overflows the new data section"), n);
           npos += op.length;
           break;
         }
@@ -374,15 +388,67 @@ count_and_verify_instructions (int *ninst,
     }
   if (tpos != tview_len)
     return svn_error_create (SVN_ERR_SVNDIFF_INVALID_OPS, NULL,
-                             "delta does not fill the target window");
+                             _("Delta does not fill the target window"));
   if (npos != new_len)
     return svn_error_create (SVN_ERR_SVNDIFF_INVALID_OPS, NULL,
-                             "delta does not contain enough new data");
+                             _("Delta does not contain enough new data"));
 
   *ninst = n;
   return SVN_NO_ERROR;
 }
 
+/* Given the five integer fields of a window header and a pointer to
+   the remainder of the window contents, fill in a delta window
+   structure *WINDOW.  New allocations will be performed in POOL;
+   the new_data field of *WINDOW will refer directly to memory pointed
+   to by DATA. */
+static svn_error_t *
+decode_window (svn_txdelta_window_t *window, svn_filesize_t sview_offset,
+               apr_size_t sview_len, apr_size_t tview_len, apr_size_t inslen,
+               apr_size_t newlen, const unsigned char *data, apr_pool_t *pool)
+{
+  const unsigned char *end;
+  int ninst;
+  apr_size_t npos;
+  svn_txdelta_op_t *ops, *op;
+  svn_string_t *new_data;
+
+  window->sview_offset = sview_offset;
+  window->sview_len = sview_len;
+  window->tview_len = tview_len;
+
+  /* Count the instructions and make sure they are all valid.  */
+  end = data + inslen;
+  SVN_ERR (count_and_verify_instructions (&ninst, data, end, sview_len, 
+                                          tview_len, newlen));
+
+  /* Allocate a buffer for the instructions and decode them. */
+  ops = apr_palloc (pool, ninst * sizeof (*ops));
+  npos = 0;
+  window->src_ops = 0;
+  for (op = ops; op < ops + ninst; op++)
+    {
+      data = decode_instruction (op, data, end);
+      if (op->action_code == svn_txdelta_source)
+        ++window->src_ops;
+      else if (op->action_code == svn_txdelta_new)
+        {
+          op->offset = npos;
+          npos += op->length;
+        }
+    }
+
+  window->ops = ops;
+  window->num_ops = ninst;
+
+  new_data = apr_palloc (pool, sizeof (*new_data));
+  new_data->data = (const char *) data;
+  new_data->len = newlen;
+  window->new_data = new_data;
+
+  return SVN_NO_ERROR;
+}
+
 static svn_error_t *
 write_handler (void *baton,
                const char *buffer,
@@ -390,11 +456,9 @@ write_handler (void *baton,
 {
   struct decode_baton *db = (struct decode_baton *) baton;
   const unsigned char *p, *end;
-  svn_filesize_t val, sview_offset;
-  apr_size_t sview_len, tview_len, inslen, newlen, remaining, npos;
+  svn_filesize_t sview_offset;
+  apr_size_t sview_len, tview_len, inslen, newlen, remaining;
   apr_size_t buflen = *len;
-  svn_txdelta_op_t *op;
-  int ninst;
 
   /* Chew up four bytes at the beginning for the header.  */
   if (db->header_bytes < 4)
@@ -403,7 +467,8 @@ write_handler (void *baton,
       if (nheader > buflen)
         nheader = buflen;
       if (memcmp (buffer, "SVN\0" + db->header_bytes, nheader) != 0)
-        return svn_error_create (SVN_ERR_SVNDIFF_INVALID_HEADER, NULL, "svndiff has invalid header");
+        return svn_error_create (SVN_ERR_SVNDIFF_INVALID_HEADER, NULL,
+                                 _("Svndiff has invalid header"));
       buflen -= nheader;
       buffer += nheader;
       db->header_bytes += nheader;
@@ -427,103 +492,62 @@ write_handler (void *baton,
   while (1)
     {
       apr_pool_t *newpool;
-      svn_txdelta_window_t window = { 0 };
-      svn_string_t new_data;
-      svn_txdelta_op_t *ops;
+      svn_txdelta_window_t window;
 
       /* Read the header, if we have enough bytes for that.  */
       p = (const unsigned char *) db->buffer->data;
       end = (const unsigned char *) db->buffer->data + db->buffer->len;
 
-      p = decode_int (&val, p, end);
+      p = decode_file_offset (&sview_offset, p, end);
       if (p == NULL)
         return SVN_NO_ERROR;
-      sview_offset = val;
 
-      p = decode_int (&val, p, end);
+      p = decode_size (&sview_len, p, end);
       if (p == NULL)
         return SVN_NO_ERROR;
-      sview_len = (apr_size_t) val; /* FIXME: Decode to apr_size_t! */
 
-      p = decode_int (&val, p, end);
+      p = decode_size (&tview_len, p, end);
       if (p == NULL)
         return SVN_NO_ERROR;
-      tview_len = (apr_size_t) val; /* FIXME: Decode to apr_size_t! */
 
-      p = decode_int (&val, p, end);
+      p = decode_size (&inslen, p, end);
       if (p == NULL)
         return SVN_NO_ERROR;
-      inslen = (apr_size_t) val; /* FIXME: Decode to apr_size_t! */
 
-      p = decode_int (&val, p, end);
+      p = decode_size (&newlen, p, end);
       if (p == NULL)
         return SVN_NO_ERROR;
-      newlen = (apr_size_t) val; /* FIXME: Decode to apr_size_t! */
 
-      /* Check for integer overflow (don't want to let the input trick
-         us into invalid pointer games using negative numbers).  */
-      /* FIXME: Some of these are apr_size_t, which is
-         unsigned. Should they be apr_ptrdiff_t instead? --xbc */
-      if (sview_offset < 0 || sview_len < 0 || tview_len < 0 || inslen < 0
-          || newlen < 0 || inslen + newlen < 0 || sview_offset + sview_len < 0)
+      /* Check for integer overflow.  */
+      if (sview_offset < 0 || inslen + newlen < inslen
+          || sview_len + tview_len < sview_len
+          || sview_offset + sview_len < sview_offset)
         return svn_error_create (SVN_ERR_SVNDIFF_CORRUPT_WINDOW, NULL, 
-                                 "svndiff contains corrupt window header");
+                                 _("Svndiff contains corrupt window header"));
 
       /* Check for source windows which slide backwards.  */
       if (sview_len > 0
           && (sview_offset < db->last_sview_offset
               || (sview_offset + sview_len
                   < db->last_sview_offset + db->last_sview_len)))
-        return svn_error_create (SVN_ERR_SVNDIFF_BACKWARD_VIEW, NULL, 
-                                 "svndiff has backwards-sliding source views");
+        return svn_error_create
+          (SVN_ERR_SVNDIFF_BACKWARD_VIEW, NULL, 
+           _("Svndiff has backwards-sliding source views"));
 
       /* Wait for more data if we don't have enough bytes for the
          whole window.  */
       if ((apr_size_t) (end - p) < inslen + newlen)
         return SVN_NO_ERROR;
 
-      /* Count the instructions and make sure they are all valid.  */
-      end = p + inslen;
-      SVN_ERR (count_and_verify_instructions (&ninst, p, end, sview_len, 
-                                              tview_len, newlen));
-
-      /* Build the window structure.  */
-      window.sview_offset = sview_offset;
-      window.sview_len = sview_len;
-      window.tview_len = tview_len;
-
-      ops = apr_palloc (db->subpool, ninst * sizeof (*ops));
-      npos = 0;
-      for (op = ops; op < ops + ninst; op++)
-        {
-          /* ### Why don't we use a build baton and svn_txdelta__make_window
-                 like everyone else?  --xbc */
-          /* FIXME: The way things stand now, every svndiff insn is decoded
-             twice. We should integrate what count_and_verify_instructions
-             does here, instead.  --xbc */
-          p = decode_instruction (op, p, end);
-          if (op->action_code == svn_txdelta_source)
-            ++window.src_ops;
-          else if (op->action_code == svn_txdelta_new)
-            {
-              op->offset = npos;
-              npos += op->length;
-            }
-        }
-      window.num_ops = ninst;
-      window.ops = ops;
-
-      new_data.data = (const char *)p;
-      new_data.len = newlen;
-      window.new_data = &new_data;
-
-      /* Send it off.  */
-      SVN_ERR(db->consumer_func (&window, db->consumer_baton));
+      /* Decode the window and send it off. */
+      SVN_ERR (decode_window (&window, sview_offset, sview_len, tview_len,
+                              inslen, newlen, p, db->subpool));
+      SVN_ERR (db->consumer_func (&window, db->consumer_baton));
 
       /* Make a new subpool and buffer, saving aside the remaining
          data in the old buffer.  */
       newpool = svn_pool_create (db->pool);
-      p += newlen;
+      p += inslen + newlen;
       remaining = db->buffer->data + db->buffer->len - (const char *) p;
       db->buffer = 
         svn_stringbuf_ncreate ((const char *) p, remaining, newpool);
@@ -556,7 +580,7 @@ close_handler (void *baton)
   if ((db->error_on_early_close)
       && (db->header_bytes < 4 || db->buffer->len != 0))
     return svn_error_create (SVN_ERR_SVNDIFF_UNEXPECTED_END, NULL,
-                             "unexpected end of svndiff input");
+                             _("Unexpected end of svndiff input"));
 
   /* Tell the window consumer that we're done, and clean up.  */
   err = db->consumer_func (NULL, db->consumer_baton);
@@ -589,3 +613,114 @@ svn_txdelta_parse_svndiff (svn_txdelta_window_handler_t handler,
   svn_stream_set_close (stream, close_handler);
   return stream;
 }
+
+
+/* Routines for reading one svndiff window at a time. */
+
+/* Read one byte from STREAM into *BYTE. */
+static svn_error_t *
+read_one_byte (unsigned char *byte, svn_stream_t *stream)
+{
+  char c;
+  apr_size_t len = 1;
+
+  SVN_ERR (svn_stream_read (stream, &c, &len));
+  if (len == 0)
+    return svn_error_create (SVN_ERR_SVNDIFF_UNEXPECTED_END, NULL,
+                             _("Unexpected end of svndiff input"));
+  *byte = (unsigned char) c;
+  return SVN_NO_ERROR;
+}
+
+/* Read and decode one integer from STREAM into *SIZE. */
+static svn_error_t *
+read_one_size (apr_size_t *size, svn_stream_t *stream)
+{
+  unsigned char c;
+
+  *size = 0;
+  while (1)
+    {
+      SVN_ERR (read_one_byte (&c, stream));
+      *size = (*size << 7) | (c & 0x7f);
+      if (!(c & 0x80))
+        break;
+    }
+  return SVN_NO_ERROR;
+}
+
+/* Read a window header from STREAM and check it for integer overflow. */
+static svn_error_t *
+read_window_header (svn_stream_t *stream, svn_filesize_t *sview_offset,
+                    apr_size_t *sview_len, apr_size_t *tview_len,
+                    apr_size_t *inslen, apr_size_t *newlen)
+{
+  unsigned char c;
+
+  /* Read the source view offset by hand, since it's not an apr_size_t. */
+  *sview_offset = 0;
+  while (1)
+    {
+      SVN_ERR (read_one_byte (&c, stream));
+      *sview_offset = (*sview_offset << 7) | (c & 0x7f);
+      if (!(c & 0x80))
+        break;
+    }
+
+  /* Read the four size fields. */
+  SVN_ERR (read_one_size (sview_len, stream));
+  SVN_ERR (read_one_size (tview_len, stream));
+  SVN_ERR (read_one_size (inslen, stream));
+  SVN_ERR (read_one_size (newlen, stream));
+
+  /* Check for integer overflow.  */
+  if (*sview_offset < 0 || *inslen + *newlen < *inslen
+      || *sview_len + *tview_len < *sview_len
+      || *sview_offset + *sview_len < *sview_offset)
+    return svn_error_create (SVN_ERR_SVNDIFF_CORRUPT_WINDOW, NULL, 
+                             _("Svndiff contains corrupt window header"));
+
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *
+svn_txdelta_read_svndiff_window (svn_txdelta_window_t **window,
+                                 svn_stream_t *stream,
+                                 int svndiff_version,
+                                 apr_pool_t *pool)
+{
+  svn_filesize_t sview_offset;
+  apr_size_t sview_len, tview_len, inslen, newlen, len;
+  unsigned char *buf;
+
+  SVN_ERR (read_window_header (stream, &sview_offset, &sview_len, &tview_len,
+                               &inslen, &newlen));
+  len = inslen + newlen;
+  buf = apr_palloc(pool, len);
+  SVN_ERR (svn_stream_read (stream, (char*)buf, &len));
+  if (len < inslen + newlen)
+    return svn_error_create (SVN_ERR_SVNDIFF_UNEXPECTED_END, NULL,
+                             _("Unexpected end of svndiff input"));
+  *window = apr_palloc (pool, sizeof(**window));
+  SVN_ERR (decode_window (*window, sview_offset, sview_len, tview_len, inslen,
+                          newlen, buf, pool));
+  return SVN_NO_ERROR;
+}
+
+
+svn_error_t *
+svn_txdelta_skip_svndiff_window (apr_file_t *file,
+                                 int svndiff_version,
+                                 apr_pool_t *pool)
+{
+  svn_stream_t *stream = svn_stream_from_aprfile(file, pool);
+  svn_filesize_t sview_offset;
+  apr_size_t sview_len, tview_len, inslen, newlen;
+  apr_off_t offset;
+
+  SVN_ERR (read_window_header (stream, &sview_offset, &sview_len, &tview_len,
+                               &inslen, &newlen));
+
+  offset = inslen + newlen;
+  return svn_io_file_seek (file, APR_CUR, &offset, pool);
+}
