@@ -25,7 +25,6 @@
 
 #include "svn_delta.h"
 #include "svn_io.h"
-#include "svn_md5.h"
 #include "svn_pools.h"
 #include "delta.h"
 
@@ -51,6 +50,24 @@ struct svn_txdelta_stream_t {
 };
 
 
+/* Target-push stream descriptor. */
+
+struct tpush_baton {
+  /* These are copied from parameters passed to svn_txdelta_target_push. */
+  svn_stream_t *source;
+  svn_txdelta_window_handler_t wh;
+  void *whb;
+  apr_pool_t *pool;
+
+  /* Private data */
+  char *buf;
+  svn_filesize_t source_offset;
+  apr_size_t source_len;
+  svn_boolean_t source_done;
+  apr_size_t target_len;
+};
+
+
 /* Text delta applicator.  */
 
 struct apply_baton {
@@ -107,6 +124,34 @@ svn_txdelta__make_window (const svn_txdelta__ops_baton_t *build_baton,
 }
 
 
+/* Compute and return a delta window using the vdelta or xdelta algorithm on
+   DATA, which contains SOURCE_LEN bytes of source data and TARGET_LEN
+   bytes of target data.  SOURCE_OFFSET gives the offset of the source
+   data, and is simply copied into the window's sview_offset field. */
+static svn_txdelta_window_t *
+compute_window (const char *data, apr_size_t source_len, apr_size_t target_len,
+                svn_filesize_t source_offset, apr_pool_t *pool)
+{
+  svn_txdelta__ops_baton_t build_baton = { 0 };
+  svn_txdelta_window_t *window;
+
+  /* Compute the delta operations. */
+  build_baton.new_data = svn_stringbuf_create ("", pool);
+
+  if (source_len == 0)
+    svn_txdelta__vdelta (&build_baton, data, source_len, target_len, pool);
+  else
+    svn_txdelta__xdelta (&build_baton, data, source_len, target_len, pool);
+  
+  /* Create and return the delta window. */
+  window = svn_txdelta__make_window (&build_baton, pool);
+  window->sview_offset = source_offset;
+  window->sview_len = source_len;
+  window->tview_len = target_len;
+  return window;
+}
+
+
 
 svn_txdelta_window_t *
 svn_txdelta__copy_window (const svn_txdelta_window_t *window,
@@ -221,7 +266,7 @@ svn_txdelta (svn_txdelta_stream_t **stream,
   (*stream)->target = target;
   (*stream)->more = TRUE;
   (*stream)->pos = 0;
-  (*stream)->buf = apr_palloc (pool, 2 * SVN_STREAM_CHUNK_SIZE);
+  (*stream)->buf = apr_palloc (pool, 2 * SVN_DELTA_WINDOW_SIZE);
 
   /* Initialize MD5 digest calculation. */
   apr_md5_init (&((*stream)->context));
@@ -256,9 +301,8 @@ svn_txdelta_next_window (svn_txdelta_window_t **window,
                          apr_pool_t *pool)
 {
   svn_error_t *err;
-  apr_size_t source_len = SVN_STREAM_CHUNK_SIZE;
-  apr_size_t target_len = SVN_STREAM_CHUNK_SIZE;
-  svn_txdelta__ops_baton_t build_baton = { 0 };
+  apr_size_t source_len = SVN_DELTA_WINDOW_SIZE;
+  apr_size_t target_len = SVN_DELTA_WINDOW_SIZE;
   
   /* Read the source stream. */
   err = svn_stream_read (stream->source, stream->buf, &source_len);
@@ -289,18 +333,9 @@ svn_txdelta_next_window (svn_txdelta_window_t **window,
       apr_md5_update (&(stream->context), stream->buf + source_len,
                       target_len);
     }
-  
-  /* Compute the delta operations. */
-  build_baton.new_data = svn_stringbuf_create ("", pool);
-  svn_txdelta__vdelta (&build_baton, stream->buf,
-                       source_len, target_len,
-                       pool);
-  
-  /* Create the delta window. */
-  *window = svn_txdelta__make_window (&build_baton, pool);
-  (*window)->sview_offset = stream->pos - source_len;
-  (*window)->sview_len = source_len;
-  (*window)->tview_len = target_len;
+
+  *window = compute_window(stream->buf, source_len, target_len,
+                           stream->pos - source_len, pool);
   
   /* That's it. */
   return SVN_NO_ERROR;
@@ -319,6 +354,110 @@ svn_txdelta_md5_digest (svn_txdelta_stream_t *stream)
 }
 
 
+
+/* Functions for implementing a "target push" delta. */
+
+/* This is the write handler for a target-push delta stream.  It reads
+ * source data, buffers target data, and fires off delta windows when
+ * the target data buffer is full. */
+static svn_error_t *
+tpush_write_handler (void *baton, const char *data, apr_size_t *len)
+{
+  struct tpush_baton *tb = baton;
+  apr_size_t chunk_len, data_len = *len;
+  apr_pool_t *pool = svn_pool_create (tb->pool);
+  svn_txdelta_window_t *window;
+
+  while (data_len > 0)
+    {
+      svn_pool_clear (pool);
+
+      /* Make sure we're all full up on source data, if possible. */
+      if (tb->source_len == 0 && !tb->source_done)
+        {
+          tb->source_len = SVN_DELTA_WINDOW_SIZE;
+          SVN_ERR (svn_stream_read (tb->source, tb->buf, &tb->source_len));
+          if (tb->source_len < SVN_DELTA_WINDOW_SIZE)
+            tb->source_done = TRUE;
+        }
+
+      /* Copy in the target data, up to SVN_DELTA_WINDOW_SIZE. */
+      chunk_len = SVN_DELTA_WINDOW_SIZE - tb->target_len;
+      if (chunk_len > data_len)
+        chunk_len = data_len;
+      memcpy (tb->buf + tb->source_len + tb->target_len, data, chunk_len);
+      data += chunk_len;
+      data_len -= chunk_len;
+      tb->target_len += chunk_len;
+
+      /* If we're full of target data, compute and fire off a window. */
+      if (tb->target_len == SVN_DELTA_WINDOW_SIZE)
+        {
+          window = compute_window (tb->buf, tb->source_len, tb->target_len,
+                                   tb->source_offset, pool);
+          SVN_ERR (tb->wh (window, tb->whb));
+          tb->source_offset += tb->source_len;
+          tb->source_len = 0;
+          tb->target_len = 0;
+        }
+    }
+
+  svn_pool_destroy(pool);
+  return SVN_NO_ERROR;
+}
+
+
+/* This is the close handler for a target-push delta stream.  It sends
+ * a final window if there is any buffered target data, and then sends
+ * a NULL window signifying the end of the window stream. */
+static svn_error_t *
+tpush_close_handler (void *baton)
+{
+  struct tpush_baton *tb = baton;
+  svn_txdelta_window_t *window;
+
+  /* Send a final window if we have any residual target data. */
+  if (tb->target_len > 0)
+    {
+      window = compute_window(tb->buf, tb->source_len, tb->target_len,
+                              tb->source_offset, tb->pool);
+      SVN_ERR(tb->wh(window, tb->whb));
+    }
+
+  /* Send a final NULL window signifying the end. */
+  SVN_ERR(tb->wh(NULL, tb->whb));
+  return SVN_NO_ERROR;
+}
+
+
+svn_stream_t *
+svn_txdelta_target_push (svn_txdelta_window_handler_t handler,
+                         void *handler_baton, svn_stream_t *source,
+                         apr_pool_t *pool)
+{
+  struct tpush_baton *tb;
+  svn_stream_t *stream;
+
+  /* Initialize baton. */
+  tb = apr_palloc (pool, sizeof(*tb));
+  tb->source = source;
+  tb->wh = handler;
+  tb->whb = handler_baton;
+  tb->pool = pool;
+  tb->buf = apr_palloc (pool, 2 * SVN_DELTA_WINDOW_SIZE);
+  tb->source_offset = 0;
+  tb->source_len = 0;
+  tb->source_done = FALSE;
+  tb->target_len = 0;
+
+  /* Create and return writable stream. */
+  stream = svn_stream_create (tb, pool);
+  svn_stream_set_write (stream, tpush_write_handler);
+  svn_stream_set_close (stream, tpush_close_handler);
+  return stream;
+}
+
+
 
 /* Functions for applying deltas.  */
 
@@ -351,7 +490,6 @@ svn_txdelta__apply_instructions (svn_txdelta_window_t *window,
                                   ? op->length : *tlen - tpos);
 
       /* Check some invariants common to all instructions.  */
-      assert (op->offset >= 0 && op->length >= 0);
       assert (tpos + op->length <= window->tview_len);
 
       switch (op->action_code)
@@ -594,14 +732,14 @@ svn_error_t *svn_txdelta_send_txstream (svn_txdelta_stream_t *txstream,
 
   do
     {
+      /* free the window (if any) */
+      svn_pool_clear (wpool);
+
       /* read in a single delta window */
       SVN_ERR( svn_txdelta_next_window (&window, txstream, wpool));
 
       /* shove it at the handler */
       SVN_ERR( (*handler)(window, handler_baton));
-
-      /* free the window (if any) */
-      svn_pool_clear (wpool);
     }
   while (window != NULL);
 
