@@ -24,24 +24,15 @@
 #include <apr_pools.h>
 #include <apr_tables.h>
 #include <apr_strings.h>
-#include <apr_portable.h>
 #include <apr_xml.h>
 
 #include <ne_socket.h>
-#include <ne_basic.h>
-#include <ne_utils.h>
-#include <ne_basic.h>
-#include <ne_207.h>
-#include <ne_props.h>
-#include <ne_xml.h>
 
 #include "svn_error.h"
 #include "svn_pools.h"
-#include "svn_delta.h"
-#include "svn_io.h"
-#include "svn_ra.h"
 #include "svn_path.h"
 #include "svn_xml.h"
+#include "../libsvn_ra/ra_loader.h"
 
 #include "ra_dav.h"
 
@@ -79,6 +70,9 @@ struct log_baton
   svn_log_message_receiver_t receiver;
   void *receiver_baton;
 
+  int limit;
+  int count;
+
   /* If `receiver' returns error, it is stored here. */
   svn_error_t *err;
 };
@@ -112,15 +106,6 @@ log_validate(void *userdata, svn_ra_dav__xml_elmid parent,
   return SVN_RA_DAV__XML_VALID;
 }
 
-
-static const char *get_attr(const char **atts, const char *which)
-{
-  for (; atts && *atts; atts += 2)
-    if (strcmp(*atts, which) == 0)
-      return atts[1];
-  return NULL;
-}
-
 /*
  * This implements the `svn_ra_dav__xml_startelm_cb' prototype.
  */
@@ -150,8 +135,8 @@ log_start_element(void *userdata,
         {
           lb->this_path_item->action 
             = (elm->id == ELEM_added_path) ? 'A' : 'R';
-          copyfrom_path = get_attr(atts, "copyfrom-path");
-          copyfrom_revstr = get_attr(atts, "copyfrom-rev");
+          copyfrom_path = svn_xml_get_attr_value("copyfrom-path", atts);
+          copyfrom_revstr = svn_xml_get_attr_value("copyfrom-rev", atts);
           if (copyfrom_path && copyfrom_revstr
               && (SVN_IS_VALID_REVNUM
                   (copyfrom_rev = SVN_STR_TO_REV (copyfrom_revstr))))
@@ -217,14 +202,28 @@ log_end_element(void *userdata,
       break;
     case ELEM_log_item:
       {
-        svn_error_t *err = (*(lb->receiver))(lb->receiver_baton,
+        svn_error_t *err;
+        /* Compatability cruft so that we can provide limit functionality 
+           even if the server doesn't support it.
+
+           If we've seen as many log entries as we're going to show just
+           error out of the XML parser so we can avoid having to parse the
+           remaining XML, but set lb->err to SVN_NO_ERROR so no error will
+           end up being shown to the user. */
+        if (lb->limit && (++lb->count > lb->limit))
+          {
+            lb->err = SVN_NO_ERROR;
+            return SVN_RA_DAV__XML_INVALID;
+          }
+ 
+        err = (*(lb->receiver))(lb->receiver_baton,
                                              lb->changed_paths,
                                              lb->revision,
                                              lb->author,
                                              lb->date,
                                              lb->msg,
                                              lb->subpool);
-        
+
         reset_log_item (lb);
         
         if (err)
@@ -294,10 +293,11 @@ log_end_element(void *userdata,
 }
 
 
-svn_error_t * svn_ra_dav__get_log(void *session_baton,
+svn_error_t * svn_ra_dav__get_log(svn_ra_session_t *session,
                                   const apr_array_header_t *paths,
                                   svn_revnum_t start,
                                   svn_revnum_t end,
+                                  int limit,
                                   svn_boolean_t discover_changed_paths,
                                   svn_boolean_t strict_node_history,
                                   svn_log_message_receiver_t receiver,
@@ -316,7 +316,7 @@ svn_error_t * svn_ra_dav__get_log(void *session_baton,
    */
 
   int i;
-  svn_ra_session_t *ras = session_baton;
+  svn_ra_dav__session_t *ras = session->priv;
   svn_stringbuf_t *request_body = svn_stringbuf_create("", ras->pool);
   struct log_baton lb;
   svn_string_t bc_url, bc_relative;
@@ -365,6 +365,13 @@ svn_error_t * svn_ra_dav__get_log(void *session_baton,
                            apr_psprintf(ras->pool,
                                         "<S:end-revision>%ld"
                                         "</S:end-revision>", end));
+  if (limit)
+    {
+      svn_stringbuf_appendcstr(request_body,
+                               apr_psprintf(ras->pool,
+                                            "<S:limit>%d</S:limit>", limit));
+    }
+
   if (discover_changed_paths)
     {
       svn_stringbuf_appendcstr(request_body,
@@ -379,13 +386,18 @@ svn_error_t * svn_ra_dav__get_log(void *session_baton,
                                             "<S:strict-node-history/>"));
     }
 
-  for (i = 0; i < paths->nelts; i++)
+  if (paths)
     {
-      const char *this_path =
-        apr_xml_quote_string(ras->pool, ((const char **)paths->elts)[i], 0);
-      svn_stringbuf_appendcstr(request_body, "<S:path>");
-      svn_stringbuf_appendcstr(request_body, this_path);
-      svn_stringbuf_appendcstr(request_body, "</S:path>");
+      for (i = 0; i < paths->nelts; i++)
+        {
+          const char *this_path =
+            apr_xml_quote_string(ras->pool,
+                                 ((const char **)paths->elts)[i],
+                                 0);
+          svn_stringbuf_appendcstr(request_body, "<S:path>");
+          svn_stringbuf_appendcstr(request_body, this_path);
+          svn_stringbuf_appendcstr(request_body, "</S:path>");
+        }
     }
 
   svn_stringbuf_appendcstr(request_body, log_request_tail);
@@ -394,6 +406,8 @@ svn_error_t * svn_ra_dav__get_log(void *session_baton,
   lb.receiver_baton = receiver_baton;
   lb.subpool = svn_pool_create (ras->pool);
   lb.err = NULL;
+  lb.limit = limit;
+  lb.count = 0;
   reset_log_item (&lb);
 
   /* ras's URL may not exist in HEAD, and thus it's not safe to send
@@ -422,6 +436,7 @@ svn_error_t * svn_ra_dav__get_log(void *session_baton,
                                              &lb,
                                              NULL, 
                                              NULL,
+                                             FALSE,
                                              ras->pool) );
 
   if (lb.err)
