@@ -24,7 +24,6 @@
 #include <apr_want.h>
 
 #include <ne_socket.h>
-#include <ne_xml.h>
 #include <ne_request.h>
 
 
@@ -33,6 +32,8 @@
 #include "svn_path.h"
 #include "svn_ra.h"
 #include "svn_pools.h"
+#include "svn_props.h"
+#include "svn_xml.h"
 
 #include "svn_private_config.h"
 
@@ -540,20 +541,119 @@ static int end_element(void *userdata, const svn_ra_dav__xml_elm_t *elm,
   return SVN_RA_DAV__XML_VALID;
 }
 
+
+svn_error_t * svn_ra_dav__assemble_locktoken_body(svn_stringbuf_t **body,
+                                                  apr_hash_t *lock_tokens,
+                                                  apr_pool_t *pool)
+{
+  apr_hash_index_t *hi;
+  apr_size_t buf_size;
+  const char *closing_tag = "</S:lock-token-list>";
+  apr_size_t closing_tag_size = strlen(closing_tag);
+  svn_stringbuf_t *lockbuf = 
+    svn_stringbuf_create
+    ("<S:lock-token-list xmlns:S=\"" SVN_XML_NAMESPACE "\">", pool);
+
+  buf_size = lockbuf->len;
+
+#define SVN_LOCK "<S:lock>"
+#define SVN_LOCK_LEN sizeof(SVN_LOCK)-1
+#define SVN_LOCK_CLOSE "</S:lock>"
+#define SVN_LOCK_CLOSE_LEN sizeof(SVN_LOCK_CLOSE)-1
+#define SVN_LOCK_PATH "<S:lock-path>"
+#define SVN_LOCK_PATH_LEN sizeof(SVN_LOCK_PATH)-1
+#define SVN_LOCK_PATH_CLOSE "</S:lock-path>"
+#define SVN_LOCK_PATH_CLOSE_LEN sizeof(SVN_LOCK_CLOSE)-1
+#define SVN_LOCK_TOKEN "<S:lock-token>"
+#define SVN_LOCK_TOKEN_LEN sizeof(SVN_LOCK_TOKEN)-1
+#define SVN_LOCK_TOKEN_CLOSE "</S:lock-token>"
+#define SVN_LOCK_TOKEN_CLOSE_LEN sizeof(SVN_LOCK_TOKEN_CLOSE)-1
+  
+  /* First, figure out how much string data we're talking about,
+     and allocate a stringbuf big enough to hold it all... we *never*
+     want have the stringbuf do an auto-reallocation.  */
+  for (hi = apr_hash_first(pool, lock_tokens); hi; hi = apr_hash_next(hi))
+    {
+      const void *key;
+      void *val;
+      apr_ssize_t klen;
+      
+      apr_hash_this(hi, &key, &klen, &val);
+      
+      buf_size += SVN_LOCK_LEN;
+      buf_size += SVN_LOCK_PATH_LEN;
+      buf_size += klen;
+      buf_size += SVN_LOCK_PATH_CLOSE_LEN;
+      buf_size += SVN_LOCK_TOKEN_LEN;
+      buf_size += strlen(val);
+      buf_size += SVN_LOCK_TOKEN_CLOSE_LEN;
+      buf_size += SVN_LOCK_CLOSE_LEN;
+    }
+  
+  buf_size += closing_tag_size;
+  
+  svn_stringbuf_ensure(lockbuf, buf_size + 1);
+  
+  /* Now append all the hash's keys and values into the stringbuf.
+     This is better than doing apr_pstrcat() in a loop, because
+     (1) there's no need to constantly re-alloc, and (2) the
+     stringbuf already knows the end of the buffer, so there's no
+     seek-time to the end of the string when appending. */
+  for (hi = apr_hash_first(pool, lock_tokens); hi; hi = apr_hash_next(hi))
+    {
+      const void *key;
+      void *val;
+      
+      apr_hash_this(hi, &key, NULL, &val);
+      
+      svn_stringbuf_appendcstr(lockbuf, SVN_LOCK);
+      svn_stringbuf_appendcstr(lockbuf, SVN_LOCK_PATH);
+      svn_stringbuf_appendcstr(lockbuf, key);
+      svn_stringbuf_appendcstr(lockbuf, SVN_LOCK_PATH_CLOSE);
+      svn_stringbuf_appendcstr(lockbuf, SVN_LOCK_TOKEN);
+      svn_stringbuf_appendcstr(lockbuf, val);
+      svn_stringbuf_appendcstr(lockbuf, SVN_LOCK_TOKEN_CLOSE);
+      svn_stringbuf_appendcstr(lockbuf, SVN_LOCK_CLOSE);
+    }
+  
+  svn_stringbuf_appendcstr(lockbuf, closing_tag);
+
+#undef SVN_LOCK
+#undef SVN_LOCK_LEN
+#undef SVN_LOCK_CLOSE
+#undef SVN_LOCK_CLOSE_LEN
+#undef SVN_LOCK_PATH
+#undef SVN_LOCK_PATH_LEN
+#undef SVN_LOCK_PATH_CLOSE
+#undef SVN_LOCK_PATH_CLOSE_LEN
+#undef SVN_LOCK_TOKEN
+#undef SVN_LOCK_TOKEN_LEN
+#undef SVN_LOCK_TOKEN_CLOSE
+#undef SVN_LOCK_TOKEN_CLOSE_LEN
+
+  *body = lockbuf;
+  return SVN_NO_ERROR;
+}
+
+
+
 svn_error_t * svn_ra_dav__merge_activity(
     svn_revnum_t *new_rev,
     const char **committed_date,
     const char **committed_author,
-    svn_ra_session_t *ras,
+    svn_ra_dav__session_t *ras,
     const char *repos_url,
     const char *activity_url,
     apr_hash_t *valid_targets,
+    apr_hash_t *lock_tokens,
+    svn_boolean_t keep_locks,
     svn_boolean_t disable_merge_response,
     apr_pool_t *pool)
 {
   merge_ctx_t mc = { 0 };
   const char *body;
   apr_hash_t *extra_headers = NULL;
+  svn_stringbuf_t *lockbuf = svn_stringbuf_create("", pool);
 
   mc.pool = pool;
   mc.scratchpool = svn_pool_create (pool);
@@ -571,13 +671,29 @@ svn_error_t * svn_ra_dav__merge_activity(
   mc.committed_date = MAKE_BUFFER(pool);
   mc.last_author = MAKE_BUFFER(pool);
 
-  if (disable_merge_response)
+  if (disable_merge_response 
+      || (! keep_locks))
     {
-      extra_headers = apr_hash_make(pool);
+      const char *value;
+
+      value = apr_psprintf(pool, "%s %s",
+                           disable_merge_response ?
+                              SVN_DAV_OPTION_NO_MERGE_RESPONSE : "",
+                           keep_locks ?
+                              "" : SVN_DAV_OPTION_RELEASE_LOCKS);
+      
+      if (! extra_headers)
+        extra_headers = apr_hash_make(pool);
       apr_hash_set (extra_headers, SVN_DAV_OPTIONS_HEADER, APR_HASH_KEY_STRING,
-                    SVN_DAV_OPTION_NO_MERGE_RESPONSE);
+                    value);
     }
 
+  /* Need to marshal the whole [path->token] hash to the server as
+     a string within the body of the MERGE request. */
+  if ((lock_tokens != NULL)
+      && (apr_hash_count(lock_tokens) > 0))
+    SVN_ERR( svn_ra_dav__assemble_locktoken_body(&lockbuf, lock_tokens, pool) );
+
   body = apr_psprintf(pool,
                       "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
                       "<D:merge xmlns:D=\"DAV:\">"
@@ -587,13 +703,15 @@ svn_error_t * svn_ra_dav__merge_activity(
                       "<D:checked-in/><D:version-name/><D:resourcetype/>"
                       "<D:creationdate/><D:creator-displayname/>"
                       "</D:prop>"
-                      "</D:merge>", activity_url);
+                      "%s"
+                      "</D:merge>",
+                      activity_url, lockbuf->data);
 
   SVN_ERR( svn_ra_dav__parsed_request_compat(ras->sess, "MERGE", repos_url,
                                              body, 0, NULL, merge_elements,
                                              validate_element, start_element,
                                              end_element, &mc, extra_headers,
-                                             NULL, pool) );
+                                             NULL, FALSE, pool) );
   
   /* is there an error stashed away in our context? */
   if (mc.err != NULL)
