@@ -23,23 +23,18 @@
 #include <assert.h>
 #include <apr_general.h>  /* for strcasecmp() */
 #include <apr_pools.h>
-#include <apr_hash.h>
 #include <apr_tables.h>
 #include <apr_file_io.h>
 #include <apr_strings.h>
-#include <apr_lib.h>
 
 #include "svn_cmdline.h"
 #include "svn_types.h"
-#include "svn_delta.h"
 #include "svn_string.h"
 #include "svn_time.h"
 #include "svn_path.h"
-#include "svn_xml.h"
 #include "svn_error.h"
 #include "svn_utf.h"
 #include "svn_io.h"
-#include "svn_hash.h"
 #include "svn_subst.h"
 #include "svn_pools.h"
 
@@ -192,11 +187,10 @@ svn_subst_build_keywords (svn_subst_keywords_t *kw,
 }
 
 
-/*** Helpers for svn_subst_translate_stream ***/
+/*** Helpers for svn_subst_translate_stream2 ***/
 
 
-/* Write out LEN bytes of BUF into STREAM, using POOL to allocate any
-   svn_error_t errors that might occur along the way. */
+/* Write out LEN bytes of BUF into STREAM. */
 static svn_error_t *
 translate_write (svn_stream_t *stream,
                  const void *buf,
@@ -239,10 +233,54 @@ translate_keyword_subst (char *buf,
 
   buf_ptr = buf + 1 + keyword_len;
 
+  /* Check for fixed-length expansion. 
+   * The format of fixed length keyword and its data is
+   * Unexpanded keyword:         "$keyword::       $"
+   * Expanded keyword:           "$keyword:: value $"
+   * Expanded kw with filling:   "$keyword:: value   $"
+   * Truncated keyword:          "$keyword:: longval#$"
+   */
+  if ((buf_ptr[0] == ':') /* first char after keyword is ':' */
+      && (buf_ptr[1] == ':') /* second char after keyword is ':' */
+      && (buf_ptr[2] == ' ') /* third char after keyword is ' ' */
+      && ((buf[*len - 2] == ' ')  /* has ' ' for next to last character */
+          || (buf[*len - 2] == '#')) /* .. or has '#' for next to last character */
+      && ((6 + keyword_len) < *len))  /* holds "$kw:: x $" at least */
+    {
+      /* This is fixed length keyword, so *len remains unchanged */
+      apr_size_t max_value_len = *len - (6 + keyword_len);
+
+      if (! value)
+        {
+            /* no value, so unexpand */
+            buf_ptr += 2;
+            while (*buf_ptr != '$')
+                *(buf_ptr++) = ' ';
+        }
+      else 
+        {
+          if (value->len <= max_value_len) 
+            { /* replacement not as long as template, pad with spaces */
+              strncpy (buf_ptr + 3, value->data, value->len);
+              buf_ptr += 3 + value->len;
+              while (*buf_ptr != '$')
+                *(buf_ptr++) = ' ';
+            }
+          else
+            {
+              /* replacement needs truncating */
+              strncpy (buf_ptr + 3, value->data, max_value_len);
+              buf[*len - 2] = '#';
+              buf[*len - 1] = '$';
+            }
+        }
+      return TRUE;
+    }
+
   /* Check for unexpanded keyword. */
-  if ((buf_ptr[0] == '$')          /* "$keyword$" */
-      || ((buf_ptr[0] == ':') 
-          && (buf_ptr[1] == '$'))) /* "$keyword:$" */
+  else if ((buf_ptr[0] == '$')          /* "$keyword$" */
+           || ((buf_ptr[0] == ':') 
+               && (buf_ptr[1] == '$'))) /* "$keyword:$" */
     {
       /* unexpanded... */
       if (value)
@@ -444,9 +482,7 @@ translate_keyword (char *buf,
    REPAIR is TRUE, ignore the inconsistency, else return an
    SVN_ERR_IO_INCONSISTENT_EOL error.  If we are examining the first
    newline in the file, copy it to {SRC_FORMAT, *SRC_FORMAT_LEN} to
-   use for later consistency checks.  
-
-   Use POOL to allocate errors that may occur. */
+   use for later consistency checks. */
 static svn_error_t *
 translate_newline (const char *eol_str,
                    apr_size_t eol_str_len,
@@ -550,14 +586,15 @@ svn_subst_keywords_differ (const svn_subst_keywords_t *a,
 
 
 svn_error_t *
-svn_subst_translate_stream (svn_stream_t *s, /* src stream */
-                            svn_stream_t *d, /* dst stream */
-                            const char *eol_str,
-                            svn_boolean_t repair,
-                            const svn_subst_keywords_t *keywords,
-                            svn_boolean_t expand)
+svn_subst_translate_stream2 (svn_stream_t *s, /* src stream */
+                             svn_stream_t *d, /* dst stream */
+                             const char *eol_str,
+                             svn_boolean_t repair,
+                             const svn_subst_keywords_t *keywords,
+                             svn_boolean_t expand,
+                             apr_pool_t *pool)
 {
-  char buf[SVN_STREAM_CHUNK_SIZE + 1];
+  char *buf;
   const char *p, *interesting;
   apr_size_t len, readlen;
   apr_size_t eol_str_len = eol_str ? strlen (eol_str) : 0;
@@ -568,14 +605,17 @@ svn_subst_translate_stream (svn_stream_t *s, /* src stream */
   char       src_format[2] = { 0 };
   apr_size_t src_format_len = 0;
 
+  buf = apr_palloc (pool, SVN_STREAM_CHUNK_SIZE + 1);
+
   /* The docstring requires that *some* translation be requested. */
   assert (eol_str || keywords);
   interesting = (eol_str && keywords) ? "$\r\n" : eol_str ? "\r\n" : "$";
 
-  readlen = sizeof (buf) - 1;
-  while (readlen == sizeof (buf) - 1)
+  readlen = SVN_STREAM_CHUNK_SIZE;
+  while (readlen == SVN_STREAM_CHUNK_SIZE)
     {
       SVN_ERR (svn_stream_read (s, buf, &readlen));
+
       buf[readlen] = '\0';
 
       /* At the beginning of this loop, assume that we might be in an
@@ -591,9 +631,11 @@ svn_subst_translate_stream (svn_stream_t *s, /* src stream */
             {
               if (*p == '\n')
                 newline_buf[newline_off++] = *p++;
+
               SVN_ERR (translate_newline (eol_str, eol_str_len, src_format,
                                           &src_format_len, newline_buf,
                                           newline_off, d, repair));
+
               newline_off = 0;
             }
           else if (keyword_off && *p == '$')
@@ -605,7 +647,9 @@ svn_subst_translate_stream (svn_stream_t *s, /* src stream */
                 p++;
               else
                 keyword_off--;
+
               SVN_ERR (translate_write (d, keyword_buf, keyword_off));
+
               keyword_off = 0;
             }
           else if (keyword_off == SVN_KEYWORD_MAX_LEN - 1
@@ -613,6 +657,7 @@ svn_subst_translate_stream (svn_stream_t *s, /* src stream */
             {
               /* No closing '$' found; flush the keyword buffer. */
               SVN_ERR (translate_write (d, keyword_buf, keyword_off));
+              
               keyword_off = 0;
             }
           else if (keyword_off)
@@ -633,6 +678,7 @@ svn_subst_translate_stream (svn_stream_t *s, /* src stream */
             }
           if (len)
             SVN_ERR (translate_write (d, p, len));
+          
           p += len;
 
           /* Set up state according to the interesting character, if any. */
@@ -646,9 +692,11 @@ svn_subst_translate_stream (svn_stream_t *s, /* src stream */
               break;
             case '\n':
               newline_buf[newline_off++] = *p++;
+
               SVN_ERR (translate_newline (eol_str, eol_str_len, src_format,
                                           &src_format_len, newline_buf,
                                           newline_off, d, repair));
+
               newline_off = 0;
               break;
             }
@@ -659,6 +707,7 @@ svn_subst_translate_stream (svn_stream_t *s, /* src stream */
     SVN_ERR (translate_newline (eol_str, eol_str_len, src_format,
                                 &src_format_len, newline_buf, newline_off, d,
                                 repair));
+
   if (keyword_off)
     SVN_ERR (translate_write (d, keyword_buf, keyword_off));
 
@@ -666,6 +715,22 @@ svn_subst_translate_stream (svn_stream_t *s, /* src stream */
 }
 
 
+svn_error_t *
+svn_subst_translate_stream (svn_stream_t *s, /* src stream */
+                            svn_stream_t *d, /* dst stream */
+                            const char *eol_str,
+                            svn_boolean_t repair,
+                            const svn_subst_keywords_t *keywords,
+                            svn_boolean_t expand)
+{
+  apr_pool_t *pool = svn_pool_create (NULL);
+  svn_error_t *err = svn_subst_translate_stream2 (s, d, eol_str, repair,
+                                                  keywords, expand, pool);
+  svn_pool_destroy (pool);
+  return err;
+}
+
+
 svn_error_t *
 svn_subst_translate_cstring (const char *src,
                              const char **dst,
@@ -694,8 +759,8 @@ svn_subst_translate_cstring (const char *src,
   dst_stream = svn_stream_from_stringbuf (dst_stringbuf, pool);
 
   /* Translate src stream into dst stream. */
-  err = svn_subst_translate_stream (src_stream, dst_stream,
-                                    eol_str, repair, keywords, expand);
+  err = svn_subst_translate_stream2 (src_stream, dst_stream,
+                                     eol_str, repair, keywords, expand, pool);
   if (err)
     {
       svn_error_clear (svn_stream_close (src_stream));
@@ -757,7 +822,7 @@ detranslate_special_file (const char *src,
        contents. */
     SVN_ERR (svn_io_file_open (&s, src, APR_READ | APR_BUFFERED,
                                APR_OS_DEFAULT, pool));
-    src_stream = svn_stream_from_aprfile (d, pool);
+    src_stream = svn_stream_from_aprfile (s, pool);
 
     SVN_ERR (svn_stream_copy (src_stream, dst_stream, pool));
     break;
@@ -936,14 +1001,15 @@ svn_subst_copy_and_translate2 (const char *src,
   dst_stream = svn_stream_from_aprfile (d, subpool);
 
   /* Translate src stream into dst stream. */
-  err = svn_subst_translate_stream (src_stream, dst_stream,
-                                    eol_str, repair, keywords, expand);
+  err = svn_subst_translate_stream2 (src_stream, dst_stream, eol_str,
+                                     repair, keywords, expand, subpool);
   if (err)
     {
       if (err->apr_err == SVN_ERR_IO_INCONSISTENT_EOL)
         err = svn_error_createf 
           (SVN_ERR_IO_INCONSISTENT_EOL, err,
-           _("File '%s' has inconsistent newlines"), src);
+           _("File '%s' has inconsistent newlines"),
+           svn_path_local_style (src, pool));
       goto error;
     }
 
