@@ -25,14 +25,10 @@
 #include <stdarg.h>
 #include <assert.h>
 #include <apr_pools.h>
-#include <apr_hash.h>
 #include <apr_file_io.h>
-#include <apr_time.h>
 #include <apr_strings.h>
 #include "svn_types.h"
-#include "svn_string.h"
 #include "svn_error.h"
-#include "svn_hash.h"
 #include "svn_io.h"
 #include "svn_path.h"
 #include "svn_wc.h"
@@ -41,6 +37,8 @@
 #include "adm_files.h"
 #include "entries.h"
 
+#include "svn_private_config.h"
+
 
 /*** File names in the adm area. ***/
 
@@ -195,7 +193,7 @@ svn_wc__make_adm_thing (svn_wc_adm_access_t *adm_access,
          segfault or other obvious indicator that something went
          wrong.  Even so, not sure if it's appropriate.  Thoughts? */
       err = svn_error_create 
-        (0, NULL, "Bad type indicator");
+        (0, NULL, _("Bad type indicator"));
     }
 
   return err;
@@ -401,8 +399,9 @@ open_adm_file (apr_file_t **handle,
         {
           /* We don't handle append.  To do so we would need to copy the
              contents into the apr_file_t once it has been opened. */
-          return svn_error_create (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-                                   "APR_APPEND not supported for adm files");
+          return svn_error_create
+            (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+             _("APR_APPEND not supported for adm files"));
         }
 
       /* Need to own the temporary file, so don't reuse an existing one. */
@@ -445,8 +444,8 @@ open_adm_file (apr_file_t **handle,
       if (APR_STATUS_IS_ENOENT(err->apr_err) && (flags & APR_WRITE))
         {
           err = svn_error_quick_wrap(err,
-                               "Your .svn/tmp directory may be missing or "
-                               "corrupt; run 'svn cleanup' and try again");
+                               _("Your .svn/tmp directory may be missing or "
+                                 "corrupt; run 'svn cleanup' and try again"));
         }
     }
 
@@ -620,14 +619,15 @@ svn_wc__open_props (apr_file_t **handle,
   if (wc_format_version == 0)
     return svn_error_createf
       (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-       "'%s' is not a working copy", parent_dir);
+       _("'%s' is not a working copy"),
+       svn_path_local_style (parent_dir, pool));
 
   /* Then examine the flags to know -which- kind of prop file to get. */
 
   if (base && wcprops)
     return svn_error_create (SVN_ERR_WC_PATH_NOT_FOUND, NULL,
-                             "No such thing as 'base' "
-                             "working copy properties!");
+                             _("No such thing as 'base' "
+                               "working copy properties!"));
 
   else if (base)
     {
@@ -695,14 +695,15 @@ svn_wc__close_props (apr_file_t *fp,
   if (wc_format_version == 0)
     return svn_error_createf
       (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-       "'%s' is not a working copy", parent_dir);
+       _("'%s' is not a working copy"),
+       svn_path_local_style (parent_dir, pool));
 
   /* Then examine the flags to know -which- kind of prop file to get. */
 
   if (base && wcprops)
     return svn_error_create (SVN_ERR_WC_PATH_NOT_FOUND, NULL,
-                             "No such thing as 'base' "
-                             "working copy properties!");
+                             _("No such thing as 'base' "
+                               "working copy properties!"));
 
   else if (base)
     {
@@ -765,8 +766,8 @@ svn_wc__sync_props (const char *path,
 
   if (base && wcprops)
     return svn_error_create (SVN_ERR_WC_PATH_NOT_FOUND, NULL,
-                             "No such thing as 'base' "
-                             "working copy properties!");
+                             _("No such thing as 'base' "
+                               "working copy properties!"));
 
   else if (base)
     {
@@ -834,7 +835,8 @@ check_adm_exists (svn_boolean_t *exists,
       /* If got an error other than dir non-existence, then
          something's weird and we should return a genuine error. */
       return svn_error_createf (APR_ENOTDIR, NULL,
-                                "'%s' is not a directory", tmp_path);
+                                _("'%s' is not a directory"),
+                                svn_path_local_style (tmp_path, pool));
     }
   else if (kind == svn_node_none)
     {
@@ -881,12 +883,14 @@ check_adm_exists (svn_boolean_t *exists,
       svn_wc_adm_access_t *adm_access;
       const svn_wc_entry_t *entry;
 
-      SVN_ERR (svn_wc_adm_open (&adm_access, NULL, path, FALSE, FALSE, pool));
+      SVN_ERR (svn_wc_adm_open3 (&adm_access, NULL, path, FALSE, 0,
+                                 NULL, NULL, pool));
       SVN_ERR (svn_wc_entry (&entry, path, adm_access, FALSE, pool));
       SVN_ERR (svn_wc_adm_close (adm_access));
       if (!entry)
         return svn_error_createf (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-                                  "No entry for '%s'", path);
+                                  _("No entry for '%s'"),
+                                  svn_path_local_style (path, pool));
 
       /* The revisions must match except when adding a directory with a
          name that matches a directory scheduled for deletion. That's
@@ -895,19 +899,20 @@ check_adm_exists (svn_boolean_t *exists,
       if (entry->revision != revision
           && !(entry->schedule == svn_wc_schedule_delete && revision == 0))
         return
-          svn_error_createf (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-                             "Revision %" SVN_REVNUM_T_FMT
-                             " doesn't match existing revision %"
-                             SVN_REVNUM_T_FMT " in '%s'",
-                             revision, entry->revision, path);
+          svn_error_createf
+          (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
+           _("Revision %ld doesn't match existing revision %ld in '%s'"),
+           revision, entry->revision,
+           svn_path_local_style (path, pool));
 
       /** ### comparing URLs, should they be canonicalized first? */
       if (strcmp (entry->url, url) != 0)
         return
           svn_error_createf
           (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-           "URL '%s' doesn't match existing URL '%s' in '%s'",
-           url, entry->url, path);
+           _("URL '%s' doesn't match existing URL '%s' in '%s'"),
+           url, entry->url,
+           svn_path_local_style (path, pool));
     }
 
   *exists = wc_exists;
