@@ -20,11 +20,7 @@
 
 #include "svn_wc.h"
 #include "svn_error.h"
-#include "svn_string.h"
-#include "svn_xml.h"
-#include "svn_pools.h"
 #include "svn_path.h"
-#include "svn_io.h"
 
 #include "wc.h"
 #include "entries.h"
