@@ -30,6 +30,7 @@
 #include "svn_time.h"
 #include "svn_dav.h"
 #include "svn_md5.h"
+#include "svn_props.h"
 
 
 /*
@@ -106,24 +107,32 @@ static const dav_liveprop_group dav_svn_liveprop_group =
     &dav_svn_hooks_liveprop
 };
 
-/* Return the revision property PROPNAME's value in PROPVAL for
-   RESOURCE in the revision COMMITTED_REV after verifying that the
-   path is readable.  If the property if inaccessible, SVN_NO_ERROR is
-   returned and *PROPVAL is NULL.
+/* Set *PROPVAL to the value for the revision property PROPNAME on
+   COMMITTED_REV, in the repository identified by RESOURCE, if
+   RESOURCE's path is readable.  If it is not readable, set *PROPVAL
+   to NULL and return SVN_NO_ERROR.  Use POOL for temporary
+   allocations and the allocation of *PROPVAL.
 
-   This function must only be used to retrieve properties for which it
-   is sufficient to have read access to a single changed path in the
-   revision to have access to the revprop, e.g.
-   SVN_PROP_REVISION_AUTHOR or SVN_PROP_REVISION_DATE.
+   Note that this function does not check the readability of the
+   revision property, but the readability of a path.  The true
+   readability of a revision property is determined by investigating
+   the readability of all changed paths in the revision.  For certain
+   revision properties (e.g. svn:author and svn:date) to be readable,
+   it is enough if at least one changed path is readable.  When we
+   already have a changed path, we can skip the check for the other
+   changed paths in the revision and save a lot of work.  This means
+   that we will make a mistake when our path is unreadable and another
+   changed path is readable, but we will at least only hide too much
+   and not leak any protected properties.
 
-   The reason for this is that we only check readability of the
-   current path (which is one of the revisions' changed paths per
-   definition).  If the current path is readable, the revprop is also
-   readable.  While it's possible that the property is readable even
-   though the current path is not readable (because another path in
-   the same revision is readable), it's a silly situation worth
-   ignoring to gain the extra performance. */
-static svn_error_t *svn_svn_get_path_revprop(svn_string_t **propval,
+   WARNING: This method of only checking the readability of a path is
+   only valid to get revision properties for which it is enough if at
+   least one changed path is readable.  Using this function to get
+   revision properties for which all changed paths must be readable
+   might leak protected information because we will only test the
+   readability of a single changed path.
+*/
+static svn_error_t *dav_svn_get_path_revprop(svn_string_t **propval,
                                              const dav_resource *resource,
                                              svn_revnum_t committed_rev,
                                              const char *propname,
@@ -170,14 +179,19 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
   svn_error_t *serr;
 
   /*
-  ** None of SVN provider properties are defined if the resource does not
-  ** exist. Just bail for this case.
+  ** Almost none of the SVN provider properties are defined if the
+  ** resource does not exist.  We do need to return the one VCC
+  ** property and baseline-relative-path on lock-null resources,
+  ** however, so that svn clients can run 'svn unlock' and 'svn info'
+  ** on these things.
   **
   ** Even though we state that the SVN properties are not defined, the
   ** client cannot store dead values -- we deny that thru the is_writable
   ** hook function.
   */
-  if (!resource->exists)
+  if ((! resource->exists)
+      && (propid != DAV_PROPID_version_controlled_configuration)
+      && (propid != SVN_PROPID_baseline_relative_path))
     return DAV_PROP_INSERT_NOTSUPP;
 
   /* ### we may want to respond to DAV_PROPID_resourcetype for PRIVATE
@@ -268,7 +282,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
             return DAV_PROP_INSERT_NOTSUPP;
           }
 
-        serr = svn_svn_get_path_revprop(&last_author,
+        serr = dav_svn_get_path_revprop(&last_author,
                                         resource,
                                         committed_rev,
                                         SVN_PROP_REVISION_AUTHOR,
@@ -320,6 +334,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
            svn:mime-type property is of type text/plain.  So it seems
            safe (and consistent) to assume the same on the server.  */
         svn_string_t *pval;
+        const char *mime_type = NULL;
 
         if (resource->baselined && resource->type == DAV_RESOURCE_TYPE_VERSION)
           return DAV_PROP_INSERT_NOTSUPP;
@@ -330,29 +345,32 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
             return DAV_PROP_INSERT_NOTSUPP;
           }
 
-        serr = svn_fs_node_prop (&pval, resource->info->root.root,
-                                 resource->info->repos_path,
-                                 SVN_PROP_MIME_TYPE, p);
-
-        if ((serr != NULL) || (pval == NULL))
+        if (resource->collection) /* defaults for directories */
           {
-            svn_error_clear(serr);
-            if (resource->collection) /* defaults for directories */
-              {
-                if (resource->info->repos->xslt_uri)
-                  value = "text/xml";
-                else
-                  value = "text/html";
-              }
+            if (resource->info->repos->xslt_uri)
+              mime_type = "text/xml";
             else
-              {
-                value = "text/plain"; /* default for file */
-              }
-          }            
+              mime_type = "text/html; charset=UTF-8";
+          }
         else
           {
-            serr = svn_mime_type_validate (pval->data, p);
-            if (serr)
+            if ((serr = svn_fs_node_prop (&pval, resource->info->root.root,
+                                          resource->info->repos_path,
+                                          SVN_PROP_MIME_TYPE, p)))
+              {
+                svn_error_clear(serr);
+                pval = NULL;
+              }
+
+            if (pval)
+              mime_type = pval->data;
+            else if ((! resource->info->repos->is_svn_client) 
+                     && resource->info->r->content_type)
+              mime_type = resource->info->r->content_type;
+            else
+              mime_type = "text/plain"; /* default for file */
+
+            if ((serr = svn_mime_type_validate (mime_type, p)))
               {
                 /* Probably serr->apr == SVN_ERR_BAD_MIME_TYPE, but
                    there's no point even checking.  No matter what the
@@ -361,10 +379,9 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
                 svn_error_clear(serr);
                 return DAV_PROP_INSERT_NOTDEF;
               }
-            else
-              value = pval->data;
           }
 
+        value = mime_type;
         break;
       }
 
@@ -683,8 +700,8 @@ void dav_svn_insert_all_liveprops(request_rec *r, const dav_resource *resource,
 
     for (spec = dav_svn_props; spec->name != NULL; ++spec)
       {
-        (void) dav_svn_insert_prop(resource, spec->propid, what, phdr);
         svn_pool_clear(subpool);
+        (void) dav_svn_insert_prop(resource, spec->propid, what, phdr);
       }
 
     resource->info->pool = pool;
@@ -738,7 +755,7 @@ int dav_svn_get_last_modified_time (const char **datestring,
       return 1;
     }
 
-  serr = svn_svn_get_path_revprop(&committed_date,
+  serr = dav_svn_get_path_revprop(&committed_date,
                                   resource,
                                   committed_rev,
                                   SVN_PROP_REVISION_DATE,
