@@ -23,9 +23,7 @@
 #include <apr_xml.h>
 #include <mod_dav.h>
 
-#include "svn_pools.h"
 #include "svn_repos.h"
-#include "svn_fs.h"
 #include "svn_types.h"
 #include "svn_xml.h"
 #include "svn_path.h"
@@ -49,37 +47,18 @@ struct log_receiver_baton
 };
 
 
-static svn_error_t * send_xml(struct log_receiver_baton *lrb, 
-                              const char *fmt, ...)
-{
-  apr_status_t apr_err;
-  va_list ap;
-
-  va_start(ap, fmt);
-  apr_err = apr_brigade_vprintf(lrb->bb, ap_filter_flush, 
-                                lrb->output, fmt, ap);
-  va_end(ap);
-  if (apr_err)
-    return svn_error_create(apr_err, 0, NULL);
-  /* ### check for an aborted connection, since the brigade functions
-     don't appear to be return useful errors when the connection is
-     dropped. */
-  if (lrb->output->c->aborted)
-    return svn_error_create(SVN_ERR_APMOD_CONNECTION_ABORTED, 0, NULL);
-  return SVN_NO_ERROR;
-}
-
-
 /* If LRB->needs_header is true, send the "<S:log-report>" start
-   element and set LRB->needs_header to zero.  Else do nothing. */
+   element and set LRB->needs_header to zero.  Else do nothing.
+   This is basically duplicated in file_revs.c.  Consider factoring if
+   duplicating again. */
 static svn_error_t * maybe_send_header(struct log_receiver_baton *lrb)
 {
   if (lrb->needs_header)
     {
-      SVN_ERR( send_xml(lrb,
-                        DAV_XML_HEADER DEBUG_CR
-                        "<S:log-report xmlns:S=\"" SVN_XML_NAMESPACE "\" "
-                        "xmlns:D=\"DAV:\">" DEBUG_CR) );
+      SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output,
+                                 DAV_XML_HEADER DEBUG_CR
+                                 "<S:log-report xmlns:S=\"" SVN_XML_NAMESPACE
+                                 "\" " "xmlns:D=\"DAV:\">" DEBUG_CR) );
       lrb->needs_header = FALSE;
     }
   return SVN_NO_ERROR;
@@ -99,22 +78,28 @@ static svn_error_t * log_receiver(void *baton,
 
   SVN_ERR( maybe_send_header(lrb) );
 
-  SVN_ERR( send_xml(lrb, "<S:log-item>" DEBUG_CR "<D:version-name>%ld"
-                    "</D:version-name>" DEBUG_CR, rev) );
+  SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output,
+                             "<S:log-item>" DEBUG_CR "<D:version-name>%ld"
+                             "</D:version-name>" DEBUG_CR, rev) );
 
   if (author)
-    SVN_ERR( send_xml(lrb, "<D:creator-displayname>%s</D:creator-displayname>" 
-                      DEBUG_CR, apr_xml_quote_string(pool, author, 0)) );
+    SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output,
+                               "<D:creator-displayname>%s"
+                               "</D:creator-displayname>" DEBUG_CR,
+                               apr_xml_quote_string(pool, author, 0)) );
 
   /* ### this should be DAV:creation-date, but we need to format
      ### that date a bit differently */
   if (date)
-    SVN_ERR( send_xml(lrb, "<S:date>%s</S:date>" DEBUG_CR,
-                      apr_xml_quote_string(pool, date, 0)) );
+    SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output,
+                               "<S:date>%s</S:date>" DEBUG_CR,
+                               apr_xml_quote_string(pool, date, 0)) );
 
   if (msg)
-    SVN_ERR( send_xml(lrb, "<D:comment>%s</D:comment>" DEBUG_CR,
-                      apr_xml_quote_string(pool, msg, 0)) );
+    SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output,
+                               "<D:comment>%s</D:comment>" DEBUG_CR,
+                               apr_xml_quote_string
+                               (pool, svn_xml_fuzzy_escape (msg, pool), 0)) );
 
 
   if (changed_paths)
@@ -139,51 +124,62 @@ static svn_error_t * log_receiver(void *baton,
             case 'A':
               if (log_item->copyfrom_path 
                   && SVN_IS_VALID_REVNUM(log_item->copyfrom_rev))
-                SVN_ERR( send_xml(lrb, 
-                                  "<S:added-path"
-                                  " copyfrom-path=\"%s\"" 
-                                  " copyfrom-rev=\"%ld\">"
-                                  "%s</S:added-path>" DEBUG_CR,
-                                  apr_xml_quote_string(pool, 
-                                                       log_item->copyfrom_path,
-                                                       1), /* escape quotes */
-                                  log_item->copyfrom_rev,
-                                  apr_xml_quote_string(pool, path, 0)) );
+                SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output, 
+                                           "<S:added-path"
+                                           " copyfrom-path=\"%s\"" 
+                                           " copyfrom-rev=\"%ld\">"
+                                           "%s</S:added-path>" DEBUG_CR,
+                                           apr_xml_quote_string
+                                           (pool, 
+                                            log_item->copyfrom_path,
+                                            1), /* escape quotes */
+                                           log_item->copyfrom_rev,
+                                           apr_xml_quote_string(pool,
+                                                                path, 0)) );
               else
-                SVN_ERR( send_xml(lrb, "<S:added-path>%s</S:added-path>" 
-                                  DEBUG_CR, 
-                                  apr_xml_quote_string(pool, path, 0)) );
+                SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output,
+                                           "<S:added-path>%s</S:added-path>" 
+                                           DEBUG_CR, 
+                                           apr_xml_quote_string(pool, path,
+                                                                0)) );
               break;
 
             case 'R':
               if (log_item->copyfrom_path 
                   && SVN_IS_VALID_REVNUM(log_item->copyfrom_rev))
-                SVN_ERR( send_xml(lrb, 
-                                  "<S:replaced-path"
-                                  " copyfrom-path=\"%s\"" 
-                                  " copyfrom-rev=\"%ld\">"
-                                  "%s</S:replaced-path>" DEBUG_CR,
-                                  apr_xml_quote_string(pool, 
-                                                       log_item->copyfrom_path,
-                                                       1), /* escape quotes */
-                                  log_item->copyfrom_rev,
-                                  apr_xml_quote_string(pool, path, 0)) );
+                SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output,
+                                           "<S:replaced-path"
+                                           " copyfrom-path=\"%s\"" 
+                                           " copyfrom-rev=\"%ld\">"
+                                           "%s</S:replaced-path>" DEBUG_CR,
+                                           apr_xml_quote_string
+                                           (pool, 
+                                            log_item->copyfrom_path,
+                                            1), /* escape quotes */
+                                           log_item->copyfrom_rev,
+                                           apr_xml_quote_string(pool,
+                                                                path, 0)) );
               else
-                SVN_ERR( send_xml(lrb, "<S:replaced-path>%s</S:replaced-path>" 
-                                  DEBUG_CR, 
-                                  apr_xml_quote_string(pool, path, 0)) );
+                SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output,
+                                           "<S:replaced-path>%s"
+                                           "</S:replaced-path>" DEBUG_CR,
+                                           apr_xml_quote_string(pool, path,
+                                                                0)) );
               break;
 
             case 'D':
-              SVN_ERR( send_xml(lrb, "<S:deleted-path>%s</S:deleted-path>" 
-                                DEBUG_CR,
-                                apr_xml_quote_string(pool, path, 0)) );
+              SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output,
+                                         "<S:deleted-path>%s</S:deleted-path>" 
+                                         DEBUG_CR,
+                                         apr_xml_quote_string(pool, path,
+                                                              0)) );
               break;
 
             case 'M':
-              SVN_ERR( send_xml(lrb, "<S:modified-path>%s</S:modified-path>" 
-                                DEBUG_CR,
-                                apr_xml_quote_string(pool, path, 0)) );
+              SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output,
+                                         "<S:modified-path>%s"
+                                         "</S:modified-path>" DEBUG_CR,
+                                         apr_xml_quote_string(pool, path, 0)) );
               break;
               
             default:
@@ -192,7 +188,7 @@ static svn_error_t * log_receiver(void *baton,
         }
     }
 
-  SVN_ERR( send_xml(lrb, "</S:log-item>" DEBUG_CR) );
+  SVN_ERR( dav_svn__send_xml(lrb->bb, lrb->output, "</S:log-item>" DEBUG_CR) );
 
   return SVN_NO_ERROR;
 }
@@ -212,6 +208,7 @@ dav_error * dav_svn__log_report(const dav_resource *resource,
   dav_svn_authz_read_baton arb;
   const dav_svn_repos *repos = resource->info->repos;
   const char *target = NULL;
+  int limit = 0;
   int ns;
 
   /* These get determined from the request document. */
@@ -246,37 +243,19 @@ dav_error * dav_svn__log_report(const dav_resource *resource,
         start = SVN_STR_TO_REV(dav_xml_get_cdata(child, resource->pool, 1));
       else if (strcmp(child->name, "end-revision") == 0)
         end = SVN_STR_TO_REV(dav_xml_get_cdata(child, resource->pool, 1));
+      else if (strcmp(child->name, "limit") == 0)
+        limit = atoi(child->first_cdata.first->text);
       else if (strcmp(child->name, "discover-changed-paths") == 0)
-        {
-          /* ### todo: value doesn't matter, presence alone is enough?
-             (I.e., is that a traditional way to do things here?) */
-          discover_changed_paths = 1;
-        }
+        discover_changed_paths = 1; /* presence indicates positivity */
       else if (strcmp(child->name, "strict-node-history") == 0)
-        {
-          /* ### todo: value doesn't matter, presence alone is enough?
-             (I.e., is that a traditional way to do things here?) */
-          strict_node_history = 1;
-        }
+        strict_node_history = 1; /* presence indicates positivity */
       else if (strcmp(child->name, "path") == 0)
         {
-          /* Convert these relative paths to absolute paths in the
-             repository. */
-          target = apr_pstrdup (resource->pool, resource->info->repos_path);
-
-          /* Don't add on an empty string, but do add the target to the
-             path.  This special case means that we have passed a single
-             directory to get the log of, and we need a path to call
-             svn_fs_revisions_changed on. */
-          if (child->first_cdata.first)
-            {
-              if ((derr = dav_svn__test_canonical
-                   (child->first_cdata.first->text, resource->pool)))
-                return derr;
-              target = svn_path_join(target, child->first_cdata.first->text,
-                                     resource->pool);
-            }
-
+          const char *rel_path = dav_xml_get_cdata(child, resource->pool, 0);
+          if ((derr = dav_svn__test_canonical (rel_path, resource->pool)))
+            return derr;
+          target = svn_path_join(resource->info->repos_path, rel_path, 
+                                 resource->pool);
           (*((const char **)(apr_array_push (paths)))) = target;
         }
       /* else unknown element; skip it */
@@ -298,10 +277,11 @@ dav_error * dav_svn__log_report(const dav_resource *resource,
      flag in our log_receiver_baton structure). */
 
   /* Send zero or more log items. */
-  serr = svn_repos_get_logs2(repos->repos,
+  serr = svn_repos_get_logs3(repos->repos,
                              paths,
                              start,
                              end,
+                             limit,
                              discover_changed_paths,
                              strict_node_history,
                              dav_svn_authz_read,
@@ -324,7 +304,8 @@ dav_error * dav_svn__log_report(const dav_resource *resource,
       goto cleanup;
     }
     
-  if ((serr = send_xml(&lrb, "</S:log-report>" DEBUG_CR)))
+  if ((serr = dav_svn__send_xml(lrb.bb, lrb.output, "</S:log-report>"
+                                DEBUG_CR)))
     {
       derr = dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
                                  "Error ending REPORT response.",
