@@ -26,7 +26,7 @@
 
 #include "svn_pools.h"
 #include "svn_fs.h"
-#include "svn_repos.h"
+#include "svn_props.h"
 
 #include "dav_svn.h"
 
