@@ -21,7 +21,6 @@
 #include <apr_general.h>
 #include <apr_pools.h>
 #include <apr_time.h>
-#include <apr_thread_proc.h>
 #include <apr_file_io.h>
 #include <apr_signal.h>
 
@@ -36,7 +35,6 @@
 #include "svn_path.h"
 #include "svn_repos.h"
 #include "svn_fs.h"
-#include "svn_repos.h"
 #include "svn_time.h"
 #include "svn_utf.h"
 #include "svn_subst.h"
@@ -62,6 +60,7 @@ static svn_opt_subcommand_t
   subcommand_help,
   subcommand_history,
   subcommand_info,
+  subcommand_lock,
   subcommand_log,
   subcommand_pget,
   subcommand_plist,
@@ -74,7 +73,9 @@ enum
   { 
     svnlook__version = SVN_OPT_FIRST_LONGOPT_ID,
     svnlook__show_ids,
-    svnlook__no_diff_deleted
+    svnlook__no_diff_deleted,
+    svnlook__no_diff_added,
+    svnlook__revprop_opt
   };
 
 /*
@@ -109,6 +110,12 @@ static const apr_getopt_option_t options_table[] =
     {"no-diff-deleted", svnlook__no_diff_deleted, 0,
      N_("do not print differences for deleted files")},
 
+    {"no-diff-added", svnlook__no_diff_added, 0,
+     N_("do not print differences for added files")},
+
+    {"revprop",       svnlook__revprop_opt, 0,
+                      N_("operate on a revision property (use with -r)")},
+
     {0,               0, 0, 0}
   };
 
@@ -142,7 +149,7 @@ static const svn_opt_subcommand_desc_t cmd_table[] =
     {"diff", subcommand_diff, {0},
      N_("usage: svnlook diff REPOS_PATH\n\n"
         "Print GNU-style diffs of changed files and properties.\n"),
-     {'r', 't', svnlook__no_diff_deleted} },
+     {'r', 't', svnlook__no_diff_deleted, svnlook__no_diff_added} },
 
     {"dirs-changed", subcommand_dirschanged, {0},
      N_("usage: svnlook dirs-changed REPOS_PATH\n\n"
@@ -166,21 +173,28 @@ static const svn_opt_subcommand_desc_t cmd_table[] =
         "Print the author, datestamp, log message size, and log message.\n"),
      {'r', 't'} },
 
+    {"lock", subcommand_lock, {0},
+     N_("usage: svnlook lock REPOS_PATH PATH_IN_REPOS\n\n"
+        "If a lock exists on a path in the repository, describe it.\n"),
+     {0} },
+
     {"log", subcommand_log, {0},
      N_("usage: svnlook log REPOS_PATH\n\n"
         "Print the log message.\n"),
      {'r', 't'} },
 
     {"propget", subcommand_pget, {"pget", "pg"},
-     N_("usage: svnlook propget REPOS_PATH PROPNAME PATH_IN_REPOS\n\n"
-        "Print the raw value of a property on a path in the repository.\n"),
-     {'r', 't'} },
+     N_("usage: svnlook propget REPOS_PATH PROPNAME [PATH_IN_REPOS]\n\n"
+        "Print the raw value of a property on a path in the repository.\n"
+        "With --revprop, prints the raw value of a revision property.\n"),
+     {'r', 't', svnlook__revprop_opt} },
 
     {"proplist", subcommand_plist, {"plist", "pl"},
-     N_("usage: svnlook proplist REPOS_PATH PATH_IN_REPOS\n\n"
-        "List the properties of a path in the repository.\n"
+     N_("usage: svnlook proplist REPOS_PATH [PATH_IN_REPOS]\n\n"
+        "List the properties of a path in the repository, or\n"
+        "with the --revprop option, revision properties.\n"
         "With -v, show the property values too.\n"),
-     {'r', 't', 'v'} },
+     {'r', 't', 'v', svnlook__revprop_opt} },
 
     {"tree", subcommand_tree, {0},
      N_("usage: svnlook tree REPOS_PATH [PATH_IN_REPOS]\n\n"
@@ -214,7 +228,9 @@ struct svnlook_opt_state
   svn_boolean_t show_ids;         /* --show-ids */
   svn_boolean_t help;             /* --help */
   svn_boolean_t no_diff_deleted;  /* --no-diff-deleted */
+  svn_boolean_t no_diff_added;    /* --no-diff-added */
   svn_boolean_t verbose;          /* --verbose */
+  svn_boolean_t revprop;          /* --revprop */
 };
 
 
@@ -225,6 +241,7 @@ typedef struct svnlook_ctxt_t
   svn_boolean_t is_revision;
   svn_boolean_t show_ids;
   svn_boolean_t no_diff_deleted;
+  svn_boolean_t no_diff_added;
   svn_revnum_t rev_id;
   svn_fs_txn_t *txn;
   const char *txn_name /* UTF-8! */;
@@ -250,7 +267,7 @@ static svn_error_t *
 check_cancel (void *baton)
 {
   if (cancelled)
-    return svn_error_create (SVN_ERR_CANCELLED, NULL, "Caught signal");
+    return svn_error_create (SVN_ERR_CANCELLED, NULL, _("Caught signal"));
   else
     return SVN_NO_ERROR;
 }
@@ -270,8 +287,8 @@ check_lib_versions (void)
       { NULL, NULL }
     };
 
-   SVN_VERSION_DEFINE (my_version);
-   return svn_ver_check_list (&my_version, checklist);
+  SVN_VERSION_DEFINE (my_version);
+  return svn_ver_check_list (&my_version, checklist);
 }
 
 
@@ -552,7 +569,7 @@ open_writable_binary_file (apr_file_t **fh,
       else if (kind != svn_node_dir)
         {
           return svn_error_createf (err->apr_err, err,
-                                    "Error creating dir '%s' (path exists)", 
+                                    _("Error creating dir '%s' (path exists)"),
                                     full_path);
         }
     }
@@ -788,6 +805,7 @@ print_diff_tree (svn_fs_root_t *root,
                  const char *path /* UTF-8! */,
                  const char *base_path /* UTF-8! */,
                  svn_boolean_t no_diff_deleted,
+                 svn_boolean_t no_diff_added,
                  const char *tmpdir,
                  apr_pool_t *pool)
 {
@@ -888,7 +906,8 @@ print_diff_tree (svn_fs_root_t *root,
                                         : _("Index")))),
                                      path));
 
-      if ((! no_diff_deleted) || (node->action != 'D'))
+      if (!(node->action == 'D' && no_diff_deleted)
+          && !(node->action == 'A' && no_diff_added))
         {
           svn_diff_t *diff;
 
@@ -967,6 +986,7 @@ print_diff_tree (svn_fs_root_t *root,
                             svn_path_join (path, node->name, subpool),
                             svn_path_join (base_path, node->name, subpool),
                             no_diff_deleted,
+                            no_diff_added,
                             tmpdir,
                             subpool));
   while (node->sibling)
@@ -977,6 +997,7 @@ print_diff_tree (svn_fs_root_t *root,
                                 svn_path_join (path, node->name, subpool),
                                 svn_path_join (base_path, node->name, subpool),
                                 no_diff_deleted,
+                                no_diff_added,
                                 tmpdir,
                                 subpool));
     }
@@ -1052,12 +1073,12 @@ print_tree (svn_fs_root_t *root,
       void *val;
       svn_fs_dirent_t *entry;
 
+      svn_pool_clear (subpool);
       apr_hash_this (hi, &key, &keylen, &val);
       entry = val;
       SVN_ERR (print_tree (root, svn_path_join (path, entry->name, pool),
                            entry->id, (entry->kind == svn_node_dir),
                            indentation + 1, show_ids, subpool));
-      svn_pool_clear (subpool);
     }
   svn_pool_destroy (subpool);
 
@@ -1328,7 +1349,8 @@ do_diff (svnlook_ctxt_t *c, apr_pool_t *pool)
       SVN_ERR (svn_fs_revision_root (&base_root, c->fs, base_rev_id, pool));
       SVN_ERR (create_unique_tmpdir (&tmpdir, pool));
       err = print_diff_tree (root, base_root, tree, "", "",
-                             c->no_diff_deleted, tmpdir, pool);
+                             c->no_diff_deleted, c->no_diff_added,
+                             tmpdir, pool);
       if (err)
         {
           svn_error_clear (svn_io_remove_dir (tmpdir, pool));
@@ -1417,7 +1439,8 @@ do_history (svnlook_ctxt_t *c,
 
 /* Print the value of property PROPNAME on PATH in the repository.
    Error with SVN_ERR_FS_NOT_FOUND if PATH does not exist, or with
-   SVN_ERR_PROPERTY_NOT_FOUND if no such property on PATH. */
+   SVN_ERR_PROPERTY_NOT_FOUND if no such property on PATH.
+   If PATH is NULL, operate on a revision property. */
 static svn_error_t *
 do_pget (svnlook_ctxt_t *c,
          const char *propname,
@@ -1431,13 +1454,26 @@ do_pget (svnlook_ctxt_t *c,
   apr_size_t len;
   
   SVN_ERR (get_root (&root, c, pool));
-  SVN_ERR (verify_path (&kind, root, path, pool));
-  SVN_ERR (svn_fs_node_prop (&prop, root, path, propname, pool));
+  if (path != NULL)
+    {
+      SVN_ERR (verify_path (&kind, root, path, pool));
+      SVN_ERR (svn_fs_node_prop (&prop, root, path, propname, pool));
+    }
+  else
+      SVN_ERR (svn_fs_revision_prop (&prop, c->fs, c->rev_id, propname, pool));
 
   if (prop == NULL)
-    return svn_error_createf
-      (SVN_ERR_PROPERTY_NOT_FOUND, NULL,
-       _("Property '%s' not found on path '%s'"), propname, path);
+    {
+       if (path == NULL)
+         return svn_error_createf
+           (SVN_ERR_PROPERTY_NOT_FOUND, NULL,
+            _("Property '%s' not found on revision %ld"), propname, c->rev_id);
+       else
+         return svn_error_createf
+           (SVN_ERR_PROPERTY_NOT_FOUND, NULL,
+            _("Property '%s' not found on path '%s' in revision %ld"),
+            propname, path, c->rev_id);
+    }
 
   /* Else. */
 
@@ -1457,7 +1493,8 @@ do_pget (svnlook_ctxt_t *c,
 /* Print the property names of all properties on PATH in the repository.
    If VERBOSE, print their values too.
    Error with SVN_ERR_FS_NOT_FOUND if PATH does not exist, or with
-   SVN_ERR_PROPERTY_NOT_FOUND if no such property on PATH. */
+   SVN_ERR_PROPERTY_NOT_FOUND if no such property on PATH.
+   If PATH is NULL, operate on a revision properties. */
 static svn_error_t *
 do_plist (svnlook_ctxt_t *c,
           const char *path,
@@ -1470,10 +1507,15 @@ do_plist (svnlook_ctxt_t *c,
   apr_hash_index_t *hi;
   svn_node_kind_t kind;
 
-  SVN_ERR (get_root (&root, c, pool));
-  SVN_ERR (verify_path (&kind, root, path, pool));
-  SVN_ERR (svn_fs_node_proplist (&props, root, path, pool));
   SVN_ERR (svn_stream_for_stdout (&stdout_stream, pool));
+  if (path != NULL)
+    {
+      SVN_ERR (get_root (&root, c, pool));
+      SVN_ERR (verify_path (&kind, root, path, pool));
+      SVN_ERR (svn_fs_node_proplist (&props, root, path, pool));
+    }
+  else
+      SVN_ERR (svn_fs_revision_proplist (&props, c->fs, c->rev_id, pool));
 
   for (hi = apr_hash_first (pool, props); hi; hi = apr_hash_next (hi))
     {
@@ -1537,7 +1579,7 @@ warning_func (void *baton,
 {
   if (! err)
     return;
-  svn_handle_error (err, stderr, FALSE);
+  svn_handle_error2 (err, stderr, FALSE, "svnlook: ");
 }
 
 
@@ -1554,6 +1596,7 @@ get_ctxt_baton (svnlook_ctxt_t **baton_p,
   svn_fs_set_warning_func (baton->fs, warning_func, NULL);
   baton->show_ids = opt_state->show_ids;
   baton->no_diff_deleted = opt_state->no_diff_deleted;
+  baton->no_diff_added = opt_state->no_diff_added;
   baton->is_revision = opt_state->txn ? FALSE : TRUE;
   baton->rev_id = opt_state->rev;
   baton->txn_name = apr_pstrdup (pool, opt_state->txn);
@@ -1658,13 +1701,21 @@ subcommand_help (apr_getopt_t *os, void *baton, apr_pool_t *pool)
       "Note: any subcommand which takes the '--revision' and '--transaction'\n"
       "      options will, if invoked without one of those options, act on\n"
       "      the repository's youngest revision.\n"
-      "Type \"svnlook help <subcommand>\" for help on a specific subcommand.\n"
+      "Type 'svnlook help <subcommand>' for help on a specific subcommand.\n"
       "\n"
       "Available subcommands:\n");
 
+  const char *fs_desc_start
+    = _("The following repository back-end (FS) modules are available:\n\n");
+
+  svn_stringbuf_t *version_footer;
+
+  version_footer = svn_stringbuf_create (fs_desc_start, pool);
+  SVN_ERR (svn_fs_print_modules (version_footer, pool));
+
   SVN_ERR (svn_opt_print_help (os, "svnlook", 
                                opt_state ? opt_state->version : FALSE, 
-                               FALSE, NULL,
+                               FALSE, version_footer->data,
                                header, cmd_table, options_table, NULL,
                                pool));
   
@@ -1687,6 +1738,55 @@ subcommand_history (apr_getopt_t *os, void *baton, apr_pool_t *pool)
   return SVN_NO_ERROR;
 }
 
+
+/* This implements `svn_opt_subcommand_t'. */
+static svn_error_t *
+subcommand_lock (apr_getopt_t *os, void *baton, apr_pool_t *pool)
+{
+  struct svnlook_opt_state *opt_state = baton;
+  svnlook_ctxt_t *c;
+  const char *path;
+  svn_lock_t *lock;
+
+  if (opt_state->arg1)
+    path = opt_state->arg1;
+  else
+    return svn_error_create (SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
+                             _("Missing path argument"));
+
+  SVN_ERR (get_ctxt_baton (&c, opt_state, pool));
+
+  SVN_ERR (svn_fs_get_lock (&lock, c->fs, path, pool));
+
+  if (lock)
+    {
+      const char *cr_date, *exp_date = "";
+      int comment_lines = 0;
+
+      cr_date = svn_time_to_human_cstring (lock->creation_date, pool);
+
+      if (lock->expiration_date)
+        exp_date = svn_time_to_human_cstring (lock->expiration_date, pool);
+
+      if (lock->comment)
+        comment_lines = svn_cstring_count_newlines (lock->comment) + 1;
+
+      SVN_ERR (svn_cmdline_printf (pool, _("UUID Token: %s\n"), lock->token));
+      SVN_ERR (svn_cmdline_printf (pool, _("Owner: %s\n"), lock->owner));
+      SVN_ERR (svn_cmdline_printf (pool, _("Created: %s\n"), cr_date));
+      SVN_ERR (svn_cmdline_printf (pool, _("Expires: %s\n"), exp_date));
+      SVN_ERR (svn_cmdline_printf (pool,
+                                   (comment_lines != 1)
+                                   ? _("Comment (%i lines):\n%s\n")
+                                   : _("Comment (%i line):\n%s\n"),
+                                   comment_lines, 
+                                   lock->comment ? lock->comment : ""));
+    }
+
+  return SVN_NO_ERROR;
+}
+
+
 /* This implements `svn_opt_subcommand_t'. */
 static svn_error_t *
 subcommand_info (apr_getopt_t *os, void *baton, apr_pool_t *pool)
@@ -1724,17 +1824,19 @@ subcommand_pget (apr_getopt_t *os, void *baton, apr_pool_t *pool)
     {
       return svn_error_createf
         (SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
+         opt_state->revprop ?  _("Missing propname argument") :
          _("Missing propname and repository path arguments"));
     }
-  else if (opt_state->arg2 == NULL)
+  else if (!opt_state->revprop && opt_state->arg2 == NULL)
     {
-      return svn_error_createf
+      return svn_error_create
         (SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
          _("Missing propname or repository path argument"));
     }
 
   SVN_ERR (get_ctxt_baton (&c, opt_state, pool));
-  SVN_ERR (do_pget (c, opt_state->arg1, opt_state->arg2, pool));
+  SVN_ERR (do_pget (c, opt_state->arg1,
+                    opt_state->revprop ? NULL : opt_state->arg2, pool));
   return SVN_NO_ERROR;
 }
 
@@ -1744,14 +1846,15 @@ subcommand_plist (apr_getopt_t *os, void *baton, apr_pool_t *pool)
 {
   struct svnlook_opt_state *opt_state = baton;
   svnlook_ctxt_t *c;
-
-  if (opt_state->arg1 == NULL)
-    return svn_error_createf
+  
+  if (!opt_state->revprop && opt_state->arg1 == NULL)
+    return svn_error_create
       (SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
        _("Missing repository path argument"));
 
   SVN_ERR (get_ctxt_baton (&c, opt_state, pool));
-  SVN_ERR (do_plist (c, opt_state->arg1, opt_state->verbose, pool));
+  SVN_ERR (do_plist (c, opt_state->revprop ? NULL : opt_state->arg1, 
+                     opt_state->verbose, pool));
   return SVN_NO_ERROR;
 }
 
@@ -1832,7 +1935,7 @@ main (int argc, const char * const *argv)
   err = check_lib_versions ();
   if (err)
     {
-      svn_handle_error (err, stderr, FALSE);
+      svn_handle_error2 (err, stderr, FALSE, "svnlook: ");
       svn_error_clear (err);
       svn_pool_destroy (pool);
       return EXIT_FAILURE;
@@ -1898,6 +2001,10 @@ main (int argc, const char * const *argv)
         case '?':
           opt_state.help = TRUE;
           break;
+          
+        case svnlook__revprop_opt:
+          opt_state.revprop = TRUE;
+          break;
 
         case svnlook__version:
           opt_state.version = TRUE;
@@ -1912,6 +2019,10 @@ main (int argc, const char * const *argv)
           opt_state.no_diff_deleted = TRUE;
           break;
 
+        case svnlook__no_diff_added:
+          opt_state.no_diff_added = TRUE;
+          break;
+
         default:
           subcommand_help (NULL, NULL, pool);
           svn_pool_destroy (pool);
@@ -1942,7 +2053,7 @@ main (int argc, const char * const *argv)
         {
           svn_error_clear
             (svn_cmdline_fprintf (stderr, pool,
-                                  _("subcommand argument required\n")));
+                                  _("Subcommand argument required\n")));
           subcommand_help (NULL, NULL, pool);
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
@@ -1958,7 +2069,7 @@ main (int argc, const char * const *argv)
                                              pool);
               if (err)
                 {
-                  svn_handle_error (err, stderr, FALSE);
+                  svn_handle_error2 (err, stderr, FALSE, "svnlook: ");
                   svn_pool_destroy (pool);
                   svn_error_clear (err);
                   return EXIT_FAILURE;
@@ -1998,7 +2109,7 @@ main (int argc, const char * const *argv)
         {
           svn_error_clear
             (svn_cmdline_fprintf (stderr, pool,
-                                  _("repository argument required\n")));
+                                  _("Repository argument required\n")));
           subcommand_help (NULL, NULL, pool);
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
@@ -2055,7 +2166,7 @@ main (int argc, const char * const *argv)
           svn_error_clear
             (svn_cmdline_fprintf
              (stderr, pool,
-              _("subcommand '%s' doesn't accept option '%s'\n"
+              _("Subcommand '%s' doesn't accept option '%s'\n"
                 "Type 'svnlook help %s' for usage.\n"),
               subcommand->name, optstr, subcommand->name));
           svn_pool_destroy (pool);
@@ -2087,12 +2198,12 @@ main (int argc, const char * const *argv)
     {
       if (err->apr_err == SVN_ERR_CL_ARG_PARSING_ERROR)
         {
-          svn_handle_error (err, stderr, 0);
+          svn_handle_error2 (err, stderr, FALSE, "svnlook: ");
           svn_opt_subcommand_help (subcommand->name, cmd_table,
                                    options_table, pool);
         }
       else
-        svn_handle_error (err, stderr, 0);
+        svn_handle_error2 (err, stderr, FALSE, "svnlook: ");
       svn_pool_destroy (pool);
       return EXIT_FAILURE;
     }
