@@ -6,7 +6,7 @@
 #  See http://subversion.tigris.org for more information.
 #    
 # ====================================================================
-# Copyright (c) 2001 Sam Tobin-Hochstadt.  All rights reserved.
+# Copyright (c) 2001 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -104,8 +104,10 @@ class SVNTypeMismatch(SVNTreeError):
 # depending on the parsing context:
 
 #   - in the 'svn co/up' use-case, each line of output starts with two
-#     characters from the set of (A, D, G, U, C, _).  This status code
-#     is stored in a attribute named 'status'.
+#     characters from the set of (A, D, G, U, C, _) or 'Restored'.  The
+#     status code is stored in a attribute named 'status'.  In the case
+#     of a restored file, the word 'Restored' is stored in an attribute
+#     named 'verb'.
 
 #   - in the 'svn ci/im' use-case, each line of output starts with one
 #      of the words (Adding, Deleting, Sending).  This verb is stored in
@@ -114,9 +116,9 @@ class SVNTypeMismatch(SVNTreeError):
 #   - in the 'svn status' use-case (which is always run with the -v
 #     (--verbose) flag), each line of output contains a working revision
 #     number and a two-letter status code similar to the 'svn co/up'
-#     case.  The repository revision is also printed.  All of this
-#     information is stored in attributes named 'wc_rev', 'status', and
-#     'repos_rev', respectively.
+#     case.  This information is stored in attributes named 'wc_rev'
+#     and 'status'.  The repository revision is also printed, but it
+#     is ignored.
 
 #   - in the working-copy use-case, the att-hash is ignored.
 
@@ -191,6 +193,8 @@ class SVNTreeNode:
     ### self.children is None (file) and self.children == [] (empty
     ### diretory), but it seems that most places that construct
     ### SVNTreeNode objects don't even try to do that.  --xbc
+    ###
+    ### See issue #1611 about this problem.  -kfogel
     if self.children is not None:
       print "    Children:  ", len(self.children)
     else:
@@ -259,7 +263,7 @@ def create_from_path(path, contents=None, props={}, atts={}):
   # get a list of all the names in the path
   # each of these will be a child of the former
   if os.sep != "/":
-    path = path.replace(os.sep, "/")
+    path = string.replace(path, os.sep, "/")
   elements = path.split("/")
   if len(elements) == 0:
     ### we should raise a less generic error here. which?
@@ -294,13 +298,24 @@ def get_props(path):
   props = {}
   output, errput = main.run_svn(1, "proplist", path, "--verbose")
 
+  first_value = 0
   for line in output:
     if line.startswith('Properties on '):
       continue
-    name, value = line.split(' : ')
-    name = string.strip(name)
-    value = string.strip(value)
-    props[name] = value
+    # Not a misprint; "> 0" really is preferable to ">= 0" in this case.
+    if line.find(' : ') > 0:
+      name, value = line.split(' : ')
+      name = string.strip(name)
+      value = string.strip(value)
+      props[name] = value
+      first_value = 1
+    else:    # Multi-line property, so re-use the current name.
+      if first_value:
+        # Undo, as best we can, the strip(value) that was done before
+        # we knew this was a multiline property.
+        props[name] = props[name] + "\n"
+        first_value = 0
+      props[name] = props[name] + line
 
   return props
 
@@ -400,7 +415,7 @@ def compare_trees(a, b,
 
   If A and B are both files, then return if their contents,
   properties, and names are all the same; else raise a SVNTreeUnequal.
-  If A is a file and B is a directory, raise a SVNTypeMismatch; same
+  If A is a file and B is a directory, raise a SVNTreeUnequal; same
   vice-versa.  If both are directories, then for each entry that
   exists in both, call compare_trees on the two entries; otherwise, if
   the entry exists only in A, invoke FUNC_A on it, and likewise for
@@ -497,8 +512,8 @@ def dump_tree(n,indent=""):
   else:
     print "%s%s" % (indent, n.name)
 
-  indent = indent.replace("-"," ")
-  indent = indent.replace("+"," ")
+  indent = string.replace(indent, "-", " ")
+  indent = string.replace(indent, "+", " ")
   for i in range(len(tmp_children)):
     c = tmp_children[i]
     if i == len(tmp_children
@@ -544,20 +559,31 @@ def build_generic_tree(nodelist):
 
 # Parse co/up output into a tree.
 #
-#   Tree nodes will contain no contents, and only one 'status' att.
+#   Tree nodes will contain no contents, a 'status' att, and a
+#   'writelocked' att.
 
 def build_tree_from_checkout(lines):
   "Return a tree derived by parsing the output LINES from 'co' or 'up'."
   
   root = SVNTreeNode(root_node_name)
-  rm = re.compile ('^([MAGCUD_ ][MAGCUD_ ]) (.+)')
+  rm1 = re.compile ('^([MAGCUD_ ][MAGCUD_ ])([B ])\s+(.+)')
+  # There may be other verbs we need to match, in addition to
+  # "Restored".  If so, add them as alternatives in the first match
+  # group below.
+  rm2 = re.compile ('^(Restored)\s+\'(.+)\'')
   
   for line in lines:
-    match = rm.search(line)
+    match = rm1.search(line)
     if match and match.groups():
-      new_branch = create_from_path(match.group(2), None, {},
+      new_branch = create_from_path(match.group(3), None, {},
                                     {'status' : match.group(1)})
       root.add_child(new_branch)
+    else:
+      match = rm2.search(line)
+      if match and match.groups():
+        new_branch = create_from_path(match.group(2), None, {},
+                                      {'verb' : match.group(1)})
+        root.add_child(new_branch)
 
   return root
 
@@ -590,44 +616,99 @@ def build_tree_from_commit(lines):
 #
 #   Tree nodes will contain no contents, and these atts:
 #
-#          'status', 'wc_rev', 'repos_rev'
-#             ... and possibly 'locked', 'copied', IFF columns non-empty.
+#          'status', 'wc_rev',
+#             ... and possibly 'locked', 'copied', 'writelocked',
+#             IFF columns non-empty.
 # 
 
 def build_tree_from_status(lines):
-  "Return a tree derived by parsing the output LINES from 'st'."
+  "Return a tree derived by parsing the output LINES from 'st -vuq'."
 
   root = SVNTreeNode(root_node_name)
-  rm = re.compile ('^.+\:.+(\d+)')
-  lastline = string.strip(lines.pop())
-  match = rm.search(lastline)
-  if match and match.groups():
-    repos_rev = match.group(1)
-  else:
-    repos_rev = '?'
     
+  # 'status -v' output looks like this:
+  #
+  #      "%c%c%c%c%c%c %c   %6s   %6s %-12s %s\n"
+  #
+  # (Taken from 'print_status' in subversion/clients/cmdline/status.c.)
+  #
+  # Here are the parameters.  The middle number in parens is the
+  # match.group(), followed by a brief description of the field:
+  #
+  #    - text status           (1)  (single letter)
+  #    - prop status           (1)  (single letter)
+  #    - wc-lockedness flag    (2)  (single letter: "L" or " ")
+  #    - copied flag           (3)  (single letter: "+" or " ")
+  #    - switched flag         (4)  (single letter: "S" or " ")
+  #    - repos lock status     (5)  (single letter: "K", "O", "B", "T", " ")
+  #
+  #    [one space]
+  #
+  #    - out-of-date flag      (6)  (single letter: "*" or " ")
+  #
+  #    [three spaces]
+  #
+  #    - working revision      (7)  (either digits or "-")
+  #
+  #    [one space]
+  #
+  #    - last-changed revision (8)  (either digits or "?")
+  #
+  #    [one space]
+  #
+  #    - last author           (9)  (string of non-whitespace characters)
+  #
+  #    [one space]
+  #
+  #    - path                 (10)  (string of characters until newline)
+
   # Try http://www.wordsmith.org/anagram/anagram.cgi?anagram=ACDRMGU
-  rm = re.compile ('^([MACDRUG_ ][MACDRUG_ ])(.)(.)(.)  .   [^0-9-]+(\d+|-)( +\S+ +\S+ +)(.+)')
+  rm = re.compile('^([!MACDRUG_ ][MACDRUG_ ])([L ])([+ ])([S ])([KOBT ]) ([* ])   [^0-9-]*(\d+|-|\?) +(\d|-|\?)+ +(\S+) +(.+)')
   for line in lines:
+
+    # Quit when we hit an externals status announcement (### someday we can fix
+    # the externals tests to expect the additional flood of externals status
+    # data).
+    if re.match(r'^Performing', line):
+      break
+    
     match = rm.search(line)
     if match and match.groups():
-      if match.group(6) != '-': # ignore items that only exist on repos
+      if match.group(9) != '-': # ignore items that only exist on repos
         atthash = {'status' : match.group(1),
-                   'wc_rev' : match.group(5),
-                   'repos_rev' : repos_rev}
+                   'wc_rev' : match.group(7)}
         if match.group(2) != ' ':
           atthash['locked'] = match.group(2)
         if match.group(3) != ' ':
           atthash['copied'] = match.group(3)
         if match.group(4) != ' ':
           atthash['switched'] = match.group(4)
-        new_branch = create_from_path(match.group(7), None, {}, atthash)
+        if match.group(5) != ' ':
+          atthash['writelocked'] = match.group(5)
+        new_branch = create_from_path(match.group(10), None, {}, atthash)
 
       root.add_child(new_branch)
 
   return root
 
 
+# Parse merge "skipped" output
+
+def build_tree_from_skipped(lines):
+
+  root = SVNTreeNode(root_node_name)
+  ### Will get confused by spaces in the filename
+  rm = re.compile ("^Skipped.* '([^ ]+)'\n")
+
+  for line in lines:
+    match = rm.search(line)
+    if match and match.groups():
+      new_branch = create_from_path(match.group(1))
+      root.add_child(new_branch)
+
+  return root
+
+
 ####################################################################
 # Build trees by looking at the working copy
 
